package com.example.heresdkandroidref.activity;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.appcompat.app.AppCompatActivity;
import androidx.appcompat.widget.Toolbar;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import android.content.Intent;
import android.os.Bundle;
import android.util.Log;
import android.view.View;
import android.widget.TextView;

import com.example.heresdkandroidref.R;
import com.example.heresdkandroidref.adapters.InstalledRegionsAdapter;
import com.example.heresdkandroidref.adapters.RegionsMapAdapter;
import com.example.heresdkandroidref.events.GetRegionsEvent;
import com.example.heresdkandroidref.features.OfflineMapsExample;
import com.example.heresdkandroidref.features.OfflineMapsExampleOld;
import com.example.heresdkandroidref.interfaces.RegionItemClickListener;
import com.example.heresdkandroidref.utils.LoadingDialog;
import com.example.heresdkandroidref.utils.PermissionsRequestor;
import com.google.gson.Gson;
import com.here.sdk.maploader.InstalledRegion;
import com.here.sdk.maploader.InstalledRegionStatus;
import com.here.sdk.maploader.Region;
import com.here.sdk.maploader.RegionId;
import com.here.sdk.mapview.MapScheme;
import com.here.sdk.mapview.MapView;

import org.greenrobot.eventbus.EventBus;
import org.greenrobot.eventbus.Subscribe;
import org.greenrobot.eventbus.ThreadMode;

import java.util.ArrayList;
import java.util.List;

public class DownloadMapsActivity extends AppCompatActivity implements RegionItemClickListener {
    private static final String TAG = DownloadMapsActivity.class.getSimpleName();
    private static OfflineMapsExample offlineMapsExample;
    private PermissionsRequestor permissionsRequestor;
    private MapView mapView;
    private LoadingDialog dialog;
    RegionsMapAdapter regionsMapAdapter;
    InstalledRegionsAdapter installedRegionsAdapter;
    RecyclerView regionsRv;
    RecyclerView installedRegionsRv;
    TextView downloadedMapsTV;
    ArrayList<Region> regionArrayList = new ArrayList<>();
    List<Region> installedRegionsList = new ArrayList<>();

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_download_maps);
        initViews(savedInstanceState);
        handleAndroidPermissions();
    }

    private void initViews(Bundle savedInstanceState) {
        Toolbar toolbar = findViewById(R.id.toolbar);
        toolbar.setTitle("Download Maps");
        setSupportActionBar(toolbar);

        getSupportActionBar().setDisplayHomeAsUpEnabled(true);
        getSupportActionBar().setDisplayShowHomeEnabled(true);
//        getSupportActionBar().setTitle("Download Maps");

        // Get a MapView instance from layout.
        mapView = findViewById(R.id.map_view);
        mapView.onCreate(savedInstanceState);
        regionsRv = findViewById(R.id.region_rv);
        installedRegionsRv = findViewById(R.id.installed_region_rv);
        downloadedMapsTV = findViewById(R.id.downloadedMapsTV);
        this.dialog = new LoadingDialog(this);
    }

    @Override
    public boolean onSupportNavigateUp() {
        onBackPressed();
        return true;
    }

    private void handleAndroidPermissions() {
        permissionsRequestor = new PermissionsRequestor(this);
        permissionsRequestor.request(new PermissionsRequestor.ResultListener() {

            @Override
            public void permissionsGranted() {
                loadMapScene();
            }

            @Override
            public void permissionsDenied() {
                Log.e(TAG, "Permissions denied by user.");
            }
        });
    }


    @Override
    public void onRequestPermissionsResult(int requestCode, @NonNull String[] permissions, @NonNull int[] grantResults) {
        super.onRequestPermissionsResult(requestCode, permissions, grantResults);
        permissionsRequestor.onRequestPermissionsResult(requestCode, grantResults);
    }

    private void loadMapScene() {
        mapView.getMapScene().loadScene(MapScheme.NORMAL_DAY, mapError -> {
            if (mapError == null) {
                LoadingDialog.showDialog(dialog);
                offlineMapsExample = new OfflineMapsExample(mapView);
            } else {
                Log.d(TAG, "Loading map failed: mapErrorCode: " + mapError.name());
            }
        });
    }

    public Region _findInstalledRegionByID(List<Region> regions, RegionId regionId) {
        for (Region region : regions) {
            if (region.regionId.id == regionId.id) {
                return region;
            }
            if (region.childRegions != null) {
                Region foundRegion = _findInstalledRegionByID(region.childRegions, regionId);
                if (foundRegion != null) {
                    return foundRegion;
                }
            }
        }

        return null;
    }


    @Override
    protected void onPause() {
        super.onPause();
        mapView.onPause();
    }

    @Override
    protected void onResume() {
        super.onResume();
        mapView.onResume();
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        mapView.onDestroy();
    }


    @Override
    protected void onStart() {
        super.onStart();
        EventBus.getDefault().register(this);

    }

    @Override
    protected void onStop() {
        EventBus.getDefault().unregister(this);
        super.onStop();
    }


    @Subscribe(threadMode = ThreadMode.MAIN)
    public void onGetRegionEvent(GetRegionsEvent event) {
        if (event != null) {
            LoadingDialog.dismissDialog(this.dialog);
            regionArrayList.clear();
            regionArrayList.addAll(event.getRegionsList());
            //init recycler views
            if (regionsMapAdapter == null) {
                regionsRv.setLayoutManager(new LinearLayoutManager(this, LinearLayoutManager.VERTICAL, false));
                regionsRv.setHasFixedSize(true);
                regionsMapAdapter = new RegionsMapAdapter(this, regionArrayList, null, offlineMapsExample);
                regionsMapAdapter.setItemClickListener(this);
                regionsRv.setAdapter(regionsMapAdapter);
            } else {
                regionsMapAdapter.customUpdateList(regionArrayList);
            }
            initInstalledMapsList(event.getInstalledRegionsList());
        } else {
            LoadingDialog.dismissDialog(this.dialog);
        }
    }

    private void initInstalledMapsList(List<InstalledRegion> newInstalledRegionsList) {
        installedRegionsList.clear();
        List<InstalledRegion> filteredInstalledRegions = new ArrayList<>();
        for (InstalledRegion installedRegion : newInstalledRegionsList) {
            if (installedRegion.status == InstalledRegionStatus.INSTALLED) {
                filteredInstalledRegions.add(installedRegion);
            }
        }
        if (!filteredInstalledRegions.isEmpty()) {
            for (InstalledRegion installedRegion : filteredInstalledRegions) {
                Region region = _findInstalledRegionByID(regionArrayList, installedRegion.regionId);
                Region newRegion = new Region(region.regionId);
                //newRegion.parentId = installedRegion.parentId;
                newRegion.name = region.name;
                newRegion.sizeOnDiskInBytes = region.sizeOnDiskInBytes;
                newRegion.sizeOnNetworkInBytes = region.sizeOnNetworkInBytes;
                installedRegionsList.add(newRegion);
            }
            if (installedRegionsAdapter == null) {
                //installed map list
                installedRegionsAdapter = new InstalledRegionsAdapter(DownloadMapsActivity.this, installedRegionsList, null, offlineMapsExample);
                installedRegionsRv.setLayoutManager(new LinearLayoutManager(DownloadMapsActivity.this, LinearLayoutManager.VERTICAL, false));
                installedRegionsRv.setHasFixedSize(true);
                installedRegionsRv.setAdapter(installedRegionsAdapter);
            } else {
                installedRegionsAdapter.customUpdateList(installedRegionsList);
            }
            installedRegionsRv.setVisibility(View.VISIBLE);
            downloadedMapsTV.setVisibility(View.VISIBLE);
        } else {
            installedRegionsRv.setVisibility(View.GONE);
            downloadedMapsTV.setVisibility(View.GONE);
        }
    }

    public void onDownloadListClicked(View view) {
//        offlineMapsExample.onDownloadListClicked();
    }

    public void onDownloadMapClicked(View view) {
//        offlineMapsExample.onDownloadMapClicked("");
    }

    public void onCancelMapDownloadClicked(View view) {
//        offlineMapsExample.onCancelMapDownloadClicked();
    }

    public void onSearchPlaceClicked(View view) {
//        offlineMapsExample.onSearchPlaceClicked();
    }


    @Override
    public void onRegionNavigateClick(View view, int position) {
        Gson gson = new Gson();
        String region = gson.toJson(regionArrayList.get(position));
        Intent intent = new Intent(DownloadMapsActivity.this, DownloadRegionMapsActivity.class);
        intent.putExtra("region", region);
        startActivityForResult(intent, 123);
    }

    @Override
    protected void onActivityResult(int requestCode, int resultCode, @Nullable Intent data) {
        super.onActivityResult(requestCode, resultCode, data);
        if (requestCode == 123) {
//            if (offlineMapsExample != null && installedRegionsList.size() != offlineMapsExample.getInstalledRegionsList().size()) {
            if (offlineMapsExample != null) {
//                offlineMapsExample.onDownloadListClicked();
                initInstalledMapsList(offlineMapsExample.getInstalledRegionsList());
            }
        }
    }

    @Override
    public void onRegionDownloadClick(View view, int position) {
//        offlineMapsExample.onDownloadMapClicked(regionArrayList.get(position).name);
    }

    @Override
    public void onRegionDownloadCancelClick(View view, int position) {

    }

    @Override
    public void onRegionDeleteClick(View view, int position) {

    }

    public static OfflineMapsExample getOfflineMapsExample() {
        return offlineMapsExample;
    }

    public void setOfflineMapsExample(OfflineMapsExample offlineMapsExample) {
        DownloadMapsActivity.offlineMapsExample = offlineMapsExample;
    }
}