package com.example.heresdkandroidref.activity;

import android.content.Intent;
import android.os.Bundle;
import android.util.Log;
import android.view.View;

import androidx.annotation.Nullable;
import androidx.appcompat.app.AppCompatActivity;
import androidx.appcompat.widget.Toolbar;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import com.example.heresdkandroidref.R;
import com.example.heresdkandroidref.adapters.RegionsMapAdapter;
import com.example.heresdkandroidref.features.OfflineMapsExample;
import com.example.heresdkandroidref.features.OfflineMapsExampleOld;
import com.example.heresdkandroidref.interfaces.RegionItemClickListener;
import com.example.heresdkandroidref.utils.LoadingDialog;
import com.google.gson.Gson;
import com.here.sdk.maploader.Region;
import com.here.sdk.mapview.MapError;
import com.here.sdk.mapview.MapScene;
import com.here.sdk.mapview.MapScheme;
import com.here.sdk.mapview.MapView;

import java.util.ArrayList;

public class DownloadRegionMapsActivity extends AppCompatActivity implements RegionItemClickListener {
    private static final String TAG = DownloadRegionMapsActivity.class.getSimpleName();
    private OfflineMapsExample offlineMapsExample;
    private MapView mapView;
    private LoadingDialog dialog;
    RegionsMapAdapter regionsMapAdapter;
    RecyclerView regionMapsRv;
    Region region;
    ArrayList<Region> regionArrayList= new ArrayList<>();
    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_download_region_maps);

        initViews(savedInstanceState);
        loadMapScene();
    }

    private void initViews(Bundle savedInstanceState){
        Toolbar toolbar =  findViewById(R.id.toolbar);
        toolbar.setTitle("Download Maps");
        setSupportActionBar(toolbar);

        getSupportActionBar().setDisplayHomeAsUpEnabled(true);
        getSupportActionBar().setDisplayShowHomeEnabled(true);

        // Get a MapView instance from layout.
        mapView = findViewById(R.id.map_view);
        mapView.onCreate(savedInstanceState);
        regionMapsRv =  findViewById(R.id.region_map_rv);
        this.dialog = new LoadingDialog(this);

        Gson gson = new Gson();
        String regionData = getIntent().getStringExtra("region");
        region = gson.fromJson(regionData, Region.class);
        regionArrayList.addAll(region.childRegions != null ? region.childRegions : new ArrayList<>());

        LoadingDialog.showDialog(dialog);
    }

    @Override
    public boolean onSupportNavigateUp() {
        onBackPressed();
        return true;
    }


    private void loadMapScene() {
        mapView.getMapScene().loadScene(MapScheme.NORMAL_DAY, new MapScene.LoadSceneCallback() {
            @Override
            public void onLoadScene(@Nullable MapError mapError) {
                if (mapError == null) {

//                    offlineMapsExample = new OfflineMapsExample(mapView);
//                    offlineMapsExample.onDownloadListClicked();
                    offlineMapsExample = DownloadMapsActivity.getOfflineMapsExample();
                    offlineMapsExample.changeSnackBarView(mapView);
                    regionsMapAdapter = new RegionsMapAdapter(DownloadRegionMapsActivity.this, regionArrayList,null,offlineMapsExample);
                    regionsMapAdapter.setItemClickListener(DownloadRegionMapsActivity.this);
                    regionMapsRv.setLayoutManager(new LinearLayoutManager(DownloadRegionMapsActivity.this, LinearLayoutManager.VERTICAL, false));
                    regionMapsRv.setAdapter(regionsMapAdapter);
                    LoadingDialog.dismissDialog(dialog);
                } else {
                    Log.d(TAG, "Loading map failed: mapErrorCode: " + mapError.name());
                }
            }
        });
    }



    @Override
    protected void onPause() {
        super.onPause();
        mapView.onPause();
    }

    @Override
    protected void onResume() {
        super.onResume();
        mapView.onResume();
    }

    @Override
    protected void onDestroy() {
        offlineMapsExample.onCancelAllMapDownloadClicked();
        super.onDestroy();
        mapView.onDestroy();
    }



    public void onDownloadListClicked(View view) {
//        offlineMapsExample.onDownloadListClicked();
    }

    public void onDownloadMapClicked(View view) {

    }

    public void onCancelMapDownloadClicked(View view) {
//        offlineMapsExample.onCancelMapDownloadClicked();
    }

    public void onSearchPlaceClicked(View view) {
//        offlineMapsExample.onSearchPlaceClicked();
    }

    @Override
    public void onRegionNavigateClick(View view, int position) {
        Gson gson = new Gson();
        String region = gson.toJson(regionArrayList.get(position));
        Intent intent=new Intent(DownloadRegionMapsActivity.this,DownloadRegionMapsActivity.class);
        intent.putExtra("region", region);
        startActivity(intent);
    }

    @Override
    public void onRegionDownloadClick(View view, int position) {
        offlineMapsExample.onDownloadMapClicked(regionArrayList.get(position),regionsMapAdapter,position);
    }

    @Override
    public void onRegionDownloadCancelClick(View view, int position) {
        offlineMapsExample.onCancelMapDownloadClicked(position);
    }

    @Override
    public void onRegionDeleteClick(View view, int position) {
        offlineMapsExample.onDeleteMapClicked(regionArrayList.get(position),regionsMapAdapter,position);
    }

    @Override
    public void onBackPressed() {
        setResult(RESULT_OK);
        finish();
    }
}