package com.example.heresdkandroidref.activity;

import androidx.appcompat.app.ActionBarDrawerToggle;
import androidx.appcompat.app.AppCompatActivity;
import androidx.appcompat.widget.SearchView;
import androidx.appcompat.widget.Toolbar;
import androidx.drawerlayout.widget.DrawerLayout;
import androidx.recyclerview.widget.DefaultItemAnimator;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import android.content.Context;
import android.content.Intent;
import android.os.Bundle;
import android.os.Environment;
import android.view.Menu;
import android.view.MenuItem;
import android.view.SubMenu;
import android.view.View;
import android.widget.LinearLayout;
import android.widget.TextView;

import com.example.heresdkandroidref.R;
import com.example.heresdkandroidref.adapters.RegionsMapAdapter;
import com.example.heresdkandroidref.adapters.TracksAdapter;
import com.example.heresdkandroidref.interfaces.GPXTracksClickListener;
import com.example.heresdkandroidref.interfaces.ItemsClickListener;
import com.example.heresdkandroidref.models.PlaceData;
import com.example.heresdkandroidref.utils.LoadingDialog;
import com.google.android.material.bottomsheet.BottomSheetBehavior;
import com.google.android.material.navigation.NavigationView;

import java.io.File;
import java.util.AbstractCollection;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

public class GpxTracksActivity extends AppCompatActivity implements GPXTracksClickListener {
    private static final String TAG = GpxTracksActivity.class.getSimpleName();
    private RecyclerView tracksRV;
    private TextView noTracksFoundTV;
    private LoadingDialog dialog;
    private ArrayList<File> tracksArrayList = new ArrayList<>();
    private TracksAdapter tracksAdapter;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_gpx_tracks);
        initViews();
    }

    public void initViews() {
        Toolbar toolbar = findViewById(R.id.toolbar);
        toolbar.setTitle("GPX Tracks");
        setSupportActionBar(toolbar);

        getSupportActionBar().setDisplayHomeAsUpEnabled(true);
        getSupportActionBar().setDisplayShowHomeEnabled(true);

        tracksRV = findViewById(R.id.tracksRV);
        noTracksFoundTV = findViewById(R.id.noTracksFoundTV);
        this.dialog = new LoadingDialog(this);
        //init recyclerview
        LoadingDialog.showDialog(dialog);
        initRecyclerView(this);
    }

    private void initRecyclerView(Context context) {
        // Get the documents directory of device.
        File dir = context.getExternalFilesDir(Environment.DIRECTORY_DOCUMENTS);

        assert dir != null;
        tracksArrayList.addAll(dir.listFiles() != null ? Arrays.asList(dir.listFiles()) : new ArrayList<>());
        //init recycler views
        if (!tracksArrayList.isEmpty()) {
            tracksRV.setLayoutManager(new LinearLayoutManager(this, LinearLayoutManager.VERTICAL, false));
            tracksRV.setHasFixedSize(true);
            tracksAdapter = new TracksAdapter(this, tracksArrayList, this);
            tracksRV.setAdapter(tracksAdapter);
            //manage views
            tracksRV.setVisibility(View.VISIBLE);
            noTracksFoundTV.setVisibility(View.GONE);
        } else {
            tracksRV.setVisibility(View.GONE);
            noTracksFoundTV.setVisibility(View.VISIBLE);
        }
        LoadingDialog.dismissDialog(this.dialog);
    }

    @Override
    public boolean onSupportNavigateUp() {
        onBackPressed();
        return true;
    }

    @Override
    public void onItemClick(View view, int position) {
        startActivity(new Intent(this, GpxPlayerActivity.class)
                .putExtra("gpxFile", tracksArrayList.get(position).getName())
        );
    }

    @Override
    public void onItemDeleteClick(View view, int position) {
        File selectedFile = tracksArrayList.get(position);
        if (selectedFile != null) {
            selectedFile.delete();
        }
    }
}