
package com.example.heresdkandroidref.activity;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.appcompat.app.ActionBarDrawerToggle;
import androidx.appcompat.app.AlertDialog;
import androidx.appcompat.app.AppCompatActivity;
import androidx.appcompat.widget.SearchView;
import androidx.appcompat.widget.Toolbar;
import androidx.core.view.GravityCompat;
import androidx.drawerlayout.widget.DrawerLayout;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.pm.ActivityInfo;
import android.graphics.Typeface;
import android.net.Uri;
import android.os.Handler;
import android.os.Looper;
import android.provider.Settings;
import android.text.Spannable;
import android.text.SpannableString;
import android.util.Log;

import android.os.Bundle;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.SubMenu;
import android.view.View;
import android.view.WindowManager;
import android.widget.LinearLayout;
import android.widget.TextView;
import android.widget.Toast;

import com.example.heresdkandroidref.R;
import com.example.heresdkandroidref.adapters.SuggestionsAdapter;
import com.example.heresdkandroidref.events.GetPlacesEvent;
import com.example.heresdkandroidref.features.OfflineMapsExample;
import com.example.heresdkandroidref.interfaces.SearchedItemClickListener;
import com.example.heresdkandroidref.models.PlaceData;
import com.example.heresdkandroidref.utils.SharedPref;
import com.example.heresdkandroidref.utils.CustomTypeFace;
import com.example.heresdkandroidref.utils.PermissionsRequestor;
import com.example.heresdkandroidref.features.Positioning;
import com.example.heresdkandroidref.features.Routing;
import com.example.heresdkandroidref.features.Search;
import com.example.heresdkandroidref.utils.Utils;
import com.google.android.material.bottomsheet.BottomSheetBehavior;
import com.google.android.material.floatingactionbutton.FloatingActionButton;
import com.google.android.material.navigation.NavigationView;
import com.google.gson.Gson;
import com.here.sdk.core.GeoCoordinates;
import com.here.sdk.core.Metadata;
import com.here.sdk.core.engine.CatalogConfiguration;
import com.here.sdk.core.engine.CatalogVersionHint;
import com.here.sdk.core.engine.DesiredCatalog;
import com.here.sdk.core.engine.LogControl;
import com.here.sdk.core.engine.SDKNativeEngine;
import com.here.sdk.core.engine.SDKOptions;
import com.here.sdk.core.errors.InstantiationErrorException;
import com.here.sdk.gestures.PanListener;
import com.here.sdk.mapview.MapError;
import com.here.sdk.mapview.MapFeatureModes;
import com.here.sdk.mapview.MapFeatures;
import com.here.sdk.mapview.MapMeasure;
import com.here.sdk.mapview.MapPolyline;
import com.here.sdk.mapview.MapScheme;
import com.here.sdk.mapview.MapView;
import com.here.sdk.mapview.MapScene;
import com.here.sdk.mapview.VisibilityState;
import com.here.sdk.search.Place;
import com.here.sdk.search.PlaceSerializationException;

import org.greenrobot.eventbus.EventBus;
import org.greenrobot.eventbus.Subscribe;
import org.greenrobot.eventbus.ThreadMode;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class MainActivity extends AppCompatActivity implements SearchedItemClickListener, NavigationView.OnNavigationItemSelectedListener, Search.MarkerDetailListener {

    private static final String TAG = MainActivity.class.getSimpleName();
    private PermissionsRequestor permissionsRequestor;
    private boolean doubleBackToExitPressedOnce = false;
    private MapView mapView;
    private Positioning positioning;
    private Routing routing;
    private Search search;
    private BottomSheetBehavior searchSheetBehavior;
    private BottomSheetBehavior searchResultSheetBehavior;
    private FloatingActionButton resetLocFab;
    private RecyclerView suggestionsRv;
    private final ArrayList<PlaceData> placesSuggestions = new ArrayList<>();
    //    ArrayList<RecentPlaceModel> recentPlaceModels= new ArrayList<>();
    private SearchView searchView;
    private TextView headerTxt;
    private TextView titleTxt;
    private TextView distanceTxt;
    private TextView addressTxt;
    public static boolean enableMapUpdate = true;
    private Place selectedPlace;
//    private final boolean categorySearched=false;
    private OfflineMapsExample offlineMapsExample;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        initializeHERESDK();
        setContentView(R.layout.activity_main);
        initViews(savedInstanceState);
        handleAndroidPermissions();
        // Usually, you need to initialize the HERE SDK only once during the lifetime of an application.

        setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_LOCKED);
        // Keeping the screen alive is essential for a car navigation app.
        getWindow().addFlags(WindowManager.LayoutParams.FLAG_KEEP_SCREEN_ON);
        searchListeners();
//        setSheetBehaviorListener();
        addGestureListener();
        new Handler(Looper.getMainLooper()).postDelayed(() -> Utils.disableSoftKeyboard(this), 1000);
    }

    public void initViews(Bundle savedInstanceState) {
        Toolbar myToolbar = (Toolbar) findViewById(R.id.toolbar);
        setSupportActionBar(myToolbar);
        LinearLayout searchLayoutBottomSheet = findViewById(R.id.search_bottom_sheet);
        LinearLayout searchResultsLayoutBottomSheet = findViewById(R.id.search_results_sheet);
        searchSheetBehavior = BottomSheetBehavior.from(searchLayoutBottomSheet);
        searchResultSheetBehavior = BottomSheetBehavior.from(searchResultsLayoutBottomSheet);

        mapView = findViewById(R.id.map_view);
        mapView.onCreate(savedInstanceState);
//        FloatingActionButton searchFab = findViewById(R.id.searchFab);
        resetLocFab = findViewById(R.id.resetLocationFab);
        suggestionsRv = findViewById(R.id.rvplaces);
        searchView = findViewById(R.id.searchview);
        headerTxt = findViewById(R.id.txt_header);
        titleTxt = findViewById(R.id.txt_title);
        addressTxt = findViewById(R.id.txt_address);
        distanceTxt = findViewById(R.id.txt_distance);
        DrawerLayout drawer = findViewById(R.id.drawer_layout);

        Toolbar toolbar = findViewById(R.id.toolbar);
        setSupportActionBar(toolbar);
        ActionBarDrawerToggle toggle = new ActionBarDrawerToggle(
                this, drawer, toolbar, R.string.navigation_drawer_open, R.string.navigation_drawer_close);
        drawer.addDrawerListener(toggle);
        toggle.syncState();

        NavigationView navigationView = findViewById(R.id.nav_view);
        navigationView.setNavigationItemSelectedListener(this);
        View hView = navigationView.getHeaderView(0);
//        TextView txt_name = hView.findViewById(R.id.txt_username);
//        TextView txt_contact = hView.findViewById(R.id.txt_email);


        Menu m = navigationView.getMenu();
        for (int i = 0; i < m.size(); i++) {
            MenuItem mi = m.getItem(i);

            //for aapplying a font to subMenu ...
            SubMenu subMenu = mi.getSubMenu();
            if (subMenu != null && subMenu.size() > 0) {
                for (int j = 0; j < subMenu.size(); j++) {
                    MenuItem subMenuItem = subMenu.getItem(j);
                    applyFontToMenuItem(subMenuItem);
                }
            }
            applyFontToMenuItem(mi);
        }

    }



    private void applyFontToMenuItem(MenuItem mi) {
        Typeface font = Typeface.create(Typeface.SERIF, Typeface.NORMAL);
        SpannableString mNewTitle = new SpannableString(mi.getTitle());
        mNewTitle.setSpan(new CustomTypeFace("", font), 0, mNewTitle.length(), Spannable.SPAN_INCLUSIVE_INCLUSIVE);
        mi.setTitle(mNewTitle);
    }

    private void showRecentPlacesList() {
        headerTxt.setText("Recent Searches");
        List<PlaceData> list = null;
        try {
            list = SharedPref.getInstance().getRecentPlacesList();
        } catch (PlaceSerializationException e) {
            e.printStackTrace();
        }
        placesSuggestions.clear();
        if (list != null && !list.isEmpty()) {
            Utils.disableSoftKeyboard(this);
            showList();
            placesSuggestions.addAll(list);
            setAdapter();
        }
    }


    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        MenuInflater inflater = getMenuInflater();
        inflater.inflate(R.menu.example_menu, menu);
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        // Handle item selection
        if (item.getItemId() == R.id.about) {
            Intent intent = new Intent(this, ConsentStateActivity.class);
            startActivity(intent);
            return true;
        }
        return super.onOptionsItemSelected(item);
    }

    private void handleAndroidPermissions() {
        permissionsRequestor = new PermissionsRequestor(this);
        permissionsRequestor.request(new PermissionsRequestor.ResultListener() {

            @Override
            public void permissionsGranted() {
                loadMapScene();
            }

            @Override
            public void permissionsDenied() {
                showDialog("Error", "Cannot start app: Location service and permissions are needed for this app.");
            }
        });
    }

    @Override
    public void onRequestPermissionsResult(int requestCode, @NonNull String[] permissions, @NonNull int[] grantResults) {
        super.onRequestPermissionsResult(requestCode, permissions, grantResults);
        permissionsRequestor.onRequestPermissionsResult(requestCode, grantResults);
    }

    private void initializeHERESDK() {
        // Disable any logs from HERE SDK.
        // Make sure to call this before initializing the HERE SDK.
//        LogControl.disableLoggingToConsole();

        // Set your credentials for the HERE SDK.
        String accessKeyID = "youraccessKeyID";
        String accessKeySecret = "youraccessKeySecret";
        SDKOptions options = new SDKOptions(accessKeyID, accessKeySecret);
        options.catalogConfigurations = getCustomCatalogConfiguration();
        try {
            Context context = this;
            SDKNativeEngine.makeSharedInstance(context, options);
        } catch (InstantiationErrorException e) {
            throw new RuntimeException("Initialization of HERE SDK failed: " + e.error.name());
        }
    }
    private List<CatalogConfiguration> getCustomCatalogConfiguration() {
        List<CatalogConfiguration> catalogConfigurations = new ArrayList<>();

        // We want to start with the latest catalog version.
        boolean ignoreCachedData = true;
        CatalogVersionHint versionHint = CatalogVersionHint.latest(ignoreCachedData);

        String defaultOCMHRN = "hrn:here:data::olp-here:ocm";
        DesiredCatalog defaultOCMCatalog = new DesiredCatalog(defaultOCMHRN, versionHint);
        catalogConfigurations.add(new CatalogConfiguration(defaultOCMCatalog));

        // Specify a rich Japan map, requires special HERE credentials.
        // If the credentials are not enabled for access, the map falls back to the Japan base map.
        String japanOCMHRN = "hrn:here:data::olp-here:ocm-japan";
        DesiredCatalog japanOCMCatalog = new DesiredCatalog(japanOCMHRN, versionHint);
        catalogConfigurations.add(new CatalogConfiguration(japanOCMCatalog));

        // Optionally, set a patch HRN to enable incremental map updates.

        return catalogConfigurations;

    }
    private void loadMapScene() {
//        try {
//            mapView.getCamera().getLimits().setMaxTilt(45);
//        } catch (MapCameraLimits.MapCameraLimitsException e) {
//            e.printStackTrace();
//        }
        // Load a scene from the HERE SDK to render the map with a map scheme.
        mapView.getMapScene().loadScene(MapScheme.NORMAL_DAY, new MapScene.LoadSceneCallback() {

            @Override
            public void onLoadScene(@Nullable MapError mapError) {
                if (mapError == null) {
                    offlineMapsExample = new OfflineMapsExample(mapView);
                    Map<String, String> map = new HashMap<>();
                    map.put(MapFeatures.LANDMARKS, MapFeatureModes.LANDMARKS_TEXTURED);
                    //map.put(MapFeatures.TRAFFIC_FLOW, MapFeatureModes.TRAFFIC_FLOW_WITH_FREE_FLOW);
                    map.put(MapFeatures.VEHICLE_RESTRICTIONS, MapFeatureModes.VEHICLE_RESTRICTIONS_ACTIVE_AND_INACTIVE_DIFFERENTIATED);
                    map.put(MapFeatures.SAFETY_CAMERAS, MapFeatureModes.SAFETY_CAMERAS_ALL);
                    map.put(MapFeatures.EXTRUDED_BUILDINGS, MapFeatureModes.EXTRUDED_BUILDINGS_ALL);
                    map.put(MapFeatures.BUILDING_FOOTPRINTS, MapFeatureModes.BUILDING_FOOTPRINTS_ALL);
                    map.put(MapFeatures.ENVIRONMENTAL_ZONES, MapFeatureModes.ENVIRONMENTAL_ZONES_ALL);
                    map.put(MapFeatures.TRAFFIC_INCIDENTS, MapFeatureModes.TRAFFIC_INCIDENTS_ALL);
                    map.put(MapFeatures.CONGESTION_ZONES, MapFeatureModes.CONGESTION_ZONES_ALL);

                    List<String> placeCategoryListToHide = new ArrayList<>();
                    placeCategoryListToHide.add("400"); //EAT_AND_DRINK_RESTAURANT
                    MapScene.setPoiVisibility(placeCategoryListToHide, VisibilityState.HIDDEN);

                    mapView.getMapScene().enableFeatures(map);
                    positioning = new Positioning();
                    positioning.onMapSceneLoaded(mapView, false, MainActivity.this);
                    search = new Search(MainActivity.this, mapView, MainActivity.this);
                    routing = new Routing(MainActivity.this, mapView);
                    showRecentPlacesList();
                } else {
                    Log.d(TAG, "Loading map failed: mapError: " + mapError.name());
                }
            }
        });
    }

    public void setSheetBehaviorListener() {
        searchSheetBehavior.setBottomSheetCallback(new BottomSheetBehavior.BottomSheetCallback() {
            @Override
            public void onStateChanged(@NonNull View bottomSheet, int newState) {
                switch (newState) {
                    case BottomSheetBehavior.STATE_HIDDEN:
                    case BottomSheetBehavior.STATE_EXPANDED: {
//                        btnBottomSheet.setText(R.string.close);
                    }
                    break;
                    case BottomSheetBehavior.STATE_COLLAPSED: {
//                        btnBottomSheet.setText(R.string.expand);

                    }

                    break;
                    case BottomSheetBehavior.STATE_DRAGGING:
                    case BottomSheetBehavior.STATE_SETTLING:
                        searchSheetBehavior.setHideable(false);
                        break;
                }
            }

            @Override
            public void onSlide(@NonNull View bottomSheet, float slideOffset) {

            }
        });
        searchResultSheetBehavior.setBottomSheetCallback(new BottomSheetBehavior.BottomSheetCallback() {
            @Override
            public void onStateChanged(@NonNull View bottomSheet, int newState) {
                switch (newState) {
                    case BottomSheetBehavior.STATE_HIDDEN:
                        break;
                    case BottomSheetBehavior.STATE_EXPANDED: {
//                        btnBottomSheet.setText(R.string.close);
                    }
                    break;
                    case BottomSheetBehavior.STATE_COLLAPSED: {
//                        btnBottomSheet.setText(R.string.expand);
                    }
                    break;
                    case BottomSheetBehavior.STATE_DRAGGING:
                    case BottomSheetBehavior.STATE_SETTLING:
                        searchResultSheetBehavior.setHideable(false);
                        break;
                }
            }

            @Override
            public void onSlide(@NonNull View bottomSheet, float slideOffset) {

            }
        });
    }

    public void searchListeners() {
        searchView.setOnQueryTextListener(new SearchView.OnQueryTextListener() {
            @Override
            public boolean onQueryTextSubmit(String query) {
//                headerTxt.setText("Match Results");
//                Utils.disableSoftKeyboard(MainActivity.this);
                hideList();
                search.onSearchButtonClicked(query, true);
                return false;
            }

            @Override
            public boolean onQueryTextChange(String newText) {
                if (newText.isEmpty()) {
                    showRecentPlacesList();
                } else {
                    headerTxt.setText("Matching Results");
                    hideList();
                    if (search != null) {
                        search.onSearchButtonClicked(newText, false);
                    }
                }
                return false;
            }
        });
    }

    public void searchFabClicked(View view) {
        showSearchSheet();

    }

    public void geocodeAnAddressButtonClicked(View view) {
        search.onGeocodeButtonClicked();
    }

    public void addRouteButtonClicked(View view) {

    }

    public void addWaypointsButtonClicked(View view) {
        routing.addWaypoints(null, "car");
    }

    public void clearMapButtonClicked(View view) {
        routing.clearMap();
    }

    public void switchOnlineButtonClicked(View view) {
//        routing.onSwitchOnlineButtonClicked();
    }

    public void switchOfflineButtonClicked(View view) {
//        routing.onSwitchOfflineButtonClicked();
    }

    public void cancelBtn(View view) {
        hideSearchSheet();
    }

    public void backBtn(View view) {
        hideSearchResultsSheet();
        showSearchSheet();
    }

    void showSearchSheet() {
        searchSheetBehavior.setState(BottomSheetBehavior.STATE_EXPANDED);
    }

    void hideSearchSheet() {
        searchSheetBehavior.setState(BottomSheetBehavior.STATE_COLLAPSED);
    }

    void showSearchResultsSheet() {
        searchResultSheetBehavior.setState(BottomSheetBehavior.STATE_EXPANDED);
    }

    void hideSearchResultsSheet() {
        searchResultSheetBehavior.setState(BottomSheetBehavior.STATE_COLLAPSED);
    }


    @Override
    protected void onPause() {
        mapView.onPause();
        super.onPause();
    }

    @Override
    protected void onResume() {
        mapView.onResume();
        new Handler(Looper.getMainLooper()).postDelayed(() -> Utils.disableSoftKeyboard(this), 800);
        super.onResume();
    }

    @Override
    protected void onDestroy() {
        if (positioning != null) {
            positioning.stopLocating();
        }
        mapView.onDestroy();
        disposeHERESDK();
        super.onDestroy();
    }

    private void showDialog(String title, String message) {
        AlertDialog.Builder builder = new AlertDialog.Builder(this);
        builder.setTitle(title);
        builder.setMessage(message);
        builder.setPositiveButton("OK", new DialogInterface.OnClickListener() {
            public void onClick(DialogInterface dialog, int which) {
                Intent intent = new Intent(Settings.ACTION_APPLICATION_DETAILS_SETTINGS);
                intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
                Uri uri = Uri.fromParts("package", getPackageName(), null);
                intent.setData(uri);
                startActivity(intent);
                finish();
            }
        });
        builder.show();
    }

    @Override
    protected void onStart() {
        super.onStart();
        EventBus.getDefault().register(this);
    }

    @Override
    protected void onStop() {
        EventBus.getDefault().unregister(this);
        if (positioning != null) {
            positioning.stopLocating();
        }
        super.onStop();
    }

    @Subscribe(threadMode = ThreadMode.MAIN)
    public void onPlacesSuggestionEvent(GetPlacesEvent event) {
        if (event != null) {
//            Utils.disableSoftKeyboard(MainActivity.this);
            if (!searchView.getQuery().toString().isEmpty()) {
                showList();
                placesSuggestions.clear();
                placesSuggestions.addAll(event.getPlacesList());
                setAdapter();
            }
        }
    }

    private void setAdapter() {
        SuggestionsAdapter suggestionAdapter = new SuggestionsAdapter(this, placesSuggestions, null);
        suggestionAdapter.setItemClickListener(this);
        suggestionsRv.setLayoutManager(new LinearLayoutManager(this, LinearLayoutManager.VERTICAL, false));
        suggestionsRv.setAdapter(suggestionAdapter);
    }

    public void onSearchItemClick(View view, int position) {

        hideList();
        hideSearchSheet();
        showSearchResultsSheet();

        SharedPref.getInstance().saveRecentPlace(placesSuggestions.get(position));
//        GeoCoordinates start= mapView.getCamera().getState().targetCoordinates;
//        GeoCoordinates destination=placesSuggestions.get(position).getGeoCoordinates();
//        routingExample.addRoute(start,destination);
        // Note: getGeoCoordinates() may return null only for Suggestions.
        if (placesSuggestions.get(position).getPlace() != null) {
            titleTxt.setText(placesSuggestions.get(position).getTitle());
            addressTxt.setText(placesSuggestions.get(position).getPlace().getAddress().addressText);
            distanceTxt.setText(Utils.makeDistanceString(placesSuggestions.get(position).getPlace().getDistanceInMeters() != null ? placesSuggestions.get(position).getPlace().getDistanceInMeters() : 0));
            enableMapUpdate = false;

            Metadata metadata = new Metadata();
            metadata.setCustomValue("key_search_result", new Search.SearchResultMetadata(placesSuggestions.get(position).getPlace()));

            search.addPoiMapMarker(placesSuggestions.get(position).getPlace().getGeoCoordinates(), metadata);
            mapView.getCamera().lookAt(placesSuggestions.get(position).getPlace().getGeoCoordinates());
            setCurrentLocVisibility();
            selectedPlace = placesSuggestions.get(position).getPlace();
        } else {
            search.onSearchButtonClicked(placesSuggestions.get(position).getTitle(), true);
        }
        Utils.hideKeyboard(MainActivity.this);
//        searchExample.getMapViewCenter();
//        searchExample.getMapViewGeoBox();
    }

    void showList() {
        suggestionsRv.setVisibility(View.VISIBLE);
    }

    void hideList() {
        suggestionsRv.setVisibility(View.GONE);
    }

    public void resetLocationClicked(View view) {
        GeoCoordinates coordinates;
        if (positioning.getLocationEngine().getLastKnownLocation() == null) {
            Toast.makeText(this, "GPS is not enabled! using default location", Toast.LENGTH_SHORT).show();
            permissionsRequestor.request(new PermissionsRequestor.ResultListener() {

                @Override
                public void permissionsGranted() {

                }

                @Override
                public void permissionsDenied() {
                    showDialog("Error", "Cannot start app: Location service and permissions are needed for this app.");
                }
            });
            coordinates = Positioning.defaultCoordinates;
        } else {
            coordinates = positioning.getLocationEngine().getLastKnownLocation().coordinates;
        }
        MapMeasure mapMeasureZoom = new MapMeasure(MapMeasure.Kind.DISTANCE, Positioning.CAMERA_DISTANCE_TO_EARTH);
        mapView.getCamera().lookAt(
                coordinates, mapMeasureZoom);
        enableMapUpdate = true;
        setCurrentLocVisibility();
    }

    private void showSimpleDialog(String title, String message) {
        AlertDialog.Builder builder = new AlertDialog.Builder(this);
        builder.setTitle(title)
                .setMessage(message)
                .show();
    }

    void setCurrentLocVisibility() {
        if (!enableMapUpdate) {
            resetLocFab.setVisibility(View.VISIBLE);
        } else {
            resetLocFab.setVisibility(View.GONE);
        }

    }

    public void addGestureListener() {
        PanListener panListener = (gestureState, point2D, point2D1, v) -> {
            if (enableMapUpdate) {
                enableMapUpdate = false;
            }
            setCurrentLocVisibility();
        };
        mapView.getGestures().setPanListener(panListener);
    }

    public void routingBtn(View view) {
        startRoutingActivity(selectedPlace);
    }

    private void startRoutingActivity(Place place) {
        GeoCoordinates coordinates;
        if (positioning.getLocationEngine().getLastKnownLocation() == null) {
            Toast.makeText(this, "GPS is not enabled! using default location", Toast.LENGTH_SHORT).show();
            permissionsRequestor.request(new PermissionsRequestor.ResultListener() {

                @Override
                public void permissionsGranted() {

                }

                @Override
                public void permissionsDenied() {
                    showDialog("Error", "Cannot start app: Location service and permissions are needed for this app.");
                }
            });
            coordinates = Positioning.defaultCoordinates;
        } else {
            coordinates = positioning.getLocationEngine().getLastKnownLocation().coordinates;
        }
        Gson gson = new Gson();
        String currentLocation = gson.toJson(coordinates);

//        String placeInfo = gson.toJson(place);
        Intent intent = new Intent(MainActivity.this, RoutingActivity.class);
        intent.putExtra("fromShare", false);
        intent.putExtra("origin", "Your location");
        intent.putExtra("destination", place.getTitle());
        intent.putExtra("destinationLocation", gson.toJson(place.getGeoCoordinates()));
        intent.putExtra("currentLocation", currentLocation);
        intent.putStringArrayListExtra("waypoints", new ArrayList<>());
        startActivity(intent);
    }


    @Override
    public boolean onNavigationItemSelected(MenuItem item) {
        int id = item.getItemId();
        switch (id) {
            case R.id.nav_consent:
                Intent consentIntent = new Intent(this, ConsentStateActivity.class);
                startActivity(consentIntent);
                break;
            case R.id.nav_download_maps:
                Intent downloadMapsIntent = new Intent(this, DownloadMapsActivity.class);
                startActivity(downloadMapsIntent);
                break;
            case R.id.nav_offline:
                if (SharedPref.getInstance().getOfflineMode()) {
                    offlineMapsExample.onSwitchOfflineButtonClicked();
                    Toast.makeText(this, "Offline Mode Enabled", Toast.LENGTH_SHORT).show();
                } else {
                    offlineMapsExample.onSwitchOnlineButtonClicked();
                    Toast.makeText(this, "Offline Mode Disabled", Toast.LENGTH_SHORT).show();
                }
                SharedPref.getInstance().setOfflineMode(!SharedPref.getInstance().getOfflineMode());
                item.setChecked(SharedPref.getInstance().getOfflineMode());
                break;
            case R.id.nav_gpx_track:
                Intent intent = new Intent(this, GpxTracksActivity.class);
                startActivity(intent);
                break;
        }


        DrawerLayout drawer = (DrawerLayout) findViewById(R.id.drawer_layout);
        drawer.closeDrawer(GravityCompat.START);
        return true;
    }

    @Override
    public void onBackPressed() {
        if (searchSheetBehavior.getState() != BottomSheetBehavior.STATE_COLLAPSED) {
            hideSearchSheet();
        } else if (searchResultSheetBehavior.getState() != BottomSheetBehavior.STATE_COLLAPSED) {
            hideSearchResultsSheet();
        } else {
            makeDoubleClickPopup();
        }
    }

    //double click popup
    private void makeDoubleClickPopup() {
        if (doubleBackToExitPressedOnce) {
            super.onBackPressed();
            return;
        }
        this.doubleBackToExitPressedOnce = true;
        //make toast
        Toast.makeText(this, "Please click BACK again to exit", Toast.LENGTH_SHORT).show();
        //reset double click value
        new Handler().postDelayed(() -> doubleBackToExitPressedOnce = false, 3000);
    }

    @Override
    public void onClickPolyLine(List<MapPolyline> place) {

    }

    @Override
    public void onClickRoute(Place place) {
        selectedPlace = place;
        startRoutingActivity(place);
    }

    @Override
    public void onClickRouteNew(Place place) {

    }

    @Override
    public void onClickAddWaypoint(Place place) {

    }

    private void disposeHERESDK() {
        // Free HERE SDK resources before the application shuts down.
        // Usually, this should be called only on application termination.
        // Afterwards, the HERE SDK is no longer usable unless it is initialized again.
        SDKNativeEngine sdkNativeEngine = SDKNativeEngine.getSharedInstance();
        if (sdkNativeEngine != null) {
            sdkNativeEngine.dispose();
            // For safety reasons, we explicitly set the shared instance to null to avoid situations,
            // where a disposed instance is accidentally reused.
            SDKNativeEngine.setSharedInstance(null);
        }
    }
    @Override
    public void onCategorySearched(Place place) {
        titleTxt.setText(place.getTitle());
        addressTxt.setText(place.getAddress().addressText);
        distanceTxt.setText(Utils.makeDistanceString(place.getDistanceInMeters() != null ? place.getDistanceInMeters() : 0));
        enableMapUpdate = false;

        mapView.getCamera().lookAt(place.getGeoCoordinates());
        setCurrentLocVisibility();
        selectedPlace = place;
    }
}