package com.example.heresdkandroidref.activity;

import android.annotation.SuppressLint;
import android.content.Context;
import android.hardware.Sensor;
import android.hardware.SensorEvent;
import android.hardware.SensorEventListener;
import android.hardware.SensorManager;
import android.net.ConnectivityManager;
import android.os.Build;
import android.os.Bundle;
import android.text.method.ScrollingMovementMethod;
import android.util.Log;
import android.view.View;
import android.view.WindowManager;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.SeekBar;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.annotation.RequiresApi;
import androidx.appcompat.app.AlertDialog;
import androidx.appcompat.app.AppCompatActivity;
import androidx.cardview.widget.CardView;
import androidx.core.content.ContextCompat;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import com.example.heresdkandroidref.R;
import com.example.heresdkandroidref.adapters.LanesAdapter;
import com.example.heresdkandroidref.events.GetDetailsEvent;
import com.example.heresdkandroidref.events.GetLanesEvent;
import com.example.heresdkandroidref.events.GetNavigableLocation;
import com.example.heresdkandroidref.events.GetRouteProgressEvent;
import com.example.heresdkandroidref.events.ShowNavigationDialog;
import com.example.heresdkandroidref.events.SpeedLimitEvent;
import com.example.heresdkandroidref.events.StartNavigation;
import com.example.heresdkandroidref.features.App;
import com.example.heresdkandroidref.features.GPXRecorderExample;
import com.example.heresdkandroidref.features.Positioning;
import com.example.heresdkandroidref.utils.LoadingDialog;
import com.example.heresdkandroidref.utils.PermissionsRequestor;
import com.example.heresdkandroidref.utils.SharedPref;
import com.example.heresdkandroidref.utils.Utils;
import com.google.android.flexbox.FlexDirection;
import com.google.android.flexbox.FlexWrap;
import com.google.android.flexbox.FlexboxLayoutManager;
import com.google.android.flexbox.JustifyContent;
import com.google.android.material.floatingactionbutton.FloatingActionButton;
import com.google.gson.Gson;
import com.here.sdk.animation.AnimationListener;
import com.here.sdk.animation.AnimationState;
import com.here.sdk.core.Angle;
import com.here.sdk.core.Color;
import com.here.sdk.core.GeoCoordinates;
import com.here.sdk.core.GeoCoordinatesUpdate;
import com.here.sdk.core.GeoOrientation;
import com.here.sdk.core.GeoOrientationUpdate;
import com.here.sdk.core.GeoPolyline;
import com.here.sdk.core.Point2D;
import com.here.sdk.gestures.GestureState;
import com.here.sdk.gestures.PinchRotateListener;
import com.here.sdk.mapview.MapCamera;
import com.here.sdk.mapview.MapCameraAnimation;
import com.here.sdk.mapview.MapCameraAnimationFactory;
import com.here.sdk.mapview.MapCameraLimits;
import com.here.sdk.mapview.MapCameraListener;
import com.here.sdk.mapview.MapCameraUpdate;
import com.here.sdk.mapview.MapCameraUpdateFactory;
import com.here.sdk.mapview.MapError;
import com.here.sdk.mapview.MapFeatureModes;
import com.here.sdk.mapview.MapFeatures;
import com.here.sdk.mapview.MapMeasure;
import com.here.sdk.mapview.MapPolyline;
import com.here.sdk.mapview.MapScene;
import com.here.sdk.mapview.MapScheme;
import com.here.sdk.mapview.MapView;
import com.here.sdk.mapview.VisibilityState;
import com.here.sdk.navigation.ManeuverProgress;
import com.here.sdk.navigation.SectionProgress;
import com.here.sdk.routing.Maneuver;
import com.here.sdk.routing.ManeuverAction;
import com.here.sdk.routing.RoadTexts;
import com.here.sdk.routing.RoadType;
import com.here.sdk.routing.Route;
import com.here.sdk.routing.Waypoint;
import com.here.time.Duration;

import org.greenrobot.eventbus.EventBus;
import org.greenrobot.eventbus.Subscribe;
import org.greenrobot.eventbus.ThreadMode;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Calendar;
import java.util.HashMap;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.Optional;

public class NavigationActivity extends AppCompatActivity implements SensorEventListener, MapCameraListener {

    private static final double DISTANCE_IN_METERS = 2000;
    private static final String TAG = NavigationActivity.class.getSimpleName();
    //    static final double _kZoomDistanceToManeuver = 500;
    private final double _kDistanceToShowNextManeuver = 500;
    private final double _kKMpHinMpS = 3.6;
    private PermissionsRequestor permissionsRequestor;
    private MapView mapView;
    private GeoCoordinates destinationPlace;
    private GeoCoordinates startPlace;
    String startName, destinationName;
    private LoadingDialog dialog;
    private boolean isTrafficOn = true, isCompassOn = true;
    private boolean isRecordingOn = false;
    private Positioning positioning;
    //    private final ArrayList<Route> mapRoutes = new ArrayList<>();
//    private final ArrayList<MapPolyline> mapPolylines = new ArrayList<>();
//    private final int _selectedRouteIndex = 0;
//    private RecyclerView maneuverRv;
//    private final ArrayList<ManeuverModel> maneuverModelArrayList = new ArrayList<>();
//    private ManeuversAdapter maneuversAdapter;
//    private final List<MapMarker> maneuverMarkers = new ArrayList<>();
    private String selectedOption = "";
    private Route selectedRoute;
    //public TextView itemTime,itemDelay,itemDistance;
    private App app;
    private TextView messageView;
    private TextView arrivalTV, hourTV, minTV, kmTV, speedLimitTV, speedTV, speedHeaderTV, kmHeaderTV;
    //    private View speedLimitV;
    private ImageView newManeuverImg, maneuverImg, trafficImg, recordGpxImg;
    private TextView newManeuverDistanceTV, newManeuverActionTV;
    private TextView maneuverDistanceTV, maneuverActionTV;
    private LinearLayout newManeuverLayout;
    private FloatingActionButton voiceFab;
    private CardView recenterCV;
    private CardView compassFab;
    private RecyclerView lanesListRV;
    //    private final int previousManeuverIndex = -1;
    private Route navigationRoute;
    private SeekBar routeProgressSeekbar;
    private List<Waypoint> waypointsList = new ArrayList<>();
    private SensorManager mSensorManager;
    private Sensor mRotationSensor;
    private float[] mRotationMatrix = new float[9];
    private float[] mOrientation = new float[3];
    private GeoCoordinates currentLocation;
    private int cameraZoom = 0;
    private GPXRecorderExample gpxRecorderExample;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_navigation);
        initViews(savedInstanceState);
        getWindow().addFlags(WindowManager.LayoutParams.FLAG_KEEP_SCREEN_ON);
        handleAndroidPermissions();
    }

    private void initViews(Bundle savedInstanceState) {
        // Get a MapView instance from layout.
        mapView = findViewById(R.id.map_view);
        mapView.onCreate(savedInstanceState);
//        pickupTv =  findViewById(R.id.pickup_txt);
//        dropOffTv =  findViewById(R.id.dropoff_txt);
//        maneuverRv = findViewById(R.id.maneuver_rv);
        arrivalTV = findViewById(R.id.arrival_txt);
        voiceFab = findViewById(R.id.voiceFab);
        hourTV = findViewById(R.id.h_txt);
        minTV = findViewById(R.id.min_txt);
        kmTV = findViewById(R.id.km_txt);
        kmHeaderTV = findViewById(R.id.km_header_txt);
        speedLimitTV = findViewById(R.id.speed_limit_txt);
//        speedLimitV = findViewById(R.id.speed_limit_view);
        speedTV = findViewById(R.id.speed_txt);
        speedHeaderTV = findViewById(R.id.speed_header_txt);
        newManeuverImg = findViewById(R.id.new_maneuver_img);
        newManeuverDistanceTV = findViewById(R.id.new_maneuver_distance_txt);
        newManeuverActionTV = findViewById(R.id.new_maneuver_action_txt);
        maneuverImg = findViewById(R.id.maneuver_action_img);
        trafficImg = findViewById(R.id.traffic_img);
        recordGpxImg = findViewById(R.id.record_gpx_img);
        maneuverDistanceTV = findViewById(R.id.maneuver_distance_txt);
        maneuverActionTV = findViewById(R.id.maneuver_action_txt);
        newManeuverLayout = findViewById(R.id.new_maneuver_panel);
        recenterCV = findViewById(R.id.recenterCV);
        compassFab = findViewById(R.id.compassFab);
        lanesListRV = findViewById(R.id.lanesListRV);
//        itemDelay = findViewById(R.id.delay_txt);
//        itemDistance = findViewById(R.id.distance_txt);
        routeProgressSeekbar = findViewById(R.id.route_progress_seekbar);
        messageView = findViewById(R.id.message_view);
        // Making the textView scrollable.
        messageView.setMovementMethod(new ScrollingMovementMethod());
//        ToggleButton toggleTrackingButton = findViewById(R.id.toggleTrackingButton);
//        toggleTrackingButton.setTextOn("Camera Tracking: ON");
//        toggleTrackingButton.setTextOff("Camera Tracking: OFF");
//        toggleTrackingButton.setChecked(true);
//        toggleTrackingButton.setOnClickListener(v -> {
//            if (app == null) return;
//            if (toggleTrackingButton.isChecked()) {
//                app.toggleTrackingButtonOnClicked();
//            } else {
//                app.toggleTrackingButtonOffClicked();
//            }
//        });

        getDetails();
        recenterCV.setOnClickListener(view -> {
            if (isCompassOn) {
                app.toggleTrackingButtonOnClicked();
            } else {
                app.geoOrientation = null;
            }
            recenterCV.setVisibility(View.GONE);
        });
        //sensor configs
        mSensorManager = (SensorManager) getSystemService(Context.SENSOR_SERVICE);
        // Get a reference to the rotation vector sensor
        mRotationSensor = mSensorManager.getDefaultSensor(Sensor.TYPE_ROTATION_VECTOR);
    }

    private void getDetails() {
        try {
            Gson gson = new Gson();
            destinationPlace = gson.fromJson(getIntent().getStringExtra("destination"), GeoCoordinates.class);
            startPlace = gson.fromJson(getIntent().getStringExtra("start"), GeoCoordinates.class);
            startName = getIntent().getStringExtra("startName");
            destinationName = getIntent().getStringExtra("destinationName");
            selectedRoute = gson.fromJson(SharedPref.getInstance().getRouteData(), Route.class);
            selectedOption = getIntent().getStringExtra("selected_option");
            List<String> list = SharedPref.getInstance().getWaypoints();
            for (String waypoints : list) {
                waypointsList.add(new Gson().fromJson(waypoints, Waypoint.class));
            }
//            itemTime.setText(Utils.makeDurationString(selectedRoute.getDurationInSeconds()));
//            if (selectedRoute.getTrafficDelayInSeconds() > 60){
//                itemDelay.setText("Incl. "+Utils.makeDurationString(selectedRoute.getTrafficDelayInSeconds())+" delay");
//            } else {
//                itemDelay.setText("No delays");
//            }
//            itemDistance.setText(Utils.makeDistanceString(selectedRoute.getLengthInMeters()));
            this.dialog = new LoadingDialog(this);
            LoadingDialog.showDialog(dialog);
        } catch (Exception exception) {
            Log.e("Exception", exception.getMessage());
        }

    }

    private void handleAndroidPermissions() {
        permissionsRequestor = new PermissionsRequestor(this);
        permissionsRequestor.request(new PermissionsRequestor.ResultListener() {

            @Override
            public void permissionsGranted() {
                loadMapScene();
            }

            @Override
            public void permissionsDenied() {
                Log.e(TAG, "Permissions denied by user.");
            }
        });
    }

    @Override
    public void onRequestPermissionsResult(int requestCode, @NonNull String[] permissions, @NonNull int[] grantResults) {
        super.onRequestPermissionsResult(requestCode, permissions, grantResults);
        permissionsRequestor.onRequestPermissionsResult(requestCode, grantResults);
    }

    private void loadMapScene() {
//        try {
//            mapView.getCamera().getLimits().setMaxTilt(45);
//        } catch (MapCameraLimits.MapCameraLimitsException e) {
//            e.printStackTrace();
//        }
        mapView.getMapScene().loadScene(MapScheme.NORMAL_DAY, new MapScene.LoadSceneCallback() {
            @Override
            public void onLoadScene(@Nullable MapError mapError) {
                getWindow().addFlags(WindowManager.LayoutParams.FLAG_KEEP_SCREEN_ON);
                if (mapError == null) {
                    Map<String, String> map = new HashMap<>();
                    map.put(MapFeatures.LANDMARKS, MapFeatureModes.LANDMARKS_TEXTURED);
                    //map.put(MapFeatures.TRAFFIC_FLOW, MapFeatureModes.TRAFFIC_FLOW_WITH_FREE_FLOW);
                    map.put(MapFeatures.VEHICLE_RESTRICTIONS, MapFeatureModes.VEHICLE_RESTRICTIONS_ACTIVE_AND_INACTIVE_DIFFERENTIATED);
                    map.put(MapFeatures.SAFETY_CAMERAS, MapFeatureModes.SAFETY_CAMERAS_ALL);
                    map.put(MapFeatures.EXTRUDED_BUILDINGS, MapFeatureModes.EXTRUDED_BUILDINGS_ALL);
                    map.put(MapFeatures.BUILDING_FOOTPRINTS, MapFeatureModes.BUILDING_FOOTPRINTS_ALL);
                    map.put(MapFeatures.ENVIRONMENTAL_ZONES, MapFeatureModes.ENVIRONMENTAL_ZONES_ALL);
                    map.put(MapFeatures.TRAFFIC_INCIDENTS, MapFeatureModes.TRAFFIC_INCIDENTS_ALL);
                    map.put(MapFeatures.CONGESTION_ZONES, MapFeatureModes.CONGESTION_ZONES_ALL);
                    map.put(MapFeatures.TRAFFIC_FLOW, MapFeatureModes.TRAFFIC_FLOW_WITH_FREE_FLOW);
                    mapView.getMapScene().enableFeatures(map);
                    gpxRecorderExample = new GPXRecorderExample(mapView, NavigationActivity.this);
                    app = new App(NavigationActivity.this, mapView, messageView, routeProgressSeekbar, waypointsList);
//                    app.showRouteDetails(selectedRoute, true);
                    positioning = new Positioning();
//                    mapView.getMapScene().setLayerVisibility(MapScene.Layers.TRAFFIC_FLOW, VisibilityState.VISIBLE);
                    //mapView.getMapScene().setLayerVisibility(MapFeatures.TRAFFIC_FLOW, VisibilityState.VISIBLE);
                    mapView.getCamera().addListener(NavigationActivity.this);
                    positioning.onMapSceneLoaded(mapView, app.isSimulated, NavigationActivity.this);
//                    if (selectedOption.equalsIgnoreCase("truck") && isDeviceConnected()) {
//                        try {
//                            addRouteToMap(selectedRoute);
//                        } catch (Exception ex) {
//                            ex.printStackTrace();
//                        }
//                    }
//                    app.toggleTrackingButtonOnClicked();
                    showStartNavigationDialog();
                    setGestureHandler();
                } else {
                    Log.d(TAG, "Loading map failed: mapErrorCode: " + mapError.name());
                }
            }
        });
    }

    private void setGestureHandler() {
        mapView.getGestures().setTwoFingerPanListener((gestureState, point2D, point2D1, v) -> {
//            if(isCompassOn) {
            app.toggleTrackingButtonOffClicked();
            recenterCV.setVisibility(View.VISIBLE);
            if (app.geoOrientation == null) {
                app.geoOrientation = mapView.getCamera().getState().orientationAtTarget;
                app.zoomLevel = mapView.getCamera().getState().zoomLevel;
            }
//            }
        });
        mapView.getGestures().setPanListener((gestureState, point2D, point2D1, v) -> {
//            if(isCompassOn) {
            app.toggleTrackingButtonOffClicked();
            recenterCV.setVisibility(View.VISIBLE);
            if (app.geoOrientation == null) {
                app.geoOrientation = mapView.getCamera().getState().orientationAtTarget;
                app.zoomLevel = mapView.getCamera().getState().zoomLevel;
            }
//            }
        });
        mapView.getGestures().setPinchRotateListener((gestureState, point2D, point2D1, v, angle) -> {
//            if(isCompassOn) {
            app.toggleTrackingButtonOffClicked();
            recenterCV.setVisibility(View.VISIBLE);
            if (app.geoOrientation == null) {
                app.geoOrientation = mapView.getCamera().getState().orientationAtTarget;
                app.zoomLevel = mapView.getCamera().getState().zoomLevel;
            }
//            }
        });
    }


    @Override
    protected void onPause() {
        super.onPause();
        mapView.onPause();
        // Unregister the sensor listener
        mSensorManager.unregisterListener(this);
    }

    @Override
    protected void onResume() {
        super.onResume();
        mapView.onResume();
        // Register the sensor listener
        mSensorManager.registerListener(this, mRotationSensor, SensorManager.SENSOR_DELAY_NORMAL);
    }

    @Override
    protected void onDestroy() {
        if (positioning != null) {
            positioning.stopLocating();
        }
        if (app != null) {
            app.stopLocating();
        }
        mapView.onDestroy();
        super.onDestroy();
    }

    public void trafficBtn(View view) {
        isTrafficOn = !isTrafficOn;
        if (isTrafficOn) {
            trafficImg.setImageDrawable(ContextCompat.getDrawable(this, R.drawable.traffic_on));
            Map<String, String> map = new HashMap<>();
            // Once these traffic layers are added to the map, they will be automatically updated while panning the map.
            map.put(MapFeatures.TRAFFIC_FLOW, MapFeatureModes.TRAFFIC_FLOW_WITH_FREE_FLOW);
            // MapFeatures.TRAFFIC_INCIDENTS renders traffic icons and lines to indicate the location of incidents.
            map.put(MapFeatures.TRAFFIC_INCIDENTS, MapFeatureModes.DEFAULT);
            mapView.getMapScene().enableFeatures(map);
        } else {
            trafficImg.setImageDrawable(ContextCompat.getDrawable(this, R.drawable.traffic_off));
            List<String> map = new ArrayList<>();
            map.add(MapFeatures.TRAFFIC_FLOW);
            map.add(MapFeatures.TRAFFIC_INCIDENTS);
            mapView.getMapScene().disableFeatures(map);
        }
        //update traffic lines
//        LoadingDialog.showDialog(dialog);
        app.drawTrafficPolylineForRoute(selectedRoute, isTrafficOn);
    }

    public void recodingGPX(View view) {
        isRecordingOn = !isRecordingOn;
        if (isRecordingOn) {
            recordGpxImg.setImageDrawable(ContextCompat.getDrawable(this, R.drawable.stop_recording));
            gpxRecorderExample.recordGPXTraceButtonPressed();
        } else {
            recordGpxImg.setImageDrawable(ContextCompat.getDrawable(this, R.drawable.start_recording));
            gpxRecorderExample.stopButtonPressed(startName+" - "+destinationName+"_"+System.currentTimeMillis());
        }
    }

    public void compassClicked(View view) {
        isCompassOn = !isCompassOn;
        if (!isCompassOn) {
            app.toggleTrackingButtonOffClicked();
        } else {
            app.toggleTrackingButtonOnClicked();
        }
    }

//    private void addRouteToMap(Route route) {
//        try {
//            GeoPolyline routeGeoPolyline = route.getGeometry();
//            MapPolyline routeMapPolyline;
//            routeMapPolyline = new MapPolyline(
//                    routeGeoPolyline, 20.0, Color.valueOf(getResources().getColor(R.color.selectedRouteColor)));
//            routeMapPolyline.setDrawOrder(1);
//            //routeMapPolyline.setOutlineColor(Color.valueOf(getResources().getColor(R.color.selectedRouteBorderColor)));
//            routeMapPolyline.setProgressColor(Color.valueOf(0, 0, 0, 0)); // Clear color.
//
//            routeMapPolyline.setOutlineWidth(5.0);
//
//            mapView.getMapScene().addMapPolyline(routeMapPolyline);
////            mapPolylines.add(routeMapPolyline);
//
//            LoadingDialog.dismissDialog(dialog);
//        } catch (Exception e) {
//            e.printStackTrace();
//        }
//    }


    public void closeBtn(View view) {
        finish();
    }


//    public boolean isDeviceConnected() {
////        // An application may define here a logic to determine whether a device is connected or not.
////        // For this example app, the flag is set from UI.
////        return isDeviceConnected && !isOfflineModeEnabled;
//        ConnectivityManager connectivityManager = ((ConnectivityManager) getSystemService(Context.CONNECTIVITY_SERVICE));
//        return connectivityManager.getActiveNetworkInfo() != null && connectivityManager.getActiveNetworkInfo().isConnected();
//    }


//    public void addRouteSimulatedLocationButtonClicked(View view) {
//        if (app != null) {
//            app.addRouteSimulatedLocation(startPlace,destinationPlace);
//        }
//    }

//    public void addRouteDeviceLocationButtonClicked(View view) {
//        if (app != null) {
//            app.addRouteDeviceLocation();
//        }
//    }

    public void clearMapButtonClicked(View view) {
        if (app != null) {
            app.clearMapButtonPressed();
        }
    }

    public void voiceClicked(View view) {
        if (app != null) {
            if (app.isVoiceEnabled()) {
                app.switchVoiceEnables(false);
                voiceFab.setImageResource(R.drawable.volume_off);
            } else {
                app.switchVoiceEnables(true);
                voiceFab.setImageResource(R.drawable.volume_on);
            }
        }
    }

    public void closeClicked(View view) {
        if (app != null) {
            app.stopLocating();
        }
        finish();
    }

    @Override
    protected void onStart() {
        super.onStart();
        EventBus.getDefault().register(this);

    }

    @Override
    protected void onStop() {
        EventBus.getDefault().unregister(this);
//        if (positioning != null) {
//            positioning.stopLocating();
//        }
//        if (app != null) {
//            app.stopLocating();
//        }
        super.onStop();
    }

//    // Animate to custom guidance perspective, centered on start location of route.
//    private void animateToRouteStart(Route route) {
//        // The first coordinate marks the start location of the route.
////        GeoCoordinates startOfRoute = mapView.getCamera().getState().targetCoordinates;//route.getGeometry().vertices.get(0);
//        GeoCoordinates startOfRoute = route.getGeometry().vertices.get(0);
//        GeoCoordinatesUpdate geoCoordinatesUpdate = new GeoCoordinatesUpdate(startOfRoute);
//
//        Double bearingInDegrees = null;
//        double tiltInDegrees = 70;
//        GeoOrientationUpdate orientationUpdate = new GeoOrientationUpdate(bearingInDegrees, tiltInDegrees);
//
//        double distanceInMeters = 75;
//        MapMeasure mapMeasureZoom = new MapMeasure(MapMeasure.Kind.DISTANCE, distanceInMeters);
//
//        double bowFactor = 1;
//        MapCameraAnimation animation = MapCameraAnimationFactory.flyTo(
//                geoCoordinatesUpdate, orientationUpdate, mapMeasureZoom, bowFactor, Duration.ofSeconds(4));
//        mapView.getCamera().startAnimation(animation, new AnimationListener() {
//            @Override
//            public void onAnimationStateChanged(@NonNull AnimationState animationState) {
//                if (animationState == AnimationState.COMPLETED || animationState == AnimationState.CANCELLED) {
//                    app.toggleTrackingButtonOnClicked();
//                    app.showRouteOnMap(route, isTrafficOn);
//                    app.showRouteDetails(route, app.isSimulated);
//                }
//            }
//        });
//    }

//    private void animateToDefaultMapPerspective() {
//        GeoCoordinates targetLocation = mapView.getCamera().getState().targetCoordinates;
//        GeoCoordinatesUpdate geoCoordinatesUpdate = new GeoCoordinatesUpdate(targetLocation);
//
//        // By setting null we keep the current bearing rotation of the map.
//        Double bearingInDegrees = mapView.getCamera().getState().orientationAtTarget.bearing;
//        double tiltInDegrees = 70;
//        GeoOrientationUpdate orientationUpdate = new GeoOrientationUpdate(bearingInDegrees, tiltInDegrees);
//
//        MapMeasure mapMeasureZoom = new MapMeasure(MapMeasure.Kind.DISTANCE, 75);
//        double bowFactor = 1;
//        MapCameraAnimation animation = MapCameraAnimationFactory.flyTo(
//                geoCoordinatesUpdate, orientationUpdate, mapMeasureZoom, bowFactor, Duration.ofSeconds(2));
//        mapView.getCamera().startAnimation(animation);
//    }

    @Subscribe(threadMode = ThreadMode.MAIN)
    public void onGetDetailEvent(GetDetailsEvent event) {
        if (event != null && event.getRoutesList() != null) {
            if (!event.getRoutesList().isEmpty()) {
                Route route = event.getRoutesList().get(0);
                selectedRoute = route;
            }
//            LoadingDialog.dismissDialog(dialog);
            app.animateToRouteStart(selectedRoute);
        }
    }

    @SuppressLint("UseCompatLoadingForDrawables")
    @RequiresApi(api = Build.VERSION_CODES.N)
    @Subscribe(threadMode = ThreadMode.MAIN)
    public void onStartNavigationEvent(StartNavigation event) throws ParseException {
        if (event != null) {
            navigationRoute = event.getRoute();
            Calendar now = Calendar.getInstance();
            now.add(Calendar.HOUR, event.getRoute().getDuration().toHoursPart());
            now.add(Calendar.MINUTE, event.getRoute().getDuration().toMinutesPart());
            System.out.println(now.getTime());
            SimpleDateFormat outputFormat = new SimpleDateFormat("h:mm:ss");
            String outputText = outputFormat.format(now.getTime());
            arrivalTV.setText(outputText);
            hourTV.setText(Utils.getHourFromDuration(event.getRoute().getDuration().getSeconds()));
            minTV.setText(Utils.getMinutesFromDuration(event.getRoute().getDuration().getSeconds()));
            kmTV.setText(event.getRoute().getLengthInMeters() / 1000 + "");
            List<Maneuver> maneuvers = event.getRoute().getSections().stream().findFirst().get().getManeuvers();
            Optional<Maneuver> maneuver = event.getRoute().getSections().stream().findFirst().get().getManeuvers().stream().findFirst();
            if (maneuver != null) {
                maneuverImg.setImageDrawable(getResources().getDrawable(Utils.getManeuverImageWhite(maneuver.get())));
                maneuverActionTV.setText(Utils.getActionText(maneuver.get()));
                String cmDistanceUnits = "km";
                int cmDistance = Math.round(maneuver.get().getLengthInMeters() / 1000);
                if (cmDistance == 0) {
                    cmDistance = maneuver.get().getLengthInMeters();
                    cmDistanceUnits = "m";
                }
                maneuverDistanceTV.setText(cmDistance + " " + cmDistanceUnits);
                String nmDistanceUnits = "km";
                int nmDistance = Math.round(maneuvers.get(1).getLengthInMeters() / 1000);
                if (nmDistance == 0) {
                    nmDistance = maneuvers.get(1).getLengthInMeters();
                    nmDistanceUnits = "m";
                }
                newManeuverImg.setImageDrawable(getResources().getDrawable(Utils.getManeuverImageWhite(maneuvers.get(1))));
                newManeuverDistanceTV.setText(nmDistance + " " + nmDistanceUnits);
                newManeuverActionTV.setText(Utils.getActionText(maneuvers.get(1)));
            }

//            routeProgressSeekbar.setMax(event.getRoute().getLengthInMeters());
//            routeProgressSeekbar.setMax(event.getRoute().getSections().get(0).getSpans().size());
        }
    }

    @Subscribe(threadMode = ThreadMode.MAIN)
    public void onShowNavigationDialogEvent(ShowNavigationDialog event) {
        if (event != null) {
            LoadingDialog.dismissDialog(dialog);
        }
    }


    @Subscribe(threadMode = ThreadMode.MAIN)
    public void onGetSpeedLimitEvent(SpeedLimitEvent event) {
        if (event != null) {
            if (event.getSpeedLimit() != null && event.getSpeedLimit() > 0) {
                double speedLimit = Math.round(event.getSpeedLimit() * _kKMpHinMpS);
                speedLimitTV.setText(String.format("%s", speedLimit));
            } else {
                speedLimitTV.setText("N/A");
            }
        }
    }


    @Subscribe(threadMode = ThreadMode.MAIN)
    public void onGetNavigableLocationEvent(GetNavigableLocation event) {
        if (event != null) {
            double speed = Math.round(event.getNavigableLocation().originalLocation.speedInMetersPerSecond * _kKMpHinMpS);
            speedTV.setText(String.format("%s", speed));
            speedHeaderTV.setText("km/h");
            currentLocation = event.getNavigableLocation().originalLocation.coordinates;
            gpxRecorderExample.recordGPXWithLocation(event.getNavigableLocation().originalLocation);
        }
    }

    @Subscribe(threadMode = ThreadMode.MAIN)
    public void onGetLanesEvent(GetLanesEvent event) {
        if (event != null) {
            if (!event.getLanesList().isEmpty()) {
                lanesListRV.setHasFixedSize(true);
//                lanesListRV.setLayoutManager(new LinearLayoutManager(this, LinearLayoutManager.HORIZONTAL, false));
                FlexboxLayoutManager layoutManager = new FlexboxLayoutManager(this);
                layoutManager.setFlexDirection(FlexDirection.ROW);
                layoutManager.setJustifyContent(JustifyContent.CENTER);
                layoutManager.setFlexWrap(FlexWrap.WRAP);
                lanesListRV.setLayoutManager(layoutManager);
                lanesListRV.setAdapter(new LanesAdapter(this, event.getLanesList()));
                lanesListRV.setVisibility(View.VISIBLE);
            } else {
                lanesListRV.setVisibility(View.GONE);
            }
        }
    }

    @RequiresApi(api = Build.VERSION_CODES.N)
    @Subscribe(threadMode = ThreadMode.MAIN)
    public void onRouteProgressEvent(GetRouteProgressEvent event) {
        //Toast.makeText(this, "on progress", Toast.LENGTH_SHORT).show();
        Log.e("dmn", "on progress");

        if (event != null) {

            app.onUpdateRoute(event.visualNavigator.getRoute(), event.getRouteProgress());

            List<SectionProgress> sectionProgressList = event.getRouteProgress().sectionProgress;
            // sectionProgressList is guaranteed to be non-empty.
            SectionProgress lastSectionProgress = sectionProgressList.get(sectionProgressList.size() - 1);

            Log.d(TAG, "Distance to destination in meters: " + lastSectionProgress.remainingDistanceInMeters);
            Log.d(TAG, "Traffic delay ahead in seconds: " + lastSectionProgress.trafficDelay.getSeconds());

            int currentManeuverIndex = event.getCurrentManeuverProgress().maneuverIndex;
            int currentManeuverDistance = event.getCurrentManeuverProgress().remainingDistanceInMeters;
            int nextManeuverIndex = 0, nextManeuverDistance = 0;
            if (event.getNextManeuverProgress() != null) {
                nextManeuverIndex = event.getNextManeuverProgress().maneuverIndex;
                nextManeuverDistance = event.getNextManeuverProgress().remainingDistanceInMeters;
            }
            // Contains the progress for the next maneuver ahead and the next-next maneuvers, if any.
            List<ManeuverProgress> nextManeuverList = event.getRouteProgress().maneuverProgress;


//            if (nextManeuverList.size()!=0) {
//                currentManeuverIndex = nextManeuverList.get(0).maneuverIndex;
//                currentManeuverDistance = nextManeuverList.get(0).remainingDistanceInMeters;
//
//                if (nextManeuverList.size() > 1) {
//                    nextManeuverIndex = nextManeuverList.get(1).maneuverIndex;
//                    nextManeuverDistance = nextManeuverList.get(1).remainingDistanceInMeters;
//                }
//            }

            ManeuverProgress nextManeuverProgress = nextManeuverList.get(0);
            if (nextManeuverProgress == null) {
                Log.d(TAG, "No next maneuver available.");
                return;
            }

//            if (nextManeuver == null) {
//                // Should never happen as we retrieved the next maneuver progress above.
//                return;
//            }

//            ManeuverAction action = nextManeuver.getAction();
//            String roadName = Utils.getRoadName(nextManeuver);
//            String logMessage = action.name() + " on " + roadName +
//                    " in " + nextManeuverProgress.remainingDistanceInMeters + " meters.";

//            if (previousManeuverIndex != nextManeuverIndex) {
//                newManeuverLayout.setVisibility(View.VISIBLE);
//                messageView.setText("New maneuver: " + logMessage);
//
//
//            } else {
//                newManeuverLayout.setVisibility(View.GONE);
//                // A maneuver update contains a different distance to reach the next maneuver.
//                messageView.setText("Maneuver update: " + logMessage);
//            }

//            previousManeuverIndex = nextManeuverIndex;


//            DateTime dt = DateTime.now();
//            DateTime dtArrival = dt.add(Duration(seconds: remainingDurationInSeconds));
//


            int remainingHours = Math.round(lastSectionProgress.remainingDuration.getSeconds() / 3600);
            int remainingMinutes = Math.round((lastSectionProgress.remainingDuration.getSeconds() - remainingHours * 3600) / 60);
//
            String remainingDistanceUnits = "km";
            int remainingDistance = Math.round(lastSectionProgress.remainingDistanceInMeters / 1000);
            if (remainingDistance == 0) {
                remainingDistance = lastSectionProgress.remainingDistanceInMeters;
                remainingDistanceUnits = "m";
            }

            String cmDistanceUnits = "km";
            int cmDistance = Math.round(currentManeuverDistance / 1000);
            if (cmDistance == 0) {
                cmDistance = currentManeuverDistance;
                cmDistanceUnits = "m";
            }

            if (currentManeuverDistance < _kDistanceToShowNextManeuver) {
                newManeuverLayout.setVisibility(View.VISIBLE);
            } else {
                newManeuverLayout.setVisibility(View.GONE);
            }

            String nmDistanceUnits = "km";
            int nmDistance = Math.round(nextManeuverDistance / 1000);
            if (nmDistance == 0) {
                nmDistance = nextManeuverDistance;
                nmDistanceUnits = "m";
            }


            Calendar now = Calendar.getInstance();

            Calendar tmp = (Calendar) now.clone();
            tmp.add(Calendar.SECOND, Integer.parseInt(lastSectionProgress.remainingDuration.getSeconds() + ""));
            SimpleDateFormat dateFormatter = new SimpleDateFormat("HH:mm", Locale.getDefault());
            arrivalTV.setText(dateFormatter.format(tmp.getTime()));
            hourTV.setText(remainingHours + "");
            minTV.setText(remainingMinutes + "");
            kmTV.setText(remainingDistance + "");
            kmHeaderTV.setText(remainingDistanceUnits);
            maneuverImg.setImageDrawable(getResources().getDrawable(Utils.getManeuverImageWhite(event.getVisualNavigator().getManeuver(currentManeuverIndex))));
            maneuverDistanceTV.setText(cmDistance + " " + cmDistanceUnits);
            maneuverActionTV.setText(Utils.getActionText(event.getVisualNavigator().getManeuver(currentManeuverIndex)));
            newManeuverDistanceTV.setText(nmDistance + " " + nmDistanceUnits);
            if (event.getNextManeuverProgress() != null) {
                newManeuverImg.setImageDrawable(getResources().getDrawable(Utils.getManeuverImageWhite(event.getVisualNavigator().getManeuver(nextManeuverIndex))));
                newManeuverActionTV.setText(Utils.getActionText(event.getVisualNavigator().getManeuver(nextManeuverIndex)));
            }
//            routeProgressSeekbar.setMax(navigationRoute.getLengthInMeters());
//            routeProgressSeekbar.setProgress(navigationRoute.getLengthInMeters() - lastSectionProgress.remainingDistanceInMeters);
//            routeProgressSeekbar.setOnSeekBarChangeListener(new SeekBar.OnSeekBarChangeListener() {
//                @Override
//                public void onProgressChanged(SeekBar seekBar, int i, boolean b) {
//                    Log.e(TAG, "Progress: "+i);
//                }
//
//                @Override
//                public void onStartTrackingTouch(SeekBar seekBar) {
//
//                }
//
//                @Override
//                public void onStopTrackingTouch(SeekBar seekBar) {
//
//                }
//            });
        }
    }

//    private String getRoadName(Maneuver maneuver) {
//        RoadTexts currentRoadTexts = maneuver.getRoadTexts();
//        RoadTexts nextRoadTexts = maneuver.getNextRoadTexts();
//
//        String currentRoadName = currentRoadTexts.names.getDefaultValue();
//        String currentRoadNumber = currentRoadTexts.numbers.getDefaultValue();
//        String nextRoadName = nextRoadTexts.names.getDefaultValue();
//        String nextRoadNumber = nextRoadTexts.numbers.getDefaultValue();
//
//        String roadName = nextRoadName == null ? nextRoadNumber : nextRoadName;
//
//        // On highways, we want to show the highway number instead of a possible road name,
//        // while for inner city and urban areas road names are preferred over road numbers.
//        if (maneuver.getNextRoadType() == RoadType.HIGHWAY) {
//            roadName = nextRoadNumber == null ? nextRoadName : nextRoadNumber;
//        }
//
//        if (maneuver.getAction() == ManeuverAction.ARRIVE) {
//            // We are approaching the destination, so there's no next road.
//            roadName = currentRoadName == null ? currentRoadNumber : currentRoadName;
//        }
//
//        if (roadName == null) {
//            // Happens only in rare cases, when also the fallback is null.
//            roadName = "unnamed road";
//        }
//
//        return roadName;
//    }

    public void showStartNavigationDialog() {
        String simulatedText = "Start navigation (simulated)";
        String deviceLocText = "Start navigation (device location)";
        AlertDialog.Builder builder = new AlertDialog.Builder(NavigationActivity.this);
        builder.setTitle("Route Details")
                .setMessage("Select your navigation type")
                .setPositiveButton(simulatedText,
                        (dialog, which) -> {
                            app.isSimulated = true;
                            app.fromShare = getIntent().getBooleanExtra("fromShare",false);
                            app.addRouteSimulatedLocation(startPlace, destinationPlace, selectedOption, isTrafficOn);
                        })
                .setNeutralButton(deviceLocText,
                        (dialog, which) -> {
                            app.isSimulated = false;
                            app.fromShare = getIntent().getBooleanExtra("fromShare",false);
                            app.addRouteDeviceLocation(startPlace, destinationPlace, selectedOption, isTrafficOn);
                        })
                .show();
        EventBus.getDefault().post(new ShowNavigationDialog());
    }

    @Override
    public void onSensorChanged(SensorEvent event) {
        if (event.sensor.getType() == Sensor.TYPE_ROTATION_VECTOR) {
            // Convert the rotation vector to a rotation matrix
            SensorManager.getRotationMatrixFromVector(mRotationMatrix, event.values);
            // Convert the rotation matrix to orientation angles
            SensorManager.getOrientation(mRotationMatrix, mOrientation);
            // Convert the azimuth angle to degrees
            float azimuthInRadians = mOrientation[0];
            float azimuthInDegrees = (float) Math.toDegrees(azimuthInRadians);

            // Rotate the compass image
            if (!isCompassOn) {
                compassFab.setRotation(-azimuthInDegrees);
//                MapMeasure mapMeasureZoom = new MapMeasure(MapMeasure.Kind.ZOOM_LEVEL, 75);
                GeoOrientationUpdate orientationUpdate = new GeoOrientationUpdate((double) -azimuthInDegrees, 0.0);
                if (currentLocation != null) {
                    if (app.geoOrientation == null) {
                        mapView.getCamera().startAnimation(MapCameraAnimationFactory.flyTo(new GeoCoordinatesUpdate(currentLocation), orientationUpdate, cameraZoom == 0 ? 300 : 0, Duration.ofMillis(1)));
                        cameraZoom = 300;
                    }
                }
            }
        }
    }

    @Override
    public void onAccuracyChanged(Sensor sensor, int accuracy) {
        // Do nothing
    }

    @Override
    public void onMapCameraUpdated(@NonNull MapCamera.State state) {
        // Rotate the compass image
        if (isCompassOn) {
            compassFab.setRotation((float) state.orientationAtTarget.bearing);
        }
    }
}
