package com.example.heresdkandroidref.activity;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.appcompat.app.AppCompatActivity;
import androidx.appcompat.widget.Toolbar;
import androidx.viewpager2.widget.ViewPager2;

import android.content.Context;
import android.content.Intent;
import android.net.ConnectivityManager;
import android.os.Bundle;
import android.view.View;

import com.example.heresdkandroidref.R;
import com.example.heresdkandroidref.adapters.PreferenceAdapter;
import com.example.heresdkandroidref.utils.Constant;
import com.example.heresdkandroidref.utils.SharedPref;
import com.example.heresdkandroidref.utils.Utils;
import com.google.android.material.tabs.TabLayout;
import com.google.android.material.tabs.TabLayoutMediator;

public class PreferencesActivity extends AppCompatActivity {

    TabLayout tabLayoutTL;
    ViewPager2 viewPagerVP;
    Toolbar toolbarTB;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_preferences);
        initViews();
    }

    private void initViews() {
        toolbarTB = findViewById(R.id.toolbarTB);
        viewPagerVP = findViewById(R.id.viewPagerVP);
        tabLayoutTL = findViewById(R.id.tabLayoutTL);

        toolbarTB.setTitle("Preferences");
        setSupportActionBar(toolbarTB);
        getSupportActionBar().setDisplayHomeAsUpEnabled(true);
        getSupportActionBar().setDisplayShowHomeEnabled(true);
        //setup view pager
        viewPagerVP.setAdapter(new PreferenceAdapter(this));
        new TabLayoutMediator(tabLayoutTL, viewPagerVP, (tab, position) -> {
            if (isDeviceConnected()) {
                tab.setIcon(Constant.tabIcons[position]);
            } else {
                tab.setIcon(Constant.tabOfflineIcons[position]);
            }
        }).attach();
    }

    @Override
    public boolean onSupportNavigateUp() {
        onBackPressed();
        return true;
    }

    @Override
    public void onBackPressed() {
        Utils.disableSoftKeyboard(this);
        Intent intent = new Intent();
        setResult(RESULT_OK,intent);
        finish();
    }

    @Override
    protected void onActivityResult(int requestCode, int resultCode, @Nullable Intent data) {
        super.onActivityResult(requestCode, resultCode, data);
    }

    public boolean isDeviceConnected() {
        // An application may define here a logic to determine whether a device is connected or not.
        // For this example app, the flag is set from UI.
//        return isDeviceConnected && !isOfflineModeEnabled;
        ConnectivityManager connectivityManager = ((ConnectivityManager) getSystemService(Context.CONNECTIVITY_SERVICE));
        boolean isDeviceConnected = connectivityManager.getActiveNetworkInfo() != null && connectivityManager.getActiveNetworkInfo().isConnected();
        return isDeviceConnected && !SharedPref.getInstance().getOfflineMode();
    }
}