
package com.example.heresdkandroidref.activity;

import android.content.Intent;
import android.os.Bundle;
import android.util.Log;
import android.view.View;
import android.widget.LinearLayout;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.appcompat.app.AppCompatActivity;
import androidx.core.content.ContextCompat;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import com.example.heresdkandroidref.R;
import com.example.heresdkandroidref.adapters.ManeuversAdapter;

import com.example.heresdkandroidref.features.Positioning;
import com.example.heresdkandroidref.features.Routing;
import com.example.heresdkandroidref.interfaces.ManeuverItemClickListener;
import com.example.heresdkandroidref.models.ManeuverModel;
import com.example.heresdkandroidref.utils.LoadingDialog;
import com.example.heresdkandroidref.utils.PermissionsRequestor;
import com.example.heresdkandroidref.utils.SharedPref;
import com.example.heresdkandroidref.utils.Utils;
import com.google.android.material.bottomsheet.BottomSheetBehavior;
import com.google.gson.Gson;
import com.here.sdk.core.Color;
import com.here.sdk.core.GeoBox;
import com.here.sdk.core.GeoCoordinates;
import com.here.sdk.core.GeoOrientationUpdate;
import com.here.sdk.core.GeoPolyline;
import com.here.sdk.mapview.LineCap;
import com.here.sdk.mapview.MapError;
import com.here.sdk.mapview.MapImage;
import com.here.sdk.mapview.MapImageFactory;
import com.here.sdk.mapview.MapMarker;
import com.here.sdk.mapview.MapMeasure;
import com.here.sdk.mapview.MapMeasureDependentRenderSize;
import com.here.sdk.mapview.MapPolyline;
import com.here.sdk.mapview.MapScene;
import com.here.sdk.mapview.MapScheme;
import com.here.sdk.mapview.MapView;
import com.here.sdk.mapview.RenderSize;
import com.here.sdk.routing.Maneuver;
import com.here.sdk.routing.Route;
import com.here.sdk.routing.Section;

import java.util.ArrayList;
import java.util.List;

public class RouteDetailsActivity extends AppCompatActivity implements ManeuverItemClickListener {

    private static final String TAG = RouteDetailsActivity.class.getSimpleName();
    static final double _kZoomDistanceToManeuver = 500;
    private PermissionsRequestor permissionsRequestor;
    private MapView mapView;
    private Routing routing;
    Positioning positioning;
    GeoCoordinates destinationPlace;
    GeoCoordinates startPlace;
    String startName, destinationName;
    private LoadingDialog dialog;
    boolean isTrafficOn = false;
    ArrayList<Route> mapRoutes = new ArrayList<>();
    ArrayList<MapPolyline> mapPolylines = new ArrayList<>();
    int _selectedRouteIndex = 0;
    private BottomSheetBehavior maneuversSheetBehavior;
    RecyclerView maneuverRv;
    ArrayList<ManeuverModel> maneuverModelArrayList = new ArrayList<>();
    ManeuversAdapter maneuversAdapter;
    List<MapMarker> maneuverMarkers = new ArrayList<>();
    String selectedOption = "";
    Route selectedRoute;
    public TextView itemTime, itemDelay, itemDistance;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_route_details);


        initViews(savedInstanceState);
        setSheetBehaviorListener();

        handleAndroidPermissions();
    }

    private void initViews(Bundle savedInstanceState) {
        // Get a MapView instance from layout.
        mapView = findViewById(R.id.map_view);
        mapView.onCreate(savedInstanceState);
//        pickupTv =  findViewById(R.id.pickup_txt);
//        dropOffTv =  findViewById(R.id.dropoff_txt);
        maneuverRv = findViewById(R.id.maneuver_rv);
        itemTime = findViewById(R.id.time_txt);

        itemDelay = findViewById(R.id.delay_txt);
        itemDistance = findViewById(R.id.new_maneuver_distance_txt);


        LinearLayout poiLayoutBottomSheet = findViewById(R.id.maneuver_sheet);

        maneuversSheetBehavior = BottomSheetBehavior.from(poiLayoutBottomSheet);

        getDetails();

    }

    private void getDetails() {
        try {
            Gson gson = new Gson();
            destinationPlace = gson.fromJson(getIntent().getStringExtra("destination"), GeoCoordinates.class);
            startPlace = gson.fromJson(getIntent().getStringExtra("start"), GeoCoordinates.class);
            startName = getIntent().getStringExtra("startName");
            destinationName = getIntent().getStringExtra("destinationName");
            selectedRoute = gson.fromJson(SharedPref.getInstance().getRouteData(), Route.class);
            selectedOption = getIntent().getStringExtra("selected_option");

            itemTime.setText(Utils.makeDurationString(selectedRoute.getDuration().getSeconds()));

            if (selectedRoute.getTrafficDelay().getSeconds() > 60) {
                itemDelay.setText("Incl. " + Utils.makeDurationString(selectedRoute.getTrafficDelay().getSeconds()) + " delay");
            } else {
                itemDelay.setText("No delays");
            }

            itemDistance.setText(Utils.makeDistanceString(selectedRoute.getLengthInMeters()));
            this.dialog = new LoadingDialog(this);
            LoadingDialog.showDialog(dialog);
        } catch (Exception exception) {
            Log.e("Exception", exception.getMessage());
        }

    }

    private void setAdapter() {

        maneuversAdapter = new ManeuversAdapter(this, maneuverModelArrayList, null);
        maneuversAdapter.setItemClickListener(this);
        maneuverRv.setLayoutManager(new LinearLayoutManager(this, LinearLayoutManager.VERTICAL, false));
        maneuverRv.setAdapter(maneuversAdapter);
    }

    public void setSheetBehaviorListener() {
        maneuversSheetBehavior.setBottomSheetCallback(new BottomSheetBehavior.BottomSheetCallback() {
            @Override
            public void onStateChanged(@NonNull View bottomSheet, int newState) {
                switch (newState) {
                    case BottomSheetBehavior.STATE_HIDDEN:
                        break;
                    case BottomSheetBehavior.STATE_EXPANDED:

                        break;
                    case BottomSheetBehavior.STATE_COLLAPSED: {
//                        btnBottomSheet.setText(R.string.expand);
                    }
                    break;
                    case BottomSheetBehavior.STATE_DRAGGING:
                    case BottomSheetBehavior.STATE_SETTLING:
                        maneuversSheetBehavior.setHideable(false);
                        break;
                }
            }

            @Override
            public void onSlide(@NonNull View bottomSheet, float slideOffset) {

            }
        });
    }


    private void handleAndroidPermissions() {
        permissionsRequestor = new PermissionsRequestor(this);
        permissionsRequestor.request(new PermissionsRequestor.ResultListener() {

            @Override
            public void permissionsGranted() {
                loadMapScene();
            }

            @Override
            public void permissionsDenied() {
                Log.e(TAG, "Permissions denied by user.");
            }
        });
    }

    @Override
    public void onRequestPermissionsResult(int requestCode, @NonNull String[] permissions, @NonNull int[] grantResults) {
        super.onRequestPermissionsResult(requestCode, permissions, grantResults);
        permissionsRequestor.onRequestPermissionsResult(requestCode, grantResults);
    }

    private void loadMapScene() {
        mapView.getMapScene().loadScene(MapScheme.NORMAL_DAY, new MapScene.LoadSceneCallback() {
            @Override
            public void onLoadScene(@Nullable MapError mapError) {
                if (mapError == null) {

                    positioning = new Positioning();
                    routing = new Routing(RouteDetailsActivity.this, mapView);
//                    mapView.getCamera().lookAt(
//                            currentLoc, PositioningExample.CAMERA_DISTANCE_TO_EARTH);
                    try {
                        addRouteToMap(selectedRoute);
                    } catch (Exception ex) {
                        Log.e(TAG, "onLoadScene Error" + ex.getMessage());
                        ex.printStackTrace();
                    }


                } else {
                    Log.d(TAG, "Loading map failed: mapErrorCode: " + mapError.name());
                }
            }
        });
    }

//    public void addRouteButtonClicked(View view) {
//        routing.addRoute();
//    }

    public void addWaypointsButtonClicked(View view) {
        routing.addWaypoints(null, "car");
    }

    public void clearMapButtonClicked(View view) {
        routing.clearMap();
    }

    @Override
    protected void onPause() {
        super.onPause();
        mapView.onPause();
    }

    @Override
    protected void onResume() {
        super.onResume();
        mapView.onResume();
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        mapView.onDestroy();
    }


    public void closeBtn(View view) {
        finish();
    }

    private void zoomToManeuver(int index) {
        ArrayList<GeoCoordinates> bounds = new ArrayList<>();

        for (int i = 0; i < mapRoutes.size(); ++i) {
            GeoBox geoBox = mapRoutes.get(i).getBoundingBox();
            bounds.add(geoBox.northEastCorner);
            bounds.add(geoBox.southWestCorner);
        }
        final GeoBox geoBox = GeoBox.containing(bounds);
        GeoOrientationUpdate geoOrientationUpdate = new GeoOrientationUpdate(0.0, 0.0);

        assert geoBox != null;
        MapMeasure mapMeasureZoom = new MapMeasure(MapMeasure.Kind.DISTANCE, _kZoomDistanceToManeuver);
        mapView.getCamera().lookAt(maneuverModelArrayList.get(index).getManeuver().getCoordinates(), mapMeasureZoom);
//        _hasBeenZoomedToManeuver = true;
//        mapView.getCamera().zoomTo(14);
    }

    private void zoomToWholeRoute() {
        ArrayList<GeoCoordinates> bounds = new ArrayList<>();
        GeoBox geoBoxs = selectedRoute.getBoundingBox();
        bounds.add(geoBoxs.northEastCorner);
        bounds.add(geoBoxs.southWestCorner);
        final GeoBox geoBox = GeoBox.containing(bounds);
        GeoOrientationUpdate geoOrientationUpdate = new GeoOrientationUpdate(0.0, 0.0);

        assert geoBox != null;
        mapView.getCamera().lookAt(geoBox, geoOrientationUpdate);
    }

    private void addRouteToMap(Route route) {
        GeoPolyline routeGeoPolyline = null;
        try {
            routeGeoPolyline = route.getGeometry();
            MapPolyline routeMapPolyline;

            routeMapPolyline = new MapPolyline(
                    routeGeoPolyline, new MapPolyline.SolidRepresentation(new MapMeasureDependentRenderSize(RenderSize.Unit.PIXELS, 20.0), Color.valueOf(ContextCompat.getColor(this, R.color.selectedRouteColor)), new MapMeasureDependentRenderSize(RenderSize.Unit.PIXELS, 5), Color.valueOf(ContextCompat.getColor(this, R.color.selectedRouteBorderColor)), LineCap.ROUND));
            routeMapPolyline.setDrawOrder(1);

            mapView.getMapScene().addMapPolyline(routeMapPolyline);
            mapPolylines.add(routeMapPolyline);
            _addMarkersForManeuver(route);
            zoomToWholeRoute();
            setAdapter();
            LoadingDialog.dismissDialog(dialog);
        } catch (Exception e) {
            e.printStackTrace();
        }

    }

    void showPoiSheet() {
        maneuversSheetBehavior.setState(BottomSheetBehavior.STATE_EXPANDED);

    }

    void hidePoiSheet() {
        maneuversSheetBehavior.setState(BottomSheetBehavior.STATE_COLLAPSED);
    }


    @Override
    public void onManeuverItemClick(View view, int position) {
//        setSelectedPoi();
//        updatePoiForRoute(mapRoutes.get(_selectedRouteIndex));
    }


    void updatePoiForRoute(Route route) {
        _clearMarkers();


    }


    void _clearMarkers() {
        if (maneuverMarkers != null) {
            for (MapMarker marker : maneuverMarkers) {
                mapView.getMapScene().removeMapMarker(marker);
            }
            maneuverMarkers.clear();
        }

    }

    void _addMarkersForManeuver(Route route) {
        for (Section section : route.getSections()) {
            for (Maneuver maneuver : section.getManeuvers()
            ) {
                MapImage mapImage = MapImageFactory.fromResource(getResources(), R.drawable.maneuver);
                MapMarker mapMarker = new MapMarker(maneuver.getCoordinates(), mapImage);
                mapView.getMapScene().addMapMarker(mapMarker);
                maneuverMarkers.add(mapMarker);
                maneuverModelArrayList.add(new ManeuverModel(maneuver, maneuver.getAction().value, maneuver.getText()));
            }
        }
    }

    public void backBtnClick(View view) {
        finish();
    }

    public void navigationBtnClick(View view) {
        try {
            Gson gson = new Gson();
            String start = gson.toJson(startPlace);
            String destination = gson.toJson(destinationPlace);
            SharedPref.getInstance().saveRouteData(gson.toJson(selectedRoute));

            Intent intent = new Intent(this, NavigationActivity.class);
            intent.putExtra("start", start);
            intent.putExtra("destination", destination);
            intent.putExtra("startName", startName);
            intent.putExtra("destinationName", destinationName);
            intent.putExtra("selected_option", selectedOption);
            intent.putExtra("fromShare", getIntent().getBooleanExtra("fromShare", false));
            startActivity(intent);
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }
}
