
package com.example.heresdkandroidref.activity;

import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.net.ConnectivityManager;
import android.os.AsyncTask;
import android.os.Bundle;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.appcompat.app.AlertDialog;
import androidx.appcompat.app.AppCompatActivity;
import androidx.appcompat.widget.SearchView;
import androidx.core.content.ContextCompat;
import androidx.recyclerview.widget.ItemTouchHelper;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import android.os.Handler;
import android.util.Log;
import android.view.Gravity;
import android.view.View;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.TextView;
import android.widget.Toast;

import com.example.heresdkandroidref.R;
import com.example.heresdkandroidref.adapters.PoiAdapter;
import com.example.heresdkandroidref.adapters.RouteDetailsAdapter;
import com.example.heresdkandroidref.adapters.SuggestionsAdapter;
import com.example.heresdkandroidref.adapters.WaypointsAdapter;
import com.example.heresdkandroidref.events.GetDetailsEvent;
import com.example.heresdkandroidref.events.GetPlacesEvent;
import com.example.heresdkandroidref.features.Search;
import com.example.heresdkandroidref.interfaces.PoiItemClickListener;
import com.example.heresdkandroidref.interfaces.RouteItemClickListener;
import com.example.heresdkandroidref.interfaces.SearchedItemClickListener;
import com.example.heresdkandroidref.interfaces.WaypointItemClickListener;
import com.example.heresdkandroidref.models.MarkerForPlace;
import com.example.heresdkandroidref.models.PlaceCoordinates;
import com.example.heresdkandroidref.models.PlaceData;
import com.example.heresdkandroidref.models.PoiModel;
import com.example.heresdkandroidref.models.TrafficLineColor;
import com.example.heresdkandroidref.utils.Constant;
import com.example.heresdkandroidref.utils.LoadingDialog;
import com.example.heresdkandroidref.utils.PermissionsRequestor;
import com.example.heresdkandroidref.features.Positioning;
import com.example.heresdkandroidref.features.Routing;
import com.example.heresdkandroidref.utils.SharedPref;
import com.example.heresdkandroidref.utils.Utils;
import com.google.android.material.bottomsheet.BottomSheetBehavior;
import com.google.android.material.floatingactionbutton.FloatingActionButton;
import com.google.gson.Gson;
import com.here.sdk.core.Anchor2D;
import com.here.sdk.core.Color;
import com.here.sdk.core.GeoBox;
import com.here.sdk.core.GeoCoordinates;
import com.here.sdk.core.GeoCorridor;
import com.here.sdk.core.GeoOrientationUpdate;
import com.here.sdk.core.GeoPolyline;
import com.here.sdk.core.LanguageCode;
import com.here.sdk.core.Metadata;
import com.here.sdk.core.errors.InstantiationErrorException;
import com.here.sdk.core.threading.TaskHandle;
import com.here.sdk.mapview.LineCap;
import com.here.sdk.mapview.MapFeatureModes;
import com.here.sdk.mapview.MapFeatures;
import com.here.sdk.mapview.MapImage;
import com.here.sdk.mapview.MapImageFactory;
import com.here.sdk.mapview.MapMarker;
import com.here.sdk.mapview.MapMeasure;
import com.here.sdk.mapview.MapMeasureDependentRenderSize;
import com.here.sdk.mapview.MapPolyline;
import com.here.sdk.mapview.MapScene;
import com.here.sdk.mapview.MapScheme;
import com.here.sdk.mapview.MapView;
import com.here.sdk.mapview.RenderSize;
import com.here.sdk.mapview.VisibilityState;
import com.here.sdk.routing.Route;
import com.here.sdk.routing.Section;
import com.here.sdk.routing.Span;
import com.here.sdk.routing.TrafficSpeed;
import com.here.sdk.routing.Waypoint;
import com.here.sdk.search.CategoryQuery;
import com.here.sdk.search.Place;
import com.here.sdk.search.PlaceCategory;
import com.here.sdk.search.PlaceSerializationException;
import com.here.sdk.search.SearchOptions;

import org.greenrobot.eventbus.EventBus;
import org.greenrobot.eventbus.Subscribe;
import org.greenrobot.eventbus.ThreadMode;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class RoutingActivity extends AppCompatActivity implements SearchedItemClickListener, PoiItemClickListener, RouteItemClickListener, Search.MarkerDetailListener, View.OnClickListener, WaypointItemClickListener {

    private static final String TAG = RoutingActivity.class.getSimpleName();
    private PermissionsRequestor permissionsRequestor;
    private MapView mapView;
    private Routing routing;
    private Positioning positioning;
    private Search search;
    private FloatingActionButton resetLocFab;
    private Route selectedRoute;
    private TextView pickupTv, no_waypoints;
    private TextView dropOffTv;
    private RecyclerView detailsRv;
    private ImageView trafficImg;
    private ImageView eatPoiImg;
    private ImageView fuelPoiImg;
    private ImageView atmPoiImg;
    private View carDivider, truckDivider, scooterDivider, walkDivider;
    private String selectedOption = "car";
    private GeoCoordinates startPlace;
    private GeoCoordinates destinationPlace;
    private LoadingDialog dialog;
    private boolean isTrafficOn = false;
    private ArrayList<Route> mapRoutes = new ArrayList<>();
    private final ArrayList<MapPolyline> mapPolylines = new ArrayList<>();
    private ArrayList<MapPolyline> trafficPolylines = null;
    private int _selectedRouteIndex = 0;
    private BottomSheetBehavior poiSheetBehavior;
    private BottomSheetBehavior searchSheetBehavior;
    private BottomSheetBehavior waypointSheetBehavior;
    private RecyclerView poiRv;
    private final ArrayList<PoiModel> poiModelArrayList = new ArrayList<>();
    //    private List<String> _categories;
//    private final List<PlacesForRoute> _placesForRoutes = new ArrayList<>();
    private final List<MarkerForPlace> _markers = new ArrayList<>();
    private final List<View> _markersPinViews = new ArrayList<>();
    private final List<PlaceCategory> categories = new ArrayList<>();
    private RecyclerView suggestionsRv, waypointsRv;
    private final ArrayList<PlaceData> placesSuggestions = new ArrayList<>();
    private final ArrayList<PlaceCoordinates> waypointsSuggestions = new ArrayList<>();
    private TaskHandle _poiSearchTask;
    private SearchView searchView;
    private TextView headerTxt;
    private boolean isPickup = false;
    private boolean isDestination = false;
    private boolean isWaypoint = false;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_routing);
        initViews(savedInstanceState);
        setSheetBehaviorListener();
        searchListeners();
        setAdapter();
        handleAndroidPermissions();
    }

    private void initViews(Bundle savedInstanceState) {
        // Get a MapView instance from layout.
        mapView = findViewById(R.id.map_view);
        mapView.onCreate(savedInstanceState);
        resetLocFab = findViewById(R.id.resetLocationFab);
        pickupTv = findViewById(R.id.pickup_txt);
        dropOffTv = findViewById(R.id.dropoff_txt);
        TextView preference_txt = findViewById(R.id.preference_txt);
        detailsRv = findViewById(R.id.place_detail_rv);
        poiRv = findViewById(R.id.poi_rv);
//        LinearLayout carOption = findViewById(R.id.car_type);
//        LinearLayout truckOption = findViewById(R.id.truck_type);
        LinearLayout scooterOption = findViewById(R.id.scooter_type);
        LinearLayout walkOption = findViewById(R.id.walk_type);
        preference_txt.setOnClickListener(this);

//        ImageView carImg = findViewById(R.id.car_img);
//        ImageView truckImg = findViewById(R.id.truck_img);
//        ImageView scooterImg = findViewById(R.id.scooter_img);
//        ImageView walkImg = findViewById(R.id.walk_img);
        trafficImg = findViewById(R.id.traffic_img);

        eatPoiImg = findViewById(R.id.eat_img);
        fuelPoiImg = findViewById(R.id.fueling_img);
        atmPoiImg = findViewById(R.id.atm_img);
        no_waypoints = findViewById(R.id.no_waypoints);
//        ImageView swapImg = findViewById(R.id.swap_img);

        carDivider = findViewById(R.id.car_divider);
        truckDivider = findViewById(R.id.truck_divider);
        scooterDivider = findViewById(R.id.scooter_divider);
        walkDivider = findViewById(R.id.walk_divider);
        LinearLayout poiLayoutBottomSheet = findViewById(R.id.poi_sheet);
        poiSheetBehavior = BottomSheetBehavior.from(poiLayoutBottomSheet);
        LinearLayout searchLayoutBottomSheet = findViewById(R.id.search_bottom_sheet);
        searchSheetBehavior = BottomSheetBehavior.from(searchLayoutBottomSheet);
        LinearLayout waypointLayoutBottomSheet = findViewById(R.id.waypoint_bottom_sheet);
        waypointSheetBehavior = BottomSheetBehavior.from(waypointLayoutBottomSheet);
        suggestionsRv = findViewById(R.id.rvplaces);
        waypointsRv = findViewById(R.id.rvWaypoints);
        searchView = findViewById(R.id.searchview);
        headerTxt = findViewById(R.id.txt_header);
        if (isDeviceConnected()) {
            scooterOption.setVisibility(View.VISIBLE);
            walkOption.setVisibility(View.VISIBLE);
        } else {
            scooterOption.setVisibility(View.VISIBLE);
            walkOption.setVisibility(View.VISIBLE);
        }
        getDetails();

    }

    private void getDetails() {
        Gson gson = new Gson();
        String start = getIntent().getStringExtra("origin");
        String destination = getIntent().getStringExtra("destination");
        GeoCoordinates placeLocation = gson.fromJson(getIntent().getStringExtra("destinationLocation"), GeoCoordinates.class);
        GeoCoordinates currentLoc = gson.fromJson(getIntent().getStringExtra("currentLocation"), GeoCoordinates.class);

        pickupTv.setText(start);
        startPlace = currentLoc;
        dropOffTv.setText(destination);
        destinationPlace = placeLocation;
        this.dialog = new LoadingDialog(this);
        setupWaypointAdapter();
    }

    public void searchListeners() {
        searchView.setOnQueryTextListener(new SearchView.OnQueryTextListener() {
            @Override
            public boolean onQueryTextSubmit(String query) {
//                headerTxt.setText("Match Results");
//                Utils.disableSoftKeyboard(MainActivity.this);
                hideList();
                search.onSearchButtonClicked(query, true);
                return false;
            }

            @Override
            public boolean onQueryTextChange(String newText) {
                headerTxt.setText("Matching Results");
                hideList();
                if (search != null)
                    search.onSearchButtonClicked(newText, false);
                return false;
            }
        });
    }

    void showList() {
        suggestionsRv.setVisibility(View.VISIBLE);
    }

    void hideList() {
        suggestionsRv.setVisibility(View.GONE);
    }

    private void setAdapter() {
        poiModelArrayList.add(new PoiModel(PlaceCategory.EAT_AND_DRINK, R.drawable.eat_and_drink_icon, "Eat and Drink", false));
        poiModelArrayList.add(new PoiModel(PlaceCategory.BUSINESS_AND_SERVICES_FUELING_STATION, R.drawable.fueling_station_icon, "Fueling Stations", false));
        poiModelArrayList.add(new PoiModel(PlaceCategory.BUSINESS_AND_SERVICES_ATM, R.drawable.atm_icon, "ATM", false));
        PoiAdapter poiAdapter = new PoiAdapter(this, poiModelArrayList, null);
        poiAdapter.setItemClickListener(this);
        poiRv.setLayoutManager(new LinearLayoutManager(this, LinearLayoutManager.VERTICAL, false));
        poiRv.setAdapter(poiAdapter);
        //recent suggestions
        List<PlaceData> list = null;
        try {
            list = SharedPref.getInstance().getRecentPlacesList();
        } catch (PlaceSerializationException e) {
            e.printStackTrace();
        }
        if (list != null && !list.isEmpty()) {
            Utils.disableSoftKeyboard(this);
            showList();
            placesSuggestions.clear();
            for (PlaceData placeData : list) {
                if (placeData.getPlace() != null) placesSuggestions.add(placeData);
            }
            setSuggestionAdapter();
        }
    }

    private void setSuggestionAdapter() {
        SuggestionsAdapter suggestionAdapter = new SuggestionsAdapter(this, placesSuggestions, null);
        suggestionAdapter.setItemClickListener(this);
        suggestionsRv.setLayoutManager(new LinearLayoutManager(this, LinearLayoutManager.VERTICAL, false));
        suggestionsRv.setAdapter(suggestionAdapter);
    }

    private void setupWaypointAdapter() {
        WaypointsAdapter waypointAdapter = new WaypointsAdapter(this, waypointsSuggestions, this);
        waypointsRv.setLayoutManager(new LinearLayoutManager(this, LinearLayoutManager.VERTICAL, false));
        waypointsRv.setAdapter(waypointAdapter);
        ItemTouchHelper.SimpleCallback simpleCallback = new ItemTouchHelper.SimpleCallback(ItemTouchHelper.UP
                | ItemTouchHelper.DOWN | ItemTouchHelper.START | ItemTouchHelper.END, 0) {
            @Override
            public boolean onMove(@NonNull RecyclerView recyclerView, @NonNull RecyclerView.ViewHolder viewHolder, @NonNull RecyclerView.ViewHolder target) {

                int fromPosition = viewHolder.getAdapterPosition();
                int toPosition = target.getAdapterPosition();
                //swap waypoint suggestions
                Collections.swap(waypointsSuggestions, fromPosition, toPosition);
                recyclerView.getAdapter().notifyItemMoved(fromPosition, toPosition);
                recyclerView.getAdapter().notifyDataSetChanged();
                //swap waypoints
                routing.swapWaypoints(fromPosition, toPosition, selectedOption);
                //swap markers
                Collections.swap(search.mapMarkerList, fromPosition, toPosition);
                return false;
            }


            @Override
            public void onSwiped(@NonNull RecyclerView.ViewHolder viewHolder, int direction) {

            }
        };
        ItemTouchHelper itemTouchHelper = new ItemTouchHelper(simpleCallback);
        itemTouchHelper.attachToRecyclerView(waypointsRv);
    }

    public void setSheetBehaviorListener() {
        poiSheetBehavior.addBottomSheetCallback(new BottomSheetBehavior.BottomSheetCallback() {
            @Override
            public void onStateChanged(@NonNull View bottomSheet, int newState) {
                switch (newState) {
                    case BottomSheetBehavior.STATE_HIDDEN:
                    case BottomSheetBehavior.STATE_EXPANDED: {
//                        btnBottomSheet.setText(R.string.close);
                    }
                    break;
                    case BottomSheetBehavior.STATE_COLLAPSED: {
//                        btnBottomSheet.setText(R.string.expand);
                    }
                    break;
                    case BottomSheetBehavior.STATE_DRAGGING:
                    case BottomSheetBehavior.STATE_SETTLING:
                        poiSheetBehavior.setHideable(false);
                        break;
                    case BottomSheetBehavior.STATE_HALF_EXPANDED:
                        break;
                }
            }

            @Override
            public void onSlide(@NonNull View bottomSheet, float slideOffset) {

            }
        });
    }

    private void handleAndroidPermissions() {
        permissionsRequestor = new PermissionsRequestor(this);
        permissionsRequestor.request(new PermissionsRequestor.ResultListener() {

            @Override
            public void permissionsGranted() {
                loadMapScene();
            }

            @Override
            public void permissionsDenied() {
                Log.e(TAG, "Permissions denied by user.");
            }
        });
    }

    @Override
    public void onRequestPermissionsResult(int requestCode, @NonNull String[] permissions, @NonNull int[] grantResults) {
        super.onRequestPermissionsResult(requestCode, permissions, grantResults);
        permissionsRequestor.onRequestPermissionsResult(requestCode, grantResults);
    }

    private void loadMapScene() {
//        try {
//            mapView.getCamera().getLimits().setMaxTilt(45);
//        } catch (MapCameraLimits.MapCameraLimitsException e) {
//            e.printStackTrace();
//        }
        mapView.getMapScene().loadScene(MapScheme.NORMAL_DAY, mapError -> {
            if (mapError == null) {
                Map<String, String> map = new HashMap<>();
                map.put(MapFeatures.LANDMARKS, MapFeatureModes.LANDMARKS_TEXTURED);
                //map.put(MapFeatures.TRAFFIC_FLOW, MapFeatureModes.TRAFFIC_FLOW_WITH_FREE_FLOW);
                map.put(MapFeatures.VEHICLE_RESTRICTIONS, MapFeatureModes.VEHICLE_RESTRICTIONS_ACTIVE_AND_INACTIVE_DIFFERENTIATED);
                map.put(MapFeatures.SAFETY_CAMERAS, MapFeatureModes.SAFETY_CAMERAS_ALL);
                map.put(MapFeatures.EXTRUDED_BUILDINGS, MapFeatureModes.EXTRUDED_BUILDINGS_ALL);
                map.put(MapFeatures.BUILDING_FOOTPRINTS, MapFeatureModes.BUILDING_FOOTPRINTS_ALL);
                map.put(MapFeatures.ENVIRONMENTAL_ZONES, MapFeatureModes.ENVIRONMENTAL_ZONES_ALL);
                map.put(MapFeatures.TRAFFIC_INCIDENTS, MapFeatureModes.TRAFFIC_INCIDENTS_ALL);
                map.put(MapFeatures.CONGESTION_ZONES, MapFeatureModes.CONGESTION_ZONES_ALL);
                mapView.getMapScene().enableFeatures(map);
                positioning = new Positioning();
                positioning.onMapSceneLoaded(mapView, false, RoutingActivity.this);
                routing = new Routing(RoutingActivity.this, mapView);
                search = new Search(RoutingActivity.this, mapView, RoutingActivity.this);
//                    mapView.getCamera().lookAt(
//                            currentLoc, PositioningExample.CAMERA_DISTANCE_TO_EARTH);
                //get waypoints and setup with route
                ArrayList<String> waypointsList = getIntent().getStringArrayListExtra("waypoints");
                for (String place : waypointsList) {
                    PlaceCoordinates placeCoordinates = new Gson().fromJson(place, PlaceCoordinates.class);
                    Metadata metadata = new Metadata();
                    search.addPoiMapMarker(new Gson().fromJson(placeCoordinates.getGeoCoordinates(), GeoCoordinates.class), metadata);
                    routing.addOneWaypointToList(new Gson().fromJson(placeCoordinates.getGeoCoordinates(), GeoCoordinates.class));
                    waypointsSuggestions.add(placeCoordinates);
                }
                updateWaypointList();
                beginRoute();

            } else {
                Log.d(TAG, "Loading map failed: mapErrorCode: " + mapError.name());
            }
        });
    }

//    public void addRouteButtonClicked(View view) {
//        routing.addRoute();
//    }

    public void addWaypointsButtonClicked(View view) {
        waypointSheetBehavior.setState(BottomSheetBehavior.STATE_EXPANDED);
    }

    public void hideWaypointSheet(View view) {
        waypointSheetBehavior.setState(BottomSheetBehavior.STATE_COLLAPSED);
    }

    public void addNewWaypoint(View view) {
        isPickup = false;
        isDestination = false;
        isWaypoint = true;
        showSearchSheet();
    }

    public void clearMapButtonClicked(View view) {
        routing.clearMap();
    }

    @Override
    protected void onPause() {
        super.onPause();
        mapView.onPause();
    }

    @Override
    protected void onResume() {
        super.onResume();
        mapView.onResume();
    }

    @Override
    protected void onDestroy() {
        if (positioning != null) {
            positioning.stopLocating();
        }
        mapView.onDestroy();
        super.onDestroy();
    }

    public void resetLocationClicked(View view) {
        GeoCoordinates coordinates;
        if (positioning.getLocationEngine().getLastKnownLocation() == null) {
            Toast.makeText(this, "GPS is not enabled! using default location", Toast.LENGTH_SHORT).show();
            permissionsRequestor.request(new PermissionsRequestor.ResultListener() {

                @Override
                public void permissionsGranted() {

                }

                @Override
                public void permissionsDenied() {
                    showSimpleDialog("Error", "Cannot start app: Location service and permissions are needed for this app.");
                }
            });
            coordinates = Positioning.defaultCoordinates;
        } else {
            coordinates = positioning.getLocationEngine().getLastKnownLocation().coordinates;
        }
        MapMeasure mapMeasureZoom = new MapMeasure(MapMeasure.Kind.DISTANCE, Positioning.CAMERA_DISTANCE_TO_EARTH);
        mapView.getCamera().lookAt(coordinates, mapMeasureZoom);
//        MainActivity.enableMapUpdate = true;
        setCurrentLocVisibility();
    }

    private void showSimpleDialog(String title, String message) {
        AlertDialog.Builder builder = new AlertDialog.Builder(this);
        builder.setTitle(title)
                .setMessage(message)
                .show();
    }

    void setCurrentLocVisibility() {
        if (!MainActivity.enableMapUpdate) {
            resetLocFab.setVisibility(View.VISIBLE);
        } else {
            resetLocFab.setVisibility(View.GONE);
        }
    }

    public void swapBtn(View view) {
        GeoCoordinates tempStart = startPlace;
        String tempStartTxt = pickupTv.getText().toString();
        GeoCoordinates tempEnd = destinationPlace;
        String tempEndTxt = dropOffTv.getText().toString();
        pickupTv.setText(tempEndTxt);
        startPlace = tempEnd;
        destinationPlace = tempStart;
        dropOffTv.setText(tempStartTxt);
//        if (startPlace == currentLoc) {
//            pickupTv.setText(place.getTitle());
//            startPlace = place.getGeoCoordinates();
//            destinationPlace = currentLoc;
//            dropOffTv.setText("Your Location");
//        } else {
//            pickupTv.setText("Your Location");
//            startPlace = currentLoc;
//            destinationPlace = place.getGeoCoordinates();
//            dropOffTv.setText(place.getTitle());
//        }
        dialog = new LoadingDialog(this);
        //reset route
        beginRoute();
    }

    public void closeBtn(View view) {
        finish();
    }

    public void scooterOptionClicked(View view) {
        setSelectedOption("scooter");
    }

    public void walkOptionClicked(View view) {
        setSelectedOption("walk");
    }

    public void truckOptionClicked(View view) {
        setSelectedOption("truck");
    }

    public void carOptionClicked(View view) {
        setSelectedOption("car");
    }

    private void setSelectedOption(String option) {
        selectedOption = option;
        switch (option) {
            case "car":
                carDivider.setBackgroundColor(getResources().getColor(R.color.colorPrimary));
                truckDivider.setBackgroundColor(getResources().getColor(R.color.grey));
                scooterDivider.setBackgroundColor(getResources().getColor(R.color.grey));
                walkDivider.setBackgroundColor(getResources().getColor(R.color.grey));
                break;
            case "truck":
                truckDivider.setBackgroundColor(getResources().getColor(R.color.colorPrimary));
                carDivider.setBackgroundColor(getResources().getColor(R.color.grey));
                scooterDivider.setBackgroundColor(getResources().getColor(R.color.grey));
                walkDivider.setBackgroundColor(getResources().getColor(R.color.grey));
                break;
            case "scooter":
                scooterDivider.setBackgroundColor(getResources().getColor(R.color.colorPrimary));
                carDivider.setBackgroundColor(getResources().getColor(R.color.grey));
                truckDivider.setBackgroundColor(getResources().getColor(R.color.grey));
                walkDivider.setBackgroundColor(getResources().getColor(R.color.grey));
                break;
            case "walk":
                walkDivider.setBackgroundColor(getResources().getColor(R.color.colorPrimary));
                carDivider.setBackgroundColor(getResources().getColor(R.color.grey));
                scooterDivider.setBackgroundColor(getResources().getColor(R.color.grey));
                truckDivider.setBackgroundColor(getResources().getColor(R.color.grey));
                break;
        }
        beginRoute();
    }

    private void zoomRoutes() {
        ArrayList<GeoCoordinates> bounds = new ArrayList<>();

        for (int i = 0; i < mapRoutes.size(); ++i) {
            GeoBox geoBox = mapRoutes.get(i).getBoundingBox();
            bounds.add(geoBox.northEastCorner);
            bounds.add(geoBox.southWestCorner);
        }
        final GeoBox geoBox = GeoBox.containing(bounds);
        GeoOrientationUpdate geoOrientationUpdate = new GeoOrientationUpdate(0.0, 0.0);

        assert geoBox != null;
        mapView.getCamera().lookAt(geoBox, geoOrientationUpdate);
//        mapView.getCamera().zoomTo(14);
    }

    private void beginRoute() {
        LoadingDialog.showDialog(dialog);
        routing.addRoute(startPlace, destinationPlace, selectedOption);
    }

    @Subscribe(threadMode = ThreadMode.MAIN)
    public void onGetDetailEvent(GetDetailsEvent event) {
        if (event != null && event.getRoutesList() != null) {
            mapRoutes = event.getRoutesList();
            RouteDetailsAdapter routeDetailsAdapter = new RouteDetailsAdapter(this, event.getRoutesList(), this);
            detailsRv.setLayoutManager(new LinearLayoutManager(this, LinearLayoutManager.HORIZONTAL, true));
            detailsRv.setAdapter(routeDetailsAdapter);
            _selectedRouteIndex = 0;
            addRoutesToMap();
        } else {
            LoadingDialog.dismissDialog(this.dialog);
        }
    }

    @Override
    protected void onStart() {
        super.onStart();
        EventBus.getDefault().register(this);
    }

    @Override
    protected void onStop() {
        EventBus.getDefault().unregister(this);
        if (positioning != null) {
            positioning.stopLocating();
        }
        super.onStop();
    }

    @Subscribe(threadMode = ThreadMode.MAIN)
    public void onPlacesSuggestionEvent(GetPlacesEvent event) {
        if (event != null) {
//            Utils.disableSoftKeyboard(MainActivity.this);
            showList();
            placesSuggestions.clear();
            for (PlaceData placeData : event.getPlacesList()) {
                if (placeData.getPlace() != null) placesSuggestions.add(placeData);
            }
            setSuggestionAdapter();
        }
    }

    public void trafficBtn(View view) {
        isTrafficOn = !isTrafficOn;
        if (isTrafficOn) {
            trafficImg.setImageDrawable(ContextCompat.getDrawable(this, R.drawable.traffic_on));
            Map<String, String> map = new HashMap<>();
            // Once these traffic layers are added to the map, they will be automatically updated while panning the map.
            map.put(MapFeatures.TRAFFIC_FLOW, MapFeatureModes.TRAFFIC_FLOW_WITH_FREE_FLOW);
            // MapFeatures.TRAFFIC_INCIDENTS renders traffic icons and lines to indicate the location of incidents.
            map.put(MapFeatures.TRAFFIC_INCIDENTS, MapFeatureModes.DEFAULT);
            mapView.getMapScene().enableFeatures(map);
        } else {
            trafficImg.setImageDrawable(ContextCompat.getDrawable(this, R.drawable.traffic_off));
            List<String> map = new ArrayList<>();
            map.add(MapFeatures.TRAFFIC_FLOW);
            map.add(MapFeatures.TRAFFIC_INCIDENTS);
            mapView.getMapScene().disableFeatures(map);
        }
        LoadingDialog.showDialog(dialog);
        //update traffic lines
        if (!mapRoutes.isEmpty()) {
//            drawTrafficPolylineForRoute(mapRoutes.get(_selectedRouteIndex));
            new DrawTrafficLines().execute(new ArrayList<>(mapRoutes.get(_selectedRouteIndex).getSections()));
        }
    }

    private void clearMapRoutes() {
        for (MapPolyline polyline : mapPolylines) {
            mapView.getMapScene().removeMapPolyline(polyline);
        }
        mapPolylines.clear();
    }

    private void clearMapTrafficPolyline() {
        if (trafficPolylines != null) {
            for (MapPolyline polyline : trafficPolylines) {
                mapView.getMapScene().removeMapPolyline(polyline);
            }

            trafficPolylines.clear();
            trafficPolylines = null;
        }
    }

    private void addRoutesToMap() {
        clearMapRoutes();
        for (int i = 0; i < mapRoutes.size(); ++i) {
            addRouteToMap(mapRoutes.get(i), i == _selectedRouteIndex);
        }
        //update traffic lines
//        drawTrafficPolylineForRoute(mapRoutes.get(_selectedRouteIndex));
        new DrawTrafficLines().execute(new ArrayList<>(mapRoutes.get(_selectedRouteIndex).getSections()));
    }

    private void addRouteToMap(Route route, boolean selected) {
        try {
            GeoPolyline routeGeoPolyline = route.getGeometry();
            MapPolyline routeMapPolyline;
            if (selected) {
                selectedRoute = route;
                routeMapPolyline = new MapPolyline(
                        routeGeoPolyline, new MapPolyline.SolidRepresentation(new MapMeasureDependentRenderSize(RenderSize.Unit.PIXELS, 10), Color.valueOf(ContextCompat.getColor(this, R.color.selectedRouteColor)), new MapMeasureDependentRenderSize(RenderSize.Unit.PIXELS, 1), Color.valueOf(ContextCompat.getColor(this, R.color.selectedRouteBorderColor)), LineCap.ROUND));
                routeMapPolyline.setDrawOrder(1);
            } else {
                routeMapPolyline = new MapPolyline(
                        routeGeoPolyline, new MapPolyline.SolidRepresentation(new MapMeasureDependentRenderSize(RenderSize.Unit.PIXELS, 10), Color.valueOf(ContextCompat.getColor(this, R.color.routeColor)), new MapMeasureDependentRenderSize(RenderSize.Unit.PIXELS, 1), Color.valueOf(ContextCompat.getColor(this, R.color.routeBorderColor)), LineCap.ROUND));
                routeMapPolyline.setDrawOrder(0);
            }
            mapView.getMapScene().addMapPolyline(routeMapPolyline);
            mapPolylines.add(routeMapPolyline);
            zoomRoutes();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

//    private void drawTrafficPolylineForRoute(Route route) {
////        new Handler(Looper.myLooper()).postDelayed(() -> {
////            clearMapTrafficPolyline();
//////            List<TrafficLineColor> trafficLineColors = new ArrayList<>();
////            if (isTrafficOn) {
//////                for (Section section : route.getSections()) {
////                if (!newRoute.getSections().isEmpty()) {
////                    Section section = newRoute.getSections().get(0);
////                    List<Span> spanList = section.getSpans();
//////                    if (section.getSpans().size() > 50000) {
//////                        int dividedValue = section.getSpans().size()/50000;
//////                        for (int i = 0; i < dividedValue; i++) {
//////                            if(i<dividedValue-1){
//////                                Utils.calculateSpans(this, trafficLineColors, 50000*i, (50000*i)+50000, tempLineColor, spanPolyline, section);
//////                            }else{
//////                                Utils.calculateSpans(this, trafficLineColors, 50000*i, section.getSpans().size(), tempLineColor, spanPolyline, section);
//////                            }
//////                        }
//////                    } else {
////                    List<TrafficLineColor> trafficLineColors = Utils.calculateSpans(this, 0, spanList.size(), section);
//////                    }
////                    //draw traffic polyline map
//////                runOnUiThread(() -> {
////                    try {
////                        for (TrafficLineColor trafficLine : trafficLineColors) {
////                            GeoPolyline trafficGeoPolyline = new GeoPolyline(trafficLine.getCoordinatesList());
////                            MapPolyline trafficRouteMapPolyline = new MapPolyline(trafficGeoPolyline, 10.0, trafficLine.getLineColor());
////                            trafficRouteMapPolyline.setDrawOrder(1);
////                            trafficRouteMapPolyline.setOutlineColor(trafficLine.getLineColor());
////                            trafficRouteMapPolyline.setOutlineWidth(1.0);
////                            trafficPolylines.add(trafficRouteMapPolyline);
////                            //add polyline to map view
////                            mapView.getMapScene().addMapPolyline(trafficRouteMapPolyline);
////                        }
////                    } catch (InstantiationErrorException e) {
////                        e.printStackTrace();
////                    }
//////                });
////                }
////            }
////            LoadingDialog.dismissDialog(dialog);
////        }, 100);
//        clearMapTrafficPolyline();
//        trafficPolylines = new ArrayList<>();
//        if (route.getLengthInMeters() / 1000 > 5000) {
//            Log.d(TAG, "Skip showing traffic-on-route for longer routes.");
//            return;
//        }
//
//        for (Section section : route.getSections()) {
//            for (Span span : section.getSpans()) {
//                TrafficSpeed trafficSpeed = span.getTrafficSpeed();
//                Color lineColor = Utils.getTrafficColor(trafficSpeed.jamFactor);
//                if (lineColor == null) {
//                    // We skip rendering low traffic.
//                    continue;
//                }
//                float widthInPixels = 10;
//                MapPolyline trafficRouteMapPolyline = new MapPolyline(span.getGeometry(), widthInPixels, lineColor);
//                trafficPolylines.add(trafficRouteMapPolyline);
//                mapView.getMapScene().addMapPolyline(trafficRouteMapPolyline);
//            }
//        }
//        LoadingDialog.dismissDialog(dialog);
//    }

    private void updateRoutesToMap(int index) {
        try {
            mapPolylines.get(_selectedRouteIndex).setRepresentation(new MapPolyline.SolidRepresentation(new MapMeasureDependentRenderSize(RenderSize.Unit.PIXELS, 10), Color.valueOf(ContextCompat.getColor(this, R.color.routeColor)), new MapMeasureDependentRenderSize(RenderSize.Unit.PIXELS, 1), Color.valueOf(ContextCompat.getColor(this, R.color.routeBorderColor)), LineCap.ROUND));
            mapPolylines.get(_selectedRouteIndex).setDrawOrder(0);

            mapPolylines.get(index).setRepresentation(new MapPolyline.SolidRepresentation(new MapMeasureDependentRenderSize(RenderSize.Unit.PIXELS, 10), Color.valueOf(ContextCompat.getColor(this, R.color.selectedRouteColor)), new MapMeasureDependentRenderSize(RenderSize.Unit.PIXELS, 1), Color.valueOf(ContextCompat.getColor(this, R.color.selectedRouteBorderColor)), LineCap.ROUND));
            mapPolylines.get(index).setDrawOrder(1);

            _selectedRouteIndex = index;
            zoomRoutes();
            LoadingDialog.showDialog(dialog);
            //update map for pois
            if (!categories.isEmpty()) {
                updatePoiForRoute(mapRoutes.get(index));
            }
            //update traffic lines
//        drawTrafficPolylineForRoute(mapRoutes.get(_selectedRouteIndex));
            new DrawTrafficLines().execute(new ArrayList<>(mapRoutes.get(index).getSections()));
            //update selected route
            selectedRoute = mapRoutes.get(index);
        } catch (MapPolyline.Representation.InstantiationException |
                 MapMeasureDependentRenderSize.InstantiationException e) {
            e.printStackTrace();
        }
    }

    void showPoiSheet() {
        poiSheetBehavior.setState(BottomSheetBehavior.STATE_EXPANDED);

    }

    void hidePoiSheet() {
        poiSheetBehavior.setState(BottomSheetBehavior.STATE_COLLAPSED);
    }

    public void poiShowBtn(View view) {
        showPoiSheet();
    }

    public void poiCloseBtn(View view) {
        hidePoiSheet();
        setSelectedPoi();
        LoadingDialog.showDialog(dialog);
        updatePoiForRoute(mapRoutes.get(_selectedRouteIndex));
    }

    private void clearCategories() {
        categories.clear();
    }

    private void setSelectedPoi() {
        clearCategories();
        for (PoiModel poi : poiModelArrayList
        ) {
            if (poi.getTitle().equals(Constant.EAT_DRINK_POI)) {
                if (poi.getSelection()) {
                    eatPoiImg.setImageDrawable(ContextCompat.getDrawable(this, R.drawable.eat_and_drink_icon));
                    categories.add(new PlaceCategory(PlaceCategory.EAT_AND_DRINK));
                } else {
                    eatPoiImg.setImageDrawable(ContextCompat.getDrawable(this, R.drawable.eat_and_drink_icon_disabled));
                }
            } else if (poi.getTitle().equals(Constant.FUELING_STATIONS_POI)) {
                if (poi.getSelection()) {
                    fuelPoiImg.setImageDrawable(ContextCompat.getDrawable(this, R.drawable.fueling_station_icon));
                    categories.add(new PlaceCategory(PlaceCategory.BUSINESS_AND_SERVICES_FUELING_STATION));
                } else {
                    fuelPoiImg.setImageDrawable(ContextCompat.getDrawable(this, R.drawable.fueling_station_icon_disabled));
                }
            } else {
                if (poi.getSelection()) {
                    categories.add(new PlaceCategory(PlaceCategory.BUSINESS_AND_SERVICES_ATM));
                    atmPoiImg.setImageDrawable(ContextCompat.getDrawable(this, R.drawable.atm_icon));
                } else {
                    atmPoiImg.setImageDrawable(ContextCompat.getDrawable(this, R.drawable.atm_icon_disabled));
                }
            }
        }
    }

    @Override
    public void onPoiItemClick(View view, int position) {
//        setSelectedPoi();
//        updatePoiForRoute(mapRoutes.get(_selectedRouteIndex));
    }

    /// Searches POI for the [route].
    void updatePoiForRoute(Route route) {
        _clearMarkers();
        _clearPins();
        _stopCurrentSearch();
//        if (_placesForRoutes.size() > 0) {
//            for (PlacesForRoute place : _placesForRoutes
//            ) {
//                if (place.getRoute() == route) {
//                    _addMarkersForPlaces(place.getPlaceList());
//                }
////                else {
////                    if (_categories == null) {
////                        return;
////                    }
////                }
//            }
//        } else {
        GeoCorridor geoCorridor = new GeoCorridor(route.getGeometry().vertices, 20);
        CategoryQuery categoryQuery = new CategoryQuery(categories, new CategoryQuery.Area(geoCorridor));
        SearchOptions searchOptions = new SearchOptions();
        searchOptions.languageCode = LanguageCode.EN_US;
        searchOptions.maxItems = 100;

        new Handler().postDelayed(() -> {
            if (search.isDeviceConnected()) {
                _poiSearchTask = search.getSearchEngine().search(categoryQuery, searchOptions, (searchError, list) -> {
                    LoadingDialog.dismissDialog(dialog);
                    if (searchError != null) {
                        return;
                    }
//                    _placesForRoutes.add(new PlacesForRoute(route, list));
                    _addMarkersForPlaces(list);
                });
            } else {
                _poiSearchTask = search.getOfflineSearchEngine().search(categoryQuery, searchOptions, (searchError, list) -> {
                    LoadingDialog.dismissDialog(dialog);
                    if (searchError != null) {
                        return;
                    }
//                    _placesForRoutes.add(new PlacesForRoute(route, list));
                    _addMarkersForPlaces(list);
                });
            }
        }, 150);
//        }
    }

    private void _clearPins() {
//        List<MapView.ViewPin> viewPins = mapView.getViewPins();
        if (_markersPinViews != null) {
            for (View view : _markersPinViews) {
                mapView.unpinView(view);
            }
//            mapView.getViewPins().clear();
            _markersPinViews.clear();
        }
    }

    void _clearMarkers() {
        if (_markers != null) {
            for (MarkerForPlace markerForPlace : _markers) {
                mapView.getMapScene().removeMapMarker(markerForPlace.getMapMarker());
            }
            _markers.clear();
        }

    }

    void _addMarkersForPlaces(List<Place> places) {
        try {
            for (Place place : places) {
                int resourceId = _findPoiTypeForCategories(place.getDetails().categories);
                if (resourceId != 0) {
                    MapImage mapImage = MapImageFactory.fromResource(getResources(), resourceId);
                    MapMarker mapMarker = new MapMarker(place.getGeoCoordinates(), mapImage);
                    showMarkerInfoWindow(place, resourceId);
                    Metadata metadata = new Metadata();
                    Search.SearchResultMetadata searchResultMetadata = new Search.SearchResultMetadata(place);
                    metadata.setCustomValue("key_search_result", searchResultMetadata);
                    mapMarker.setMetadata(metadata);
                    mapView.getMapScene().addMapMarker(mapMarker);
                    _markers.add(new MarkerForPlace(mapMarker, place));
                }

            }

        } catch (Exception ex) {
            Log.e("Exception", ex.getMessage());
        }

    }

    private void showMarkerInfoWindow(Place place, int resourceId) {
        ImageView imageView = new ImageView(this);
        LinearLayout.LayoutParams lp = new LinearLayout.LayoutParams(50, 50);
        lp.setMargins(10, 0, 20, 0);
        imageView.setLayoutParams(lp);
        imageView.setImageResource(resourceId);
        TextView textView = new TextView(this);
        textView.setTextSize(16f);
        textView.setTextColor(ContextCompat.getColor(this, R.color.white));
        textView.setText(Utils.makeDistanceString(place.getDistanceInMeters() != null ? place.getDistanceInMeters() : 0));

        LinearLayout linearLayout = new LinearLayout(this);
        linearLayout.setOrientation(LinearLayout.HORIZONTAL);
        linearLayout.setGravity(Gravity.CENTER_VERTICAL);
        linearLayout.setBackgroundResource(R.color.black);
        linearLayout.setPadding(10, 10, 10, 10);
        linearLayout.addView(imageView);
        linearLayout.addView(textView);
        MapView.ViewPin viewPin = mapView.pinView(linearLayout, place.getGeoCoordinates());
        if (viewPin != null) {
            viewPin.setAnchorPoint(new Anchor2D(0.5F, 1.3));
            _markersPinViews.add(linearLayout);
        }
    }

    int _findPoiTypeForCategories(List<PlaceCategory> categories) {
        int resourceId = 0;
        for (int i = 0; i < categories.size(); ++i) {
            resourceId = findPoiTypeForCategoryId(categories.get(i).getId());
            if (resourceId != 0) {
                return resourceId;
            }
        }
        return resourceId;
    }

    int findPoiTypeForCategoryId(String categoryId) {
        for (PoiModel poi : poiModelArrayList) {
            if (poi.getId().equals(categoryId)) {
                return poi.getImage();
            }
        }

        int index = categoryId.lastIndexOf('-');
        if (index < 0) {
            return 0;
        }

        return findPoiTypeForCategoryId(categoryId.substring(0, index));
    }

    void _stopCurrentSearch() {
        if (_poiSearchTask != null) {
            _poiSearchTask.cancel();
            _poiSearchTask = null;
        }

    }

    @Override
    public void onSearchItemClick(View view, int position) {
        Utils.hideKeyboard(RoutingActivity.this);
        if (!isWaypoint) {
            if (isPickup) {
                Place place = placesSuggestions.get(position).getPlace();
                startPlace = place.getGeoCoordinates();
                pickupTv.setText(place.getTitle());
            }
            if (isDestination) {
                Place place = placesSuggestions.get(position).getPlace();
                destinationPlace = place.getGeoCoordinates();
                dropOffTv.setText(place.getTitle());
            }
            hideSearchSheet();
            beginRoute();
        } else {
            Place place = placesSuggestions.get(position).getPlace();
            Metadata metadata = new Metadata();
            metadata.setCustomValue("key_search_result", new Search.SearchResultMetadata(place));
            search.addPoiMapMarker(place.getGeoCoordinates(), metadata);
            onClickAddWaypoint(place);
            hideSearchSheet();
        }
    }

    @Override
    public void onRouteItemClick(View view, int position) {
        updateRoutesToMap(position);
    }

    @Override
    public void onRouteManeuverClick(View view, int position) {
        //update selected route
        selectedRoute = mapRoutes.get(position);
        Gson gson = new Gson();
        String start = gson.toJson(startPlace);
        String destination = gson.toJson(destinationPlace);
        SharedPref.getInstance().saveRouteData(gson.toJson(selectedRoute));

        Intent intent = new Intent(RoutingActivity.this, RouteDetailsActivity.class);
        intent.putExtra("start", start);
        intent.putExtra("destination", destination);
        intent.putExtra("startName", pickupTv.getText().toString());
        intent.putExtra("destinationName", dropOffTv.getText().toString());
        intent.putExtra("selected_option", selectedOption);
        intent.putExtra("fromShare", getIntent().getBooleanExtra("fromShare", false));
        startActivity(intent);
    }

    @Override
    public void onRouteNavigationClick(View view, int position) {
        //update selected route
        selectedRoute = mapRoutes.get(position);
        try {
            Gson gson = new Gson();
            String start = gson.toJson(startPlace);
            String destination = gson.toJson(destinationPlace);
            List<String> waypointList = new ArrayList<>();
            for (Waypoint waypoint : routing.waypointArrayList) {
                waypointList.add(new Gson().toJson(waypoint));
            }
            SharedPref.getInstance().saveWaypoints(waypointList);
            SharedPref.getInstance().saveRouteData(gson.toJson(selectedRoute));
            Intent intent = new Intent(RoutingActivity.this, NavigationActivity.class);
            intent.putExtra("start", start);
            intent.putExtra("destination", destination);
            intent.putExtra("startName", pickupTv.getText().toString());
            intent.putExtra("destinationName", dropOffTv.getText().toString());
            intent.putExtra("selected_option", selectedOption);
            intent.putExtra("fromShare", getIntent().getBooleanExtra("fromShare", false));
            startActivity(intent);
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    public void setPickupLocationClicked(View view) {
        isPickup = true;
        isDestination = false;
        isWaypoint = false;
        showSearchSheet();
    }

    public void setDestinationLocationClicked(View view) {
        isPickup = false;
        isDestination = true;
        isWaypoint = false;
        showSearchSheet();
    }

    void showSearchSheet() {
        searchSheetBehavior.setState(BottomSheetBehavior.STATE_EXPANDED);
    }

    void hideSearchSheet() {
        searchSheetBehavior.setState(BottomSheetBehavior.STATE_COLLAPSED);
    }

    public void cancelBtn(View view) {
        hideSearchSheet();
    }

    private boolean isDeviceConnected() {
        // An application may define here a logic to determine whether a device is connected or not.
        // For this example app, the flag is set from UI.
//        return isDeviceConnected && !isOfflineModeEnabled;
        ConnectivityManager connectivityManager = ((ConnectivityManager) getSystemService(Context.CONNECTIVITY_SERVICE));
        boolean isDeviceConnected = connectivityManager.getActiveNetworkInfo() != null && connectivityManager.getActiveNetworkInfo().isConnected();
        return isDeviceConnected && !SharedPref.getInstance().getOfflineMode();
    }

    @Override
    public void onClickPolyLine(List<MapPolyline> mapPolylinesList) {
        Log.e(TAG, "onClickPolyLine: " + mapPolylines.indexOf(mapPolylinesList.get(0)));
        int index = 0;
        for (int i = 0; i < mapPolylinesList.size(); i++) {
            if (mapPolylines.contains(mapPolylinesList.get(i))) {
                index = mapPolylines.indexOf(mapPolylinesList.get(i));
                break;
            }
        }
        detailsRv.smoothScrollToPosition(index);
        updateRoutesToMap(index);
    }

    @Override
    public void onClickRouteNew(Place place) {
        destinationPlace = place.getGeoCoordinates();
        dropOffTv.setText(place.getTitle());
        //refresh route
        beginRoute();
    }

    @Override
    public void onClickRoute(Place place) {

    }

    @Override
    public void onClickAddWaypoint(Place place) {
        addWaypoint(new PlaceCoordinates(place.getTitle(), place.getAddress().addressText, new Gson().toJson(place.getGeoCoordinates())));
    }

    public void addWaypoint(PlaceCoordinates placeCoordinates) {
        waypointsSuggestions.add(placeCoordinates);
        updateWaypointList();
        routing.addWaypoints(new Gson().fromJson(placeCoordinates.getGeoCoordinates(), GeoCoordinates.class), selectedOption);
    }

    private void updateWaypointList() {
        waypointsRv.getAdapter().notifyDataSetChanged();
        if (!waypointsSuggestions.isEmpty()) {
            waypointsRv.setVisibility(View.VISIBLE);
            no_waypoints.setVisibility(View.GONE);
        } else {
            waypointsRv.setVisibility(View.GONE);
            no_waypoints.setVisibility(View.VISIBLE);
        }
    }

    @Override
    public void onCategorySearched(Place place) {
//        titleTxt.setText(place.getTitle());
//        addressTxt.setText(place.getAddress().addressText);
//        distanceTxt.setText(Utils.makeDistanceString(place.getDistanceInMeters() != null ? place.getDistanceInMeters() : 0));
//        enableMapUpdate = false;
//
//        mapView.getCamera().lookAt(place.getGeoCoordinates());
//        setCurrentLocVisibility();
//        selectedPlace = place;
    }

    @Override
    public void onClick(View view) {
        if (view.getId() == R.id.preference_txt) {
            startActivityForResult(new Intent(this, PreferencesActivity.class), 2323);
        }
    }

    @Override
    protected void onActivityResult(int requestCode, int resultCode, @Nullable Intent data) {
        super.onActivityResult(requestCode, resultCode, data);
        if (resultCode == Activity.RESULT_OK && requestCode == 2323) {
            //refresh route
            Utils.disableSoftKeyboard(this);
            new Handler().postDelayed(() -> {
                setSelectedOption(selectedOption);
            }, 1000);
        }
    }

    @Override
    public void onWaypointItemClick(View view, int position) {
        waypointsSuggestions.remove(position);
        updateWaypointList();
        MapMarker marker = search.mapMarkerList.get(position);
        mapView.getMapScene().removeMapMarker(marker);
        search.mapMarkerList.remove(position);
        routing.removeWaypoints(position, selectedOption);
    }

    class DrawTrafficLines extends AsyncTask<List<Section>, Void, Void> {

        @Override
        protected void onPreExecute() {
            if (!dialog.isShowing()) {
                LoadingDialog.showDialog(dialog);
            }
            super.onPreExecute();
        }

        @SafeVarargs
        @Override
        protected final Void doInBackground(List<Section>... arrayLists) {
            System.runFinalization();
            Runtime.getRuntime().runFinalization();
            Runtime.getRuntime().gc();
            System.gc();

//            List<Section> sectionList = new ArrayList<>(arrayLists[0]);
            clearMapTrafficPolyline();

            trafficPolylines = null;
            if (isTrafficOn) {
                if (!arrayLists[0].isEmpty()) {
//                    List<TrafficLineColor> trafficLineColors = Utils.calculateSpans(RoutingActivity.this, arrayLists[0].get(0).getSpans());
//                    List<TrafficLineColor> trafficLineColors = new ArrayList<>();
//                    for (Section section : arrayLists[0]) {
//                        trafficLineColors.addAll(Utils.calculateSpans(RoutingActivity.this, section.getSpans()));
//                    }
//                    try {
//                        trafficPolylines = new ArrayList<>();
//                        for (TrafficLineColor trafficLine : trafficLineColors) {
//                            GeoPolyline trafficGeoPolyline = new GeoPolyline(trafficLine.getCoordinatesList());
//                            MapPolyline trafficRouteMapPolyline = new MapPolyline(trafficGeoPolyline, 10.0, trafficLine.getLineColor());
//                            trafficRouteMapPolyline.setDrawOrder(1);
//                            trafficRouteMapPolyline.setOutlineColor(trafficLine.getLineColor());
//                            trafficRouteMapPolyline.setOutlineWidth(1.0);
//                            trafficPolylines.add(trafficRouteMapPolyline);
//                            //add polyline to map view
//                            //  Log.e(TAG, "doInBackground: "+ trafficRouteMapPolyline);
//                            mapView.getMapScene().addMapPolyline(trafficRouteMapPolyline);
//                        }
//                    } catch (InstantiationErrorException e) {
//                        e.printStackTrace();
//                    }
                    trafficPolylines = new ArrayList<>();
                    for (Section section : arrayLists[0]) {
                        for (Span span : section.getSpans()) {
                            TrafficSpeed trafficSpeed = span.getTrafficSpeed();
                            Color lineColor = Utils.getTrafficColor(trafficSpeed.jamFactor);
                            if (lineColor == null) {
                                // We skip rendering low traffic.
                                continue;
                            }
//                            float widthInPixels = 10;
//                            MapPolyline trafficRouteMapPolyline = new MapPolyline(span.getGeometry(), widthInPixels, lineColor);
//                            trafficPolylines.add(trafficRouteMapPolyline);
//                            mapView.getMapScene().addMapPolyline(trafficRouteMapPolyline);

                            try {
                                MapPolyline trafficRouteMapPolyline = new MapPolyline(span.getGeometry(), new MapPolyline.SolidRepresentation(new MapMeasureDependentRenderSize(RenderSize.Unit.PIXELS, 10), lineColor, new MapMeasureDependentRenderSize(RenderSize.Unit.PIXELS, 1), lineColor, LineCap.ROUND));
                                trafficRouteMapPolyline.setDrawOrder(1);
                                trafficPolylines.add(trafficRouteMapPolyline);
                                //add polyline to map view
                                //  Log.e(TAG, "doInBackground: "+ trafficRouteMapPolyline);
                                mapView.getMapScene().addMapPolyline(trafficRouteMapPolyline);
                            } catch (MapPolyline.Representation.InstantiationException |
                                     MapMeasureDependentRenderSize.InstantiationException e) {
                                e.printStackTrace();
                            }
                        }
                    }
                    arrayLists[0].clear();
                }
            }
            return null;
        }

        @Override
        protected void onPostExecute(Void unused) {
            LoadingDialog.dismissDialog(dialog);
            super.onPostExecute(unused);
        }
    }
}
