package com.example.heresdkandroidref.activity;

import androidx.appcompat.app.AppCompatActivity;

import android.content.Intent;
import android.net.Uri;
import android.os.Bundle;
import android.os.Handler;
import android.util.Log;
import android.view.WindowManager;

import com.example.heresdkandroidref.R;
import com.example.heresdkandroidref.models.PlaceCoordinates;
import com.example.heresdkandroidref.models.PlaceData;
import com.google.gson.Gson;
import com.here.sdk.core.GeoCoordinates;
import com.here.sdk.search.Place;
import com.here.sdk.search.PlaceSerializationException;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;


public class SplashActivity extends AppCompatActivity {

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_splash);
        redirectScreen();
        getWindow().setSoftInputMode(WindowManager.LayoutParams.SOFT_INPUT_STATE_ALWAYS_HIDDEN);
    }

    private void redirectScreen() {
        Uri data = getIntent().getData();
        String ssp = data != null ? data.getSchemeSpecificPart() : "";
        String pointsURL = ssp.contains("drive/") ? ssp.split("drive/")[1] : "";
        String pointsURLFiltered = pointsURL.contains("?") ? pointsURL.split("\\?")[0] : "";
        String[] pointsArray = pointsURLFiltered.contains("/") ? pointsURLFiltered.split("/") : new String[]{};
        Log.e("redirectScreen: ", (ssp) + " - " + (pointsArray.length));
        new Handler().postDelayed(() -> {
            if (pointsArray.length >= 2) {
                String[] startLocation = pointsArray[0].split(":");
                String startName = startLocation[0];
                String[] startLocationLatLong = startLocation[1].split(",");
                GeoCoordinates currentCoordinates = new GeoCoordinates(Double.parseDouble(startLocationLatLong[0]), Double.parseDouble(startLocationLatLong[1]));
                String[] destinationLocation = pointsArray[pointsArray.length - 1].split(":");
                String destinationName = destinationLocation[0];
                String[] destinationLocationLatLong = destinationLocation[1].split(",");
                GeoCoordinates destinationCoordinates = new GeoCoordinates(Double.parseDouble(destinationLocationLatLong[0]), Double.parseDouble(destinationLocationLatLong[1]));
                //collect waypoints
                ArrayList<String> waypointsList = new ArrayList<>();
                if (pointsArray.length > 2) {
                    String[] waypointsArray = Arrays.copyOfRange(pointsArray, 1, pointsArray.length - 1);
                    for (String waypoint : waypointsArray) {
                        String[] location = waypoint.split(":");
                        String name = location[0];
                        String[] latLong = location[1].split(",");
                        GeoCoordinates coordinates = new GeoCoordinates(Double.parseDouble(latLong[0]), Double.parseDouble(latLong[1]));
                        PlaceCoordinates placeCoordinates = new PlaceCoordinates(name.split(",")[0], name, new Gson().toJson(coordinates));
                        waypointsList.add(new Gson().toJson(placeCoordinates));
                    }
                }
                Log.e("redirectScreen: ", "Waypoints - " + waypointsList.size());
                Intent intent1 = new Intent(SplashActivity.this, RoutingActivity.class);
                intent1.putExtra("origin", startName);
                intent1.putExtra("currentLocation", new Gson().toJson(currentCoordinates));
                intent1.putExtra("destination", destinationName);
                intent1.putExtra("fromShare", true);
                intent1.putExtra("destinationLocation", new Gson().toJson(destinationCoordinates));
                intent1.putStringArrayListExtra("waypoints", waypointsList);
                startActivity(intent1);
                finish();
            } else {
                startActivity(new Intent(SplashActivity.this, MainActivity.class));
                finish();
            }
        }, 1000);

    }

}