package com.example.heresdkandroidref.activity;

import android.content.Intent;
import android.os.Bundle;
import android.view.View;
import android.widget.AdapterView;

import androidx.appcompat.app.AppCompatActivity;
import androidx.appcompat.widget.AppCompatCheckBox;
import androidx.appcompat.widget.Toolbar;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import com.example.heresdkandroidref.R;
import com.example.heresdkandroidref.adapters.SwitchAdapter;
import com.example.heresdkandroidref.utils.Utils;
import com.google.android.material.textfield.MaterialAutoCompleteTextView;
import com.google.android.material.textfield.TextInputEditText;
import com.google.gson.Gson;
import com.here.sdk.core.CountryCode;
import com.here.sdk.routing.OptimizationMode;
import com.here.sdk.transport.TruckSpecifications;
import com.here.sdk.transport.TruckType;
import com.here.sdk.transport.WeightPerAxleGroup;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Map;

public class TruckSpecificationsActivity extends AppCompatActivity implements AdapterView.OnItemClickListener {

    Toolbar toolbarTB;
    private MaterialAutoCompleteTextView truckTypeACTV;
    TextInputEditText widthET, heightET, lengthET, axleCountET, axleWeightET, grossWeightET, trailerCountET, singleAxleGroupET, tandemAxleGroupET, tripleAxleGroupET, quadAxleGroupET, quintAxleGroupET;
    Map<String, TruckType> truckTypeMap = Utils.getTruckTypeMap();
    TruckSpecifications truckSpecifications = new TruckSpecifications();

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_truck_specification);
        String specificationData = getIntent().getStringExtra("truckSpecifications");
        truckSpecifications = !specificationData.isEmpty() ? new Gson().fromJson(specificationData, TruckSpecifications.class) : new TruckSpecifications();
        initViews();
    }

    private void initViews() {
        toolbarTB = findViewById(R.id.toolbarTB);
        truckTypeACTV = findViewById(R.id.truckTypeACTV);
        widthET = findViewById(R.id.widthET);
        heightET = findViewById(R.id.heightET);
        lengthET = findViewById(R.id.lengthET);
        axleCountET = findViewById(R.id.axleCountET);
        axleWeightET = findViewById(R.id.axleWeightET);
        grossWeightET = findViewById(R.id.grossWeightET);
        trailerCountET = findViewById(R.id.trailerCountET);
        singleAxleGroupET = findViewById(R.id.singleAxleGroupET);
        tandemAxleGroupET = findViewById(R.id.tandemAxleGroupET);
        tripleAxleGroupET = findViewById(R.id.tripleAxleGroupET);
        quadAxleGroupET = findViewById(R.id.quadAxleGroupET);
        quintAxleGroupET = findViewById(R.id.quintAxleGroupET);

        toolbarTB.setTitle("Truck Specifications");
        setSupportActionBar(toolbarTB);
        getSupportActionBar().setDisplayHomeAsUpEnabled(true);
        getSupportActionBar().setDisplayShowHomeEnabled(true);
        //set items
        truckTypeACTV.setSimpleItems(truckTypeMap.keySet().toArray(new String[0]));
        truckTypeACTV.setOnItemClickListener(this);
        //initialize specifications
        truckTypeACTV.setText(truckTypeACTV.getAdapter().getItem(new ArrayList<>(truckTypeMap.values()).indexOf(truckSpecifications.truckType)).toString(), false);
        widthET.setText(truckSpecifications.widthInCentimeters != null ? truckSpecifications.widthInCentimeters + "" : "");
        heightET.setText(truckSpecifications.heightInCentimeters != null ? truckSpecifications.heightInCentimeters + "" : "");
        lengthET.setText(truckSpecifications.lengthInCentimeters != null ? truckSpecifications.lengthInCentimeters + "" : "");
        axleCountET.setText(truckSpecifications.axleCount != null ? truckSpecifications.axleCount + "" : "");
        axleWeightET.setText(truckSpecifications.weightPerAxleInKilograms != null ? truckSpecifications.weightPerAxleInKilograms + "" : "");
        grossWeightET.setText(truckSpecifications.grossWeightInKilograms != null ? truckSpecifications.grossWeightInKilograms + "" : "");
        trailerCountET.setText(truckSpecifications.trailerCount != null ? truckSpecifications.trailerCount + "" : "");
        if (truckSpecifications.weightPerAxleGroup != null) {
            singleAxleGroupET.setText(truckSpecifications.weightPerAxleGroup.singleAxleGroupInKilograms != null ? truckSpecifications.weightPerAxleGroup.singleAxleGroupInKilograms + "" : "");
            tandemAxleGroupET.setText(truckSpecifications.weightPerAxleGroup.tandemAxleGroupInKilograms != null ? truckSpecifications.weightPerAxleGroup.tandemAxleGroupInKilograms + "" : "");
            tripleAxleGroupET.setText(truckSpecifications.weightPerAxleGroup.tripleAxleGroupInKilograms != null ? truckSpecifications.weightPerAxleGroup.tripleAxleGroupInKilograms + "" : "");
            quadAxleGroupET.setText(truckSpecifications.weightPerAxleGroup.quadAxleGroupInKilograms != null ? truckSpecifications.weightPerAxleGroup.quadAxleGroupInKilograms + "" : "");
            quintAxleGroupET.setText(truckSpecifications.weightPerAxleGroup.quintAxleGroupInKilograms != null ? truckSpecifications.weightPerAxleGroup.quintAxleGroupInKilograms + "" : "");
        }
    }

    @Override
    public boolean onSupportNavigateUp() {
        onBackPressed();
        return true;
    }

    @Override
    public void onBackPressed() {
        Intent intent = new Intent();
        truckSpecifications.truckType = truckTypeMap.get(truckTypeACTV.getText().toString());
        truckSpecifications.widthInCentimeters = widthET.getText().toString().isEmpty() ? null : Integer.parseInt(widthET.getText().toString());
        truckSpecifications.heightInCentimeters = heightET.getText().toString().isEmpty() ? null : Integer.parseInt(heightET.getText().toString());
        truckSpecifications.lengthInCentimeters = lengthET.getText().toString().isEmpty() ? null : Integer.parseInt(lengthET.getText().toString());
        truckSpecifications.axleCount = axleCountET.getText().toString().isEmpty() ? null : Integer.parseInt(axleCountET.getText().toString());
        truckSpecifications.weightPerAxleInKilograms = axleWeightET.getText().toString().isEmpty() ? null : Integer.parseInt(axleWeightET.getText().toString());
        truckSpecifications.grossWeightInKilograms = grossWeightET.getText().toString().isEmpty() ? null : Integer.parseInt(grossWeightET.getText().toString());
        truckSpecifications.trailerCount = trailerCountET.getText().toString().isEmpty() ? null : Integer.parseInt(grossWeightET.getText().toString());
        WeightPerAxleGroup weightPerAxleGroup = new WeightPerAxleGroup();
        weightPerAxleGroup.singleAxleGroupInKilograms = singleAxleGroupET.getText().toString().isEmpty() ? null : Integer.parseInt(singleAxleGroupET.getText().toString());
        weightPerAxleGroup.tandemAxleGroupInKilograms = tandemAxleGroupET.getText().toString().isEmpty() ? null : Integer.parseInt(tandemAxleGroupET.getText().toString());
        weightPerAxleGroup.tripleAxleGroupInKilograms = tripleAxleGroupET.getText().toString().isEmpty() ? null : Integer.parseInt(tripleAxleGroupET.getText().toString());
        weightPerAxleGroup.quadAxleGroupInKilograms = quadAxleGroupET.getText().toString().isEmpty() ? null : Integer.parseInt(quadAxleGroupET.getText().toString());
        weightPerAxleGroup.quintAxleGroupInKilograms = quintAxleGroupET.getText().toString().isEmpty() ? null : Integer.parseInt(quintAxleGroupET.getText().toString());
        if (weightPerAxleGroup.singleAxleGroupInKilograms != null || weightPerAxleGroup.tandemAxleGroupInKilograms != null || weightPerAxleGroup.tripleAxleGroupInKilograms != null || weightPerAxleGroup.quadAxleGroupInKilograms != null || weightPerAxleGroup.quintAxleGroupInKilograms != null) {
            truckSpecifications.weightPerAxleGroup = weightPerAxleGroup;
        } else {
            truckSpecifications.weightPerAxleGroup = null;
        }
        intent.putExtra("truckSpecifications", new Gson().toJson(truckSpecifications));
        setResult(RESULT_OK, intent);
        finish();
    }

    @Override
    public void onItemClick(AdapterView<?> parent, View view, int position, long id) {

    }
}