package com.example.heresdkandroidref.adapters;

import android.content.Context;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.recyclerview.widget.RecyclerView;

import com.example.heresdkandroidref.R;
import com.example.heresdkandroidref.interfaces.ItemType;
import com.example.heresdkandroidref.interfaces.ItemsClickListener;

import java.util.ArrayList;


public class ItemsAdapter extends RecyclerView.Adapter<ItemsAdapter.ViewHolder> {

    public Context context;
    ArrayList<String> itemsArrayList;
    public ItemsClickListener listener;
    public ItemType itemType;


    public ItemsAdapter(Context context, ArrayList<String> itemsArrayList, ItemType itemType, ItemsClickListener clickListener) {
        this.context = context;
        this.listener = clickListener;
        this.itemType = itemType;
        this.itemsArrayList = itemsArrayList;
    }

    @NonNull
    @Override
    public ViewHolder onCreateViewHolder(ViewGroup viewGroup, int i) {
        View itemView = LayoutInflater.from(viewGroup.getContext())
                .inflate(R.layout.row_phone_item, viewGroup, false);
        return new ViewHolder(itemView);
    }

    @Override
    public void onBindViewHolder(ViewHolder viewHolder, int position) {
        initData(viewHolder, position);
    }

    @Override
    public long getItemId(int position) {
        return position;
    }

    @Override
    public int getItemCount() {
        return itemsArrayList.size();
    }

    private void initData(final ViewHolder viewHolder, final int position) {
        viewHolder.poi_txt.setText(itemsArrayList.get(position));
        switch (itemType){
            case PHONES:
                viewHolder.poi_img.setImageResource(R.drawable.call);
                break;
            case EMAILS:
                viewHolder.poi_img.setImageResource(R.drawable.mail);
                break;
            case WEBSITES:
                viewHolder.poi_img.setImageResource(R.drawable.web);
                break;
        }
        viewHolder.itemView.setOnClickListener(view -> listener.onItemClick(view, position));
    }


    static class ViewHolder extends RecyclerView.ViewHolder {
        public TextView poi_txt;
        public ImageView poi_img;

        private ViewHolder(View convertView) {
            super(convertView);
            poi_txt = convertView.findViewById(R.id.poi_txt);
            poi_img = convertView.findViewById(R.id.poi_img);
        }
    }


}
