package com.example.heresdkandroidref.adapters;

import android.content.Context;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;

import androidx.annotation.NonNull;
import androidx.core.content.ContextCompat;
import androidx.recyclerview.widget.RecyclerView;

import com.example.heresdkandroidref.R;
import com.example.heresdkandroidref.utils.Utils;
import com.here.sdk.navigation.Lane;

import java.util.ArrayList;
import java.util.List;


public class LanesAdapter extends RecyclerView.Adapter<LanesAdapter.ViewHolder> {

    public Context context;
    List<Lane> lanesArrayList;


    public LanesAdapter(Context context, List<Lane> lanesArrayList) {
        this.context = context;
        this.lanesArrayList = lanesArrayList;
    }

    @NonNull
    @Override
    public ViewHolder onCreateViewHolder(ViewGroup viewGroup, int i) {
        View itemView = LayoutInflater.from(viewGroup.getContext())
                .inflate(R.layout.lane_item, viewGroup, false);
        return new ViewHolder(itemView);
    }

    @Override
    public void onBindViewHolder(ViewHolder viewHolder, int position) {
        initData(viewHolder, position);
    }

    @Override
    public long getItemId(int position) {
        return position;
    }

    @Override
    public int getItemCount() {
        return lanesArrayList.size();
    }

    private void initData(final ViewHolder viewHolder, final int position) {
        viewHolder.laneDirectionIV.setImageResource(Utils.getLanesImage(lanesArrayList.get(position).directionCategory));
        switch (lanesArrayList.get(position).recommendationState){
            case HIGHLY_RECOMMENDED:
                viewHolder.laneDirectionIV.setBackgroundColor(ContextCompat.getColor(context, android.R.color.holo_green_dark));
                break;
            case RECOMMENDED:
                viewHolder.laneDirectionIV.setBackgroundColor(ContextCompat.getColor(context, android.R.color.holo_green_light));
                break;
            case NOT_RECOMMENDED:
                viewHolder.laneDirectionIV.setBackgroundColor(ContextCompat.getColor(context, android.R.color.holo_red_dark));
                break;
        }
    }


    static class ViewHolder extends RecyclerView.ViewHolder {
        public ImageView laneDirectionIV;

        private ViewHolder(View convertView) {
            super(convertView);
            laneDirectionIV = convertView.findViewById(R.id.laneDirectionIV);
        }
    }


}
