package com.example.heresdkandroidref.adapters;

import android.content.Context;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.TextView;

import androidx.recyclerview.widget.RecyclerView;

import com.example.heresdkandroidref.R;
import com.example.heresdkandroidref.interfaces.ManeuverItemClickListener;
import com.example.heresdkandroidref.models.ManeuverModel;
import com.example.heresdkandroidref.utils.Utils;

import java.util.ArrayList;


public class ManeuversAdapter extends RecyclerView.Adapter<ManeuversAdapter.ViewHolder> {

    public Context context;
    ArrayList<ManeuverModel> maneuverModelArrayList;
    public ManeuverItemClickListener onItemClickListener;

    public void setItemClickListener(ManeuverItemClickListener clickListener) {
        onItemClickListener = clickListener;
    }


    public ManeuversAdapter(Context context, ArrayList<ManeuverModel> maneuverModels, ManeuverItemClickListener clickListener) {
        this.context=context;
        onItemClickListener = clickListener;
        this.maneuverModelArrayList = maneuverModels;
    }
    
    @Override
    public ViewHolder onCreateViewHolder(ViewGroup viewGroup, int i) {
        View itemView = LayoutInflater.from(viewGroup.getContext())
                .inflate(R.layout.row_maneuvers_item, viewGroup, false);
        return new ViewHolder(itemView);
    }

    @Override
    public void onBindViewHolder(ViewHolder viewHolder, int position) {

        initData(viewHolder,position);
    }

    @Override
    public long getItemId(int position) {
        return position;
    }

    @Override
    public int getItemCount() {
        return maneuverModelArrayList.size();
    }

    private void initData(final ViewHolder viewHolder, final int position)
    {
            final ManeuverModel maneuverModel = maneuverModelArrayList.get(position);
                viewHolder.itemTitle.setText(maneuverModel.getTitle());
                if(Utils.getManeuverImage(maneuverModel.getImage())!=0){
                    viewHolder.itemImg.setImageDrawable(context.getResources().getDrawable(Utils.getManeuverImage(maneuverModel.getImage())));
                }

    }


    class ViewHolder extends RecyclerView.ViewHolder {
        public TextView itemTitle;
        public ImageView itemImg;

        private ViewHolder(View convertView)
        {
            super(convertView);
            itemTitle = convertView.findViewById(R.id.poi_txt);
            itemImg = convertView.findViewById(R.id.poi_img);
            convertView.setOnClickListener(new View.OnClickListener() {
                @Override
                public void onClick(View view) {
                    onItemClickListener.onManeuverItemClick(view, getAdapterPosition());
                }
            });

        }
    }
    

}
