package com.example.heresdkandroidref.adapters;

import android.content.Context;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.CompoundButton;
import android.widget.ImageView;
import android.widget.Switch;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.recyclerview.widget.RecyclerView;

import com.example.heresdkandroidref.R;
import com.example.heresdkandroidref.interfaces.PoiItemClickListener;
import com.example.heresdkandroidref.models.PoiModel;

import java.util.ArrayList;


public class PoiAdapter extends RecyclerView.Adapter<PoiAdapter.ViewHolder> {

    public Context context;
    ArrayList<PoiModel> poiModelArrayList;
    public PoiItemClickListener onItemClickListener;

    public void setItemClickListener(PoiItemClickListener clickListener) {
        onItemClickListener = clickListener;
    }

    public PoiAdapter(Context context, ArrayList<PoiModel> poiModels, PoiItemClickListener clickListener) {
        this.context=context;
        onItemClickListener = clickListener;
        this.poiModelArrayList = poiModels;
    }
    
    @NonNull
    @Override
    public ViewHolder onCreateViewHolder(ViewGroup viewGroup, int i) {
        View itemView = LayoutInflater.from(viewGroup.getContext())
                .inflate(R.layout.row_poi_item, viewGroup, false);
        return new ViewHolder(itemView);
    }

    @Override
    public void onBindViewHolder(@NonNull ViewHolder viewHolder, int position) {
        initData(viewHolder,position);
    }

    @Override
    public long getItemId(int position) {
        return position;
    }

    @Override
    public int getItemCount() {
        return poiModelArrayList.size();
    }

    private void initData(final ViewHolder viewHolder, final int position) {
        final PoiModel poi = poiModelArrayList.get(position);
        viewHolder.itemTitle.setText(poi.getTitle());
        viewHolder.itemSwitch.setChecked(poi.getSelection());
        viewHolder.itemImg.setImageDrawable(context.getResources().getDrawable(poi.getImage()));
        viewHolder.itemSwitch.setOnCheckedChangeListener((compoundButton, b) -> poi.isSelected(b));
    }

    class ViewHolder extends RecyclerView.ViewHolder {
        public TextView itemTitle;
        public ImageView itemImg;
        public Switch itemSwitch;

        private ViewHolder(View convertView) {
            super(convertView);
            itemTitle = convertView.findViewById(R.id.poi_txt);
            itemImg = convertView.findViewById(R.id.poi_img);
            itemSwitch = convertView.findViewById(R.id.poi_sw);
            convertView.setOnClickListener(view -> onItemClickListener.onPoiItemClick(view, getAdapterPosition()));
        }
    }

}
