package com.example.heresdkandroidref.adapters;

import android.content.Context;
import android.content.res.ColorStateList;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.ProgressBar;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.appcompat.content.res.AppCompatResources;
import androidx.core.content.ContextCompat;
import androidx.recyclerview.widget.RecyclerView;

import com.example.heresdkandroidref.R;
import com.example.heresdkandroidref.features.OfflineMapsExample;
import com.example.heresdkandroidref.interfaces.RegionItemClickListener;
import com.google.android.material.progressindicator.CircularProgressIndicator;
import com.here.sdk.maploader.InstalledRegion;
import com.here.sdk.maploader.InstalledRegionStatus;
import com.here.sdk.maploader.Region;

import java.util.ArrayList;


public class RegionsMapAdapter extends RecyclerView.Adapter<RegionsMapAdapter.ViewHolder> {

    public Context context;
    ArrayList<Region> regionArrayList;
    ArrayList<Region> downloadingRegionsArrayList = new ArrayList<>();
    ArrayList<Integer> downloadingRegionsProgressArrayList = new ArrayList<>();
    public RegionItemClickListener onItemClickListener;
    public OfflineMapsExample offlineMapsExample;

    public void setItemClickListener(RegionItemClickListener clickListener) {
        onItemClickListener = clickListener;
    }


    public RegionsMapAdapter(Context context, ArrayList<Region> regionArrayList, RegionItemClickListener clickListener, OfflineMapsExample offlineMapsExample) {
        this.context = context;
        onItemClickListener = clickListener;
        this.regionArrayList = regionArrayList;
        this.offlineMapsExample = offlineMapsExample;
    }

    @Override
    public ViewHolder onCreateViewHolder(ViewGroup viewGroup, int i) {
        View itemView = LayoutInflater.from(viewGroup.getContext())
                .inflate(R.layout.row_region_item, viewGroup, false);
        return new ViewHolder(itemView);
    }

    @Override
    public void onBindViewHolder(ViewHolder viewHolder, int position) {

        initData(viewHolder, position);
    }

    @Override
    public long getItemId(int position) {
        return position;
    }

    @Override
    public int getItemCount() {
        return regionArrayList.size();
    }

    private void initData(@NonNull final ViewHolder viewHolder, final int position) {
        final Region region = regionArrayList.get(position);
        InstalledRegion installedRegion = offlineMapsExample.getRegionInstalledStatus(region);
        viewHolder.itemRegionName.setText(region.name);
        if (region.childRegions != null && region.childRegions.size() > 0) {
            viewHolder.itemRegionItems.setText(String.format("%d Items", region.childRegions.size()));
            ColorStateList csl = AppCompatResources.getColorStateList(context, R.color.grey);
            viewHolder.itemRegionImg.setImageDrawable(ContextCompat.getDrawable(context, android.R.drawable.arrow_down_float));
            viewHolder.itemRegionImg.setImageTintList(csl);
        } else {
            if (installedRegion.status == InstalledRegionStatus.INSTALLED) {
//                viewHolder.progressBarPB.setVisibility(View.GONE);
//                viewHolder.cancel_img.setVisibility(View.GONE);
                viewHolder.itemRegionImg.setVisibility(View.VISIBLE);
                ColorStateList csl = AppCompatResources.getColorStateList(context, android.R.color.holo_red_dark);
                viewHolder.itemRegionImg.setImageDrawable(ContextCompat.getDrawable(context, R.drawable.delete));
                viewHolder.itemRegionImg.setImageTintList(csl);
            } else {
                if (downloadingRegionsArrayList.contains(region)) {
//                    viewHolder.progressBarPB.setVisibility(View.VISIBLE);
//                    viewHolder.progressBarPB.setProgressCompat(downloadingRegionsProgressArrayList.get(downloadingRegionsArrayList.indexOf(region)),true);
//                    viewHolder.cancel_img.setVisibility(View.VISIBLE);
                    viewHolder.itemRegionImg.setVisibility(View.VISIBLE);
                    ColorStateList csl = AppCompatResources.getColorStateList(context, android.R.color.holo_red_dark);
                    viewHolder.itemRegionImg.setImageDrawable(ContextCompat.getDrawable(context, android.R.drawable.ic_menu_close_clear_cancel));
                    viewHolder.itemRegionImg.setImageTintList(csl);
                } else {
//                    viewHolder.progressBarPB.setVisibility(View.GONE);
//                    viewHolder.cancel_img.setVisibility(View.GONE);
                    viewHolder.itemRegionImg.setVisibility(View.VISIBLE);
                    ColorStateList csl = AppCompatResources.getColorStateList(context, R.color.grey);
                    viewHolder.itemRegionImg.setImageDrawable(ContextCompat.getDrawable(context, android.R.drawable.stat_sys_download_done));
                    viewHolder.itemRegionImg.setImageTintList(csl);
                }
            }
            long sizeOnDiskInMB = region.sizeOnDiskInBytes / (1024 * 1024);
            viewHolder.itemRegionItems.setText(String.format("%d MB", sizeOnDiskInMB));
        }
        viewHolder.itemRegionImg.setOnClickListener(view -> {
            if (region.childRegions != null && region.childRegions.size() > 0) {
                onItemClickListener.onRegionNavigateClick(view, position);
            } else {
                if (installedRegion.status != InstalledRegionStatus.INSTALLED) {
                    if (downloadingRegionsArrayList.contains(region)) {
                        onItemClickListener.onRegionDownloadCancelClick(view, downloadingRegionsArrayList.indexOf(region));
                    }else {
                        onItemClickListener.onRegionDownloadClick(view, position);
                    }
                }else {
                    onItemClickListener.onRegionDeleteClick(view,position);
                }
            }

        });

        viewHolder.itemMain.setOnClickListener(view -> {
            if (region.childRegions != null && region.childRegions.size() > 0) {
                onItemClickListener.onRegionNavigateClick(view, position);
            } else {
                if (installedRegion.status != InstalledRegionStatus.INSTALLED) {
                    onItemClickListener.onRegionDownloadClick(view, position);
                } else {
                    onItemClickListener.onRegionDeleteClick(view,position);
                }
            }

        });

//        viewHolder.cancel_img.setOnClickListener(view -> {
//            if (region.childRegions != null && region.childRegions.size() > 0) {
//                onItemClickListener.onRegionNavigateClick(view, position);
//            } else {
//                if (installedRegion.status != InstalledRegionStatus.INSTALLED) {
//                    if (downloadingRegionsArrayList.contains(region)) {
//                        onItemClickListener.onRegionDownloadCancelClick(view, position);
//                    }else {
//                        onItemClickListener.onRegionDownloadClick(view, position);
//                    }
//                }
//            }
//
//        });


    }

    public void customUpdateList(ArrayList<Region> regionArrayList) {
        this.regionArrayList = regionArrayList;
        notifyDataSetChanged();
    }

    static class ViewHolder extends RecyclerView.ViewHolder {
        public TextView itemRegionName, itemRegionItems;
        public ImageView itemRegionImg;
//        public ImageView cancel_img;
        public LinearLayout itemMain;
//        public CircularProgressIndicator progressBarPB;

        private ViewHolder(View convertView) {
            super(convertView);
            itemRegionItems = convertView.findViewById(R.id.region_items_txt);
            itemRegionName = convertView.findViewById(R.id.region_name_txt);
            itemRegionImg = convertView.findViewById(R.id.region_img);
            itemMain = convertView.findViewById(R.id.main_cv);
//            cancel_img = convertView.findViewById(R.id.cancel_img);
//            progressBarPB = convertView.findViewById(R.id.progressBarPB);
            convertView.setOnClickListener(new View.OnClickListener() {
                @Override
                public void onClick(View view) {

                }
            });

        }
    }

    public void startDownloadingRegion(int position, int progress) {
        downloadingRegionsArrayList.add(regionArrayList.get(position));
//        downloadingRegionsProgressArrayList.add(progress);
        notifyItemChanged(position);
    }

//    public void progressDownloadingRegion(int position, int progress) {
//        downloadingRegionsProgressArrayList.set(downloadingRegionsArrayList.indexOf(regionArrayList.get(position)),progress);
//        notifyItemChanged(position);
//    }

    public boolean isDownloadingRegionStarted(int position) {
        return downloadingRegionsArrayList.contains(regionArrayList.get(position));
    }

    public void stopDownloadingRegion(int position) {
        downloadingRegionsArrayList.remove(regionArrayList.get(position));
        notifyItemChanged(position);
    }

}
