package com.example.heresdkandroidref.adapters;

import android.content.Context;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.TextView;

import androidx.recyclerview.widget.RecyclerView;

import com.example.heresdkandroidref.R;
import com.example.heresdkandroidref.interfaces.RouteItemClickListener;
import com.example.heresdkandroidref.utils.Utils;
import com.here.sdk.routing.Route;

import java.util.ArrayList;


public class RouteDetailsAdapter extends RecyclerView.Adapter<RouteDetailsAdapter.ViewHolder> {

    public Context context;
    ArrayList<Route> placesArrayList;
    public RouteItemClickListener onItemClickListener;

    public RouteDetailsAdapter(Context context, ArrayList<Route> places, RouteItemClickListener clickListener) {
        this.context = context;
        this.onItemClickListener = clickListener;
        this.placesArrayList = places;
    }

    @Override
    public ViewHolder onCreateViewHolder(ViewGroup viewGroup, int i) {
        View itemView = LayoutInflater.from(viewGroup.getContext())
                .inflate(R.layout.row_route_item, viewGroup, false);
        return new ViewHolder(itemView);
    }

    @Override
    public void onBindViewHolder(ViewHolder viewHolder, int position) {

        initData(viewHolder, position);
    }

    @Override
    public long getItemId(int position) {
        return position;
    }

    @Override
    public int getItemCount() {
        return placesArrayList != null ? placesArrayList.size() : 0;
    }

    private void initData(final ViewHolder viewHolder, final int position) {
        final Route route = placesArrayList.get(position);
        viewHolder.itemTime.setText(Utils.makeDurationString(route.getDuration().getSeconds()));

        if (route.getTrafficDelay().getSeconds() > 60) {
            viewHolder.itemDelay.setText("Incl. " + Utils.makeDurationString(route.getTrafficDelay().getSeconds()) + " delay");
        } else {
            viewHolder.itemDelay.setText("No delays");
        }

        viewHolder.itemDistance.setText(Utils.makeDistanceString(route.getLengthInMeters()));
        viewHolder.itemRouteNavigation.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                onItemClickListener.onRouteNavigationClick(view, position);
            }
        });
        viewHolder.itemRouteManeuver.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                onItemClickListener.onRouteManeuverClick(view, position);
            }
        });

    }


    class ViewHolder extends RecyclerView.ViewHolder {
        public TextView itemTime, itemDelay, itemDistance;
        public ImageView itemRouteManeuver, itemRouteNavigation;


        private ViewHolder(View convertView) {
            super(convertView);
            itemTime = convertView.findViewById(R.id.time_txt);

            itemDelay = convertView.findViewById(R.id.delay_txt);
            itemDistance = convertView.findViewById(R.id.new_maneuver_distance_txt);
            itemRouteManeuver = convertView.findViewById(R.id.route_maneuver_img);
            itemRouteNavigation = convertView.findViewById(R.id.route_navigation_img);
            convertView.setOnClickListener(new View.OnClickListener() {
                @Override
                public void onClick(View view) {
                    onItemClickListener.onRouteItemClick(view, getAdapterPosition());
                }
            });

        }
    }


}
