package com.example.heresdkandroidref.adapters;

import android.content.Context;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.TextView;


import androidx.recyclerview.widget.RecyclerView;

import com.example.heresdkandroidref.R;

import com.example.heresdkandroidref.interfaces.SearchedItemClickListener;
import com.example.heresdkandroidref.models.PlaceData;
import com.here.sdk.search.Place;


import java.util.ArrayList;


public class SuggestionsAdapter extends RecyclerView.Adapter<SuggestionsAdapter.ViewHolder> {

    public Context context;
    ArrayList<PlaceData> placesArrayList;
    public SearchedItemClickListener onItemClickListener;

    public void setItemClickListener(SearchedItemClickListener clickListener) {
        onItemClickListener = clickListener;
    }


    public SuggestionsAdapter(Context context, ArrayList<PlaceData> places, SearchedItemClickListener clickListener) {
        this.context = context;
        onItemClickListener = clickListener;
        this.placesArrayList = places;
    }

    @Override
    public ViewHolder onCreateViewHolder(ViewGroup viewGroup, int i) {
        View itemView = LayoutInflater.from(viewGroup.getContext())
                .inflate(R.layout.row_suggestion_item, viewGroup, false);
        return new ViewHolder(itemView);
    }

    @Override
    public void onBindViewHolder(ViewHolder viewHolder, int position) {

        initData(viewHolder, position);
    }

    @Override
    public long getItemId(int position) {
        return position;
    }

    @Override
    public int getItemCount() {
        return placesArrayList.size();
    }

    private void initData(final ViewHolder viewHolder, final int position) {
        final PlaceData place = placesArrayList.get(position);
        if (place.getPlace() != null) {
            viewHolder.itemTitle.setText(place.getTitle());
            viewHolder.itemAddress.setText(place.getPlace().getAddress().addressText);
            viewHolder.itemAddress.setVisibility(View.VISIBLE);
        } else {
            viewHolder.itemTitle.setText("\"" + place.getTitle() + "\"");
            viewHolder.itemAddress.setText("");
            viewHolder.itemAddress.setVisibility(View.GONE);
        }

    }


    class ViewHolder extends RecyclerView.ViewHolder {
        public TextView itemTitle;
        public TextView itemAddress;

        private ViewHolder(View convertView) {
            super(convertView);
            itemTitle = convertView.findViewById(R.id.title_txt);
            itemAddress = convertView.findViewById(R.id.address_txt);
            if (onItemClickListener != null) {
                convertView.setOnClickListener(view -> onItemClickListener.onSearchItemClick(view, getAdapterPosition()));
            }
        }
    }


}
