package com.example.heresdkandroidref.adapters;

import android.content.Context;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.appcompat.widget.AppCompatCheckBox;
import androidx.recyclerview.widget.RecyclerView;

import com.example.heresdkandroidref.R;

import java.util.ArrayList;
import java.util.List;


public class SwitchAdapter extends RecyclerView.Adapter<SwitchAdapter.ViewHolder> {

    public Context context;
    List<String> switchItemArrayList;
    ArrayList<String> checkedItemArrayList;

    public SwitchAdapter(Context context, List<String> switchItemArrayList, ArrayList<String> checkedItemArrayList) {
        this.context = context;
        this.switchItemArrayList = switchItemArrayList;
        this.checkedItemArrayList = checkedItemArrayList;
    }

    @NonNull
    @Override
    public ViewHolder onCreateViewHolder(ViewGroup viewGroup, int i) {
        View itemView = LayoutInflater.from(viewGroup.getContext())
                .inflate(R.layout.row_switch_item, viewGroup, false);
        return new ViewHolder(itemView);
    }

    @Override
    public void onBindViewHolder(@NonNull ViewHolder viewHolder, int position) {
        initData(viewHolder, position);
    }

    @Override
    public long getItemId(int position) {
        return position;
    }

    @Override
    public int getItemCount() {
        return switchItemArrayList.size();
    }

    private void initData(final ViewHolder viewHolder, final int position) {
        String item = switchItemArrayList.get(position);
        viewHolder.itemTitleTV.setText(item);
        viewHolder.itemCheckCB.setChecked(checkedItemArrayList.contains(item));
        viewHolder.itemCheckCB.setOnCheckedChangeListener((bt, checked) -> {
            if (checked) {
                if (!checkedItemArrayList.contains(item)) {
                    checkedItemArrayList.add(switchItemArrayList.get(position));
                }
            } else {
                checkedItemArrayList.remove(switchItemArrayList.get(position));
            }
        });
    }

    public ArrayList<String> getSelectedItems() {
        return checkedItemArrayList;
    }


    class ViewHolder extends RecyclerView.ViewHolder {
        public TextView itemTitleTV;
        public AppCompatCheckBox itemCheckCB;


        private ViewHolder(View convertView) {
            super(convertView);
            itemTitleTV = convertView.findViewById(R.id.itemTitleTV);
            itemCheckCB = convertView.findViewById(R.id.itemCheckCB);
        }
    }


}
