package com.example.heresdkandroidref.adapters;

import android.content.Context;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.TextView;

import androidx.recyclerview.widget.RecyclerView;

import com.example.heresdkandroidref.R;
import com.example.heresdkandroidref.interfaces.SearchedItemClickListener;
import com.example.heresdkandroidref.interfaces.WaypointItemClickListener;
import com.example.heresdkandroidref.models.PlaceCoordinates;
import com.example.heresdkandroidref.models.PlaceData;

import java.util.ArrayList;


public class WaypointsAdapter extends RecyclerView.Adapter<WaypointsAdapter.ViewHolder> {

    public Context context;
    ArrayList<PlaceCoordinates> placesArrayList;
    public WaypointItemClickListener onItemClickListener;


    public WaypointsAdapter(Context context, ArrayList<PlaceCoordinates> places, WaypointItemClickListener clickListener) {
        this.context = context;
        onItemClickListener = clickListener;
        this.placesArrayList = places;
    }

    @Override
    public ViewHolder onCreateViewHolder(ViewGroup viewGroup, int i) {
        View itemView = LayoutInflater.from(viewGroup.getContext())
                .inflate(R.layout.row_waypoints_item, viewGroup, false);
        return new ViewHolder(itemView);
    }

    @Override
    public void onBindViewHolder(ViewHolder viewHolder, int position) {

        initData(viewHolder, position);
    }

    @Override
    public long getItemId(int position) {
        return position;
    }

    @Override
    public int getItemCount() {
        return placesArrayList.size();
    }

    private void initData(final ViewHolder viewHolder, final int position) {
        final PlaceCoordinates place = placesArrayList.get(position);
        viewHolder.itemTitle.setText(place.getTitle());
        viewHolder.itemAddress.setText(place.getAddressText());
    }


    class ViewHolder extends RecyclerView.ViewHolder {
        public TextView itemTitle;
        public TextView itemAddress;
        public ImageView itemRemove;

        private ViewHolder(View convertView) {
            super(convertView);
            itemTitle = convertView.findViewById(R.id.title_txt);
            itemAddress = convertView.findViewById(R.id.address_txt);
            itemRemove = convertView.findViewById(R.id.cancel_img);
            if (onItemClickListener != null) {
                itemRemove.setOnClickListener(view -> onItemClickListener.onWaypointItemClick(view, getAdapterPosition()));
            }
        }
    }


}
