package com.example.heresdkandroidref.features;

import android.content.Context;
import android.net.Uri;
import android.os.Environment;
import android.util.Log;
import android.widget.Toast;

import com.here.sdk.core.Color;
import com.here.sdk.core.GeoCircle;
import com.here.sdk.core.GeoCoordinates;
import com.here.sdk.core.GeoPolygon;
import com.here.sdk.core.Location;
import com.here.sdk.core.errors.InstantiationErrorException;
import com.here.sdk.mapview.MapMeasure;
import com.here.sdk.mapview.MapPolygon;
import com.here.sdk.mapview.MapView;
import com.here.sdk.navigation.GPXDocument;
import com.here.sdk.navigation.GPXOptions;
import com.here.sdk.navigation.GPXTrack;
import com.here.sdk.navigation.GPXTrackWriter;
import com.here.sdk.routing.Route;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

public class GPXRecorderExample {

    private static final String TAG = GPXRecorderExample.class.getSimpleName();

    private final Context context;
    private final MapView mapView;
    private final HEREPositioningSimulator herePositioningSimulator = new HEREPositioningSimulator();
    private GPXTrackWriter gpxTrackWriter = new GPXTrackWriter();
    private final MapPolygon mapCircle;
    private boolean isRecordingStarted = false;

    public GPXRecorderExample(MapView mapView, Context context) {
        this.mapView = mapView;
        this.context = context;

        // A map circle to indicate a GPX location.
        mapCircle = addMapCircle();
    }

//    void playGPXTraceFromAssetsButtonPressed() {
//        String gpxFileName = "berlin.gpx";
//
//        if (!checkIfGPXFileExists(gpxFileName)) {
//            Log.e(TAG, "Error: Asset not found!");
//        }
//
//        // TODO
//        AssetManager assetManager = context.getAssets();
//        try {
//            assetManager.open(gpxFileName);
//        } catch (IOException e) {
//            Log.e(TAG, "Error: Asset not found!");
//            return;
//        }
//
//        try {
//            GPXDocument gpxDocument = new GPXDocument(getFilePath(gpxFileName).getAbsolutePath(), new GPXOptions());
//            GPXTrack gpxTrackFromAssets = gpxDocument.getTracks().get(0);
//            herePositioningSimulator.startLocating(location -> trackLocation(location), gpxTrackFromAssets);
//        } catch (InstantiationErrorException | IOException e) {
//            Log.e(TAG, e.getLocalizedMessage());
//        }
//    }

    public void playRecordedGPXTraceButtonPressed(String filename) {
        try {
            // Get the documents directory of device.
            File dir = context.getExternalFilesDir(Environment.DIRECTORY_DOCUMENTS);
            // Get the GPX tracks from the saved GPXDocument.
            GPXDocument gpxDocument = new GPXDocument(dir.getAbsolutePath() + "/" + filename, new GPXOptions());
            // We stored only one track.
            GPXTrack recordedGPXTrack = gpxDocument.getTracks().get(0);

            if (recordedGPXTrack != null) {
                herePositioningSimulator.startLocating(location -> trackLocation(location), recordedGPXTrack);
            }
        } catch (InstantiationErrorException e) {
            Log.e(TAG, e.error.name());
        }
    }

    public void recordGPXTraceButtonPressed() {
        gpxTrackWriter = new GPXTrackWriter();
        isRecordingStarted = true;
    }

    public void recordGPXWithLocation(Location location) {
        if (isRecordingStarted) {
            gpxTrackWriter.onLocationUpdated(location);
        }

    }

    public void stopButtonPressed(String routeName) {
        if (isRecordingStarted) {
            isRecordingStarted = false;
            saveGPXTrack(routeName);
        }
    }

    public boolean checkIfGPXFileExists(String pathName) {
        try {
            Uri uri = Uri.fromFile(new File(pathName));
            String filepath = uri.getPath();
            GPXDocument gpxDocument = new GPXDocument(filepath, new GPXOptions());
            Log.i("GPXDocument", gpxDocument.toString());
            return true;
        } catch (InstantiationErrorException e) {
            e.printStackTrace();
        }
        return false;
    }

    public void saveGPXTrack(String routeName) {
        File dir = context.getExternalFilesDir(Environment.DIRECTORY_DOCUMENTS);
        String filePath = dir.getAbsolutePath() + "/" + routeName + ".gpx";
        GPXDocument gpxDocument = getGPXDocument(gpxTrackWriter, routeName);
        gpxDocument.save(filePath);
        Toast.makeText(context, "GPX Track Saved successfully.", Toast.LENGTH_SHORT).show();
    }

    public GPXDocument getGPXDocument(GPXTrackWriter gpxTrackWriter, String routeName) {
        GPXTrack gpxTrack = gpxTrackWriter.getTrack();
        // Optionally, we give the track some name.
        gpxTrack.setName(routeName);
        List<GPXTrack> gpxTracks = new ArrayList<>();
        gpxTracks.add(gpxTrack);
        return new GPXDocument(gpxTracks);
    }

    public MapPolygon addMapCircle() {
        GeoCircle geoCircle = new GeoCircle(new GeoCoordinates(0, 0), 0);
        GeoPolygon geoPolygon = new GeoPolygon(geoCircle);
        Color fillColor = Color.valueOf(0, 0.56f, 0.54f, 0.63f); // RGBA
        MapPolygon mapPolygon = new MapPolygon(geoPolygon, fillColor);
        mapView.getMapScene().addMapPolygon(mapPolygon);
        return mapPolygon;
    }

    public void trackLocation(Location location) {
        // Update location of circle.
        double radiusInMeters = 20;
        GeoCircle geoCircle = new GeoCircle(location.coordinates, radiusInMeters);
        GeoPolygon geoPolygon = new GeoPolygon(geoCircle);
        mapCircle.setGeometry(geoPolygon);

        // Move camera to circle.
        double distanceInMeters = 1000;
        MapMeasure mapMeasureZoom = new MapMeasure(MapMeasure.Kind.DISTANCE, distanceInMeters);
        mapView.getCamera().lookAt(location.coordinates, mapMeasureZoom);
    }

//    public File getFilePath(String fileName) throws IOException {
//        File filePath = new File(context.getCacheDir(), fileName);
//        try {
//            try (InputStream inputStream = context.getAssets().open(fileName)) {
//                FileOutputStream outputStream = new FileOutputStream(filePath);
//                try {
//                    byte[] buf = new byte[1024];
//                    int len;
//                    while ((len = inputStream.read(buf)) > 0) {
//                        outputStream.write(buf, 0, len);
//                    }
//                } finally {
//                    outputStream.close();
//                }
//            }
//        } catch (IOException e) {
//            throw new IOException("Could not open file", e);
//        }
//        return filePath;
//    }
}
