
package com.example.heresdkandroidref.features;

import android.content.Context;
import android.net.ConnectivityManager;
import android.util.Log;

import androidx.annotation.Nullable;
import androidx.appcompat.app.AlertDialog;

import com.example.heresdkandroidref.R;
import com.example.heresdkandroidref.events.GetDetailsEvent;
import com.example.heresdkandroidref.models.PlaceCoordinates;
import com.example.heresdkandroidref.utils.SharedPref;
import com.here.sdk.core.GeoCoordinates;
import com.here.sdk.core.Point2D;
import com.here.sdk.core.errors.InstantiationErrorException;
import com.here.sdk.mapview.MapCamera;
import com.here.sdk.mapview.MapImage;
import com.here.sdk.mapview.MapImageFactory;
import com.here.sdk.mapview.MapMarker;
import com.here.sdk.mapview.MapMeasure;
import com.here.sdk.mapview.MapPolyline;
import com.here.sdk.mapview.MapView;
import com.here.sdk.routing.CalculateRouteCallback;
import com.here.sdk.routing.CarOptions;
import com.here.sdk.routing.Maneuver;
import com.here.sdk.routing.ManeuverAction;
import com.here.sdk.routing.OfflineRoutingEngine;
import com.here.sdk.routing.PedestrianOptions;
import com.here.sdk.routing.Route;
import com.here.sdk.routing.RoutingEngine;
import com.here.sdk.routing.RoutingError;
import com.here.sdk.routing.RoutingInterface;
import com.here.sdk.routing.ScooterOptions;
import com.here.sdk.routing.Section;
import com.here.sdk.routing.TruckOptions;
import com.here.sdk.routing.Waypoint;

import org.greenrobot.eventbus.EventBus;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Locale;

public class Routing {

    private static final String TAG = Routing.class.getName();

    private final Context context;
    private final MapView mapView;
    private final List<MapMarker> mapMarkerList = new ArrayList<>();
    private final List<MapPolyline> mapPolylines = new ArrayList<>();
    public final List<Waypoint> waypointArrayList = new ArrayList<>();
    private RoutingInterface routingEngine;
    private final RoutingEngine onlineRoutingEngine;
    private final OfflineRoutingEngine offlineRoutingEngine;
    private GeoCoordinates startGeoCoordinates;
    private GeoCoordinates destinationGeoCoordinates;
//    private boolean isDeviceConnected = true;
//    private boolean isOfflineModeEnabled;

    public Routing(Context context, MapView mapView) {
        this.context = context;
        this.mapView = mapView;
        MapCamera camera = mapView.getCamera();
        double distanceInMeters = 1000 * 10;
        MapMeasure mapMeasureZoom = new MapMeasure(MapMeasure.Kind.DISTANCE, distanceInMeters);
        camera.lookAt(new GeoCoordinates(52.520798, 13.409408), mapMeasureZoom);
//        isOfflineModeEnabled = AppState.getOfflineMode(context);
//        if(isDeviceConnected()) {
        try {
            onlineRoutingEngine = new RoutingEngine();
        } catch (InstantiationErrorException e) {
            throw new RuntimeException("Initialization of RoutingEngine failed: " + e.error.name());
        }
//        }else {
        try {
            // Allows to calculate routes on already downloaded or cached map data.
            // For downloading offline maps, please check the OfflineMaps example app.
            // This app uses only cached map data that gets downloaded when the user
            // pans the map. Please note that the OfflineRoutingEngine may not be able
            // to calculate a route, when not all map tiles are loaded. Especially, the
            // vector tiles for lower zoom levels are required to find possible paths.
            offlineRoutingEngine = new OfflineRoutingEngine();
        } catch (InstantiationErrorException e) {
            throw new RuntimeException("Initialization of OfflineRoutingEngine failed: " + e.error.name());
        }
//        }
        setRoutingEngine();
    }

//    public void addRoute() {
//        setRoutingEngine();
//        startGeoCoordinates = createRandomGeoCoordinatesAroundMapCenter();
//        destinationGeoCoordinates = createRandomGeoCoordinatesAroundMapCenter();
//        Waypoint startWaypoint = new Waypoint(startGeoCoordinates);
//        Waypoint destinationWaypoint = new Waypoint(destinationGeoCoordinates);
//
////        List<Waypoint> waypoints =
////                new ArrayList<>(Arrays.asList(startWaypoint, destinationWaypoint));
//        List<Waypoint> waypoints = new ArrayList<>();
//        waypoints.add(startWaypoint);
//        waypoints.addAll(waypointArrayList);
//        waypoints.add(destinationWaypoint);
//
////        CarOptions carOptions = new CarOptions();
//        CarOptions carOptions = SharedPref.getInstance().getCarPreferenceData();
//        carOptions.routeOptions.enableRouteHandle = true;
//        routingEngine.calculateRoute(
//                waypoints,
//                carOptions,
//                new CalculateRouteCallback() {
//                    @Override
//                    public void onRouteCalculated(@Nullable RoutingError routingError, @Nullable List<Route> routes) {
//                        if (routingError == null && routes != null) {
//                            Route route = routes.get(0);
//                            showRouteDetails(route);
//                            showRouteOnMap(route);
//                            logRouteViolations(route);
//                            ArrayList<Route> routesList = new ArrayList<>(routes);
//                            EventBus.getDefault().post(new GetDetailsEvent(routesList));
//                        } else {
//                            showDialog(context, "Error while calculating a route:", routingError.toString());
//                        }
//                    }
//                });
//    }

    // Calculates a route with two waypoints (start / destination).
    public void addRoute(GeoCoordinates startGC, GeoCoordinates destinationGC, String options) {
        setRoutingEngine();

        startGeoCoordinates = startGC;
        destinationGeoCoordinates = destinationGC;
//        Waypoint startWaypoint = new Waypoint(startGeoCoordinates);
//        Waypoint destinationWaypoint = new Waypoint(destinationGeoCoordinates);

//        List<Waypoint> waypoints =
//                new ArrayList<>(Arrays.asList(startWaypoint, destinationWaypoint));
        List<Waypoint> waypoints = new ArrayList<>();
        waypoints.add(new Waypoint(startGeoCoordinates));
        waypoints.addAll(waypointArrayList);
        waypoints.add(new Waypoint(destinationGeoCoordinates));

        calculateRouteWithOptions(waypoints, options);

    }

    private void calculateRouteWithOptions(List<Waypoint> waypoints, String options) {
        switch (options) {
            case "car" -> {
                //CarOptions carOptions = new CarOptions();
                CarOptions carOptions = SharedPref.getInstance().getCarPreferenceData();
                carOptions.routeOptions.enableRouteHandle = true;
                routingEngine.calculateRoute(
                        waypoints,
                        carOptions,
                        (routingError, routes) -> {
                            if (routingError == null) {
                                for (Route route : routes) {
//                                    showRouteDetails(route);
                                    showRouteOnMap(route);
                                    logRouteViolations(route);
                                }
                                ArrayList<Route> routesList = new ArrayList<>(routes);
                                EventBus.getDefault().post(new GetDetailsEvent(routesList));
                            } else {
                                showDialog(context, "Error while calculating a route:", routingError.toString());
                                EventBus.getDefault().post(new GetDetailsEvent(null));
                            }
                        });
            }
            case "truck" -> {
                //TruckOptions truckOptions = new TruckOptions();
                TruckOptions truckOptions = SharedPref.getInstance().getTruckPreferenceData();
                truckOptions.routeOptions.enableRouteHandle = true;
                routingEngine.calculateRoute(
                        waypoints,
                        truckOptions,
                        (routingError, routes) -> {
                            if (routingError == null) {
                                for (Route route : routes) {
//                                    showRouteDetails(route);
                                    showRouteOnMap(route);
                                    logRouteViolations(route);
                                }
                                ArrayList<Route> routesList = new ArrayList<>(routes);
                                EventBus.getDefault().post(new GetDetailsEvent(routesList));
                            } else {
                                showDialog(context, "Error while calculating a route:", routingError.toString());
                                EventBus.getDefault().post(new GetDetailsEvent(null));
                            }
                        });
            }
            case "scooter" -> {
                //ScooterOptions scooterOptions = new ScooterOptions();
                ScooterOptions scooterOptions = SharedPref.getInstance().getScooterPreferenceData();
                scooterOptions.routeOptions.enableRouteHandle = true;
                routingEngine.calculateRoute(
                        waypoints,
                        scooterOptions,
                        (routingError, routes) -> {
                            if (routingError == null) {
                                for (Route route : routes) {
//                                    showRouteDetails(route);
                                    showRouteOnMap(route);
                                    logRouteViolations(route);
                                }
                                ArrayList<Route> routesList = new ArrayList<>(routes);
                                EventBus.getDefault().post(new GetDetailsEvent(routesList));
                            } else {
                                showDialog(context, "Error while calculating a route:", routingError.toString());
                                EventBus.getDefault().post(new GetDetailsEvent(null));
                            }
                        });
            }
            case "walk" -> {
                //PedestrianOptions pedestrianOptions = new PedestrianOptions();
                PedestrianOptions pedestrianOptions = SharedPref.getInstance().getWalkPreferenceData();
                pedestrianOptions.routeOptions.enableRouteHandle = true;
                routingEngine.calculateRoute(
                        waypoints,
                        pedestrianOptions,
                        (routingError, routes) -> {
                            if (routingError == null) {
                                for (Route route : routes) {
//                                    showRouteDetails(route);
                                    showRouteOnMap(route);
                                    logRouteViolations(route);
                                }
                                ArrayList<Route> routesList = new ArrayList<>(routes);
                                EventBus.getDefault().post(new GetDetailsEvent(routesList));
                            } else {
                                showDialog(context, "Error while calculating a route:", routingError.toString());
                                EventBus.getDefault().post(new GetDetailsEvent(null));
                            }
                        });
            }
        }
    }

    // A route may contain several warnings, for example, when a certain route option could not be fulfilled.
    // An implementation may decide to reject a route if one or more violations are detected.
    private void logRouteViolations(Route route) {
//        for (Section section : route.getSections()) {
//            for (SectionNotice notice : section.getSectionNotices()) {
//                Log.e(TAG, "This route contains the following warning: " + notice.code.toString());
//            }
//        }
    }

    private void showRouteDetails(Route route) {
        long estimatedTravelTimeInSeconds = route.getDuration().getSeconds();
        int lengthInMeters = route.getLengthInMeters();

        String routeDetails =
                "Travel Time: " + formatTime(estimatedTravelTimeInSeconds)
                        + ", Length: " + formatLength(lengthInMeters);

        showDialog(context, "Route Details", routeDetails);
    }

    private String formatTime(long sec) {
        int hours = (int) (sec / 3600);
        int minutes = (int) ((sec % 3600) / 60);

        return String.format(Locale.getDefault(), "%02d:%02d", hours, minutes);
    }

    private String formatLength(int meters) {
        int kilometers = meters / 1000;
        int remainingMeters = meters % 1000;

        return String.format(Locale.getDefault(), "%02d.%02d km", kilometers, remainingMeters);
    }

    private void showRouteOnMap(Route route) {
        clearMap();

//        // Show route as polyline.
//        GeoPolyline routeGeoPolyline = route.getGeometry();
//        float widthInPixels = 20;
//        MapPolyline routeMapPolyline = new MapPolyline(routeGeoPolyline,
//                widthInPixels,
//                Color.valueOf(0, 0.56f, 0.54f, 0.63f)); // RGBA
//
//        mapView.getMapScene().addMapPolyline(routeMapPolyline);
//        mapPolylines.add(routeMapPolyline);

        GeoCoordinates startPoint =
                route.getSections().get(0).getDeparturePlace().mapMatchedCoordinates;
        GeoCoordinates destination =
                route.getSections().get(route.getSections().size() - 1).getArrivalPlace().mapMatchedCoordinates;

        // Draw a circle to indicate starting point and destination.
        addCircleMapMarker(startPoint, R.drawable.position);
        addCircleMapMarker(destination, R.drawable.poi);

        // Log maneuver instructions per route section.
//        List<Section> sections = route.getSections();
//        for (Section section : sections) {
//            logManeuverInstructions(section);
//        }
    }

    private void logManeuverInstructions(Section section) {
        Log.d(TAG, "Log maneuver instructions per route section:");
        List<Maneuver> maneuverInstructions = section.getManeuvers();
        for (Maneuver maneuverInstruction : maneuverInstructions) {
            ManeuverAction maneuverAction = maneuverInstruction.getAction();
            GeoCoordinates maneuverLocation = maneuverInstruction.getCoordinates();
            String maneuverInfo = maneuverInstruction.getText()
                    + ", Action: " + maneuverAction.name()
                    + ", Location: " + maneuverLocation.toString();
            Log.d(TAG, maneuverInfo);
        }
    }
    public void addOneWaypointToList(GeoCoordinates geoCoordinates) {
        waypointArrayList.add(new Waypoint(geoCoordinates));
    }
    public void swapWaypoints(int fromPosition, int toPosition, String options) {
        Collections.swap(waypointArrayList,fromPosition,toPosition);
        addWaypoints(null,options);
    }
    // Calculates a route with additional waypoints.
    public void removeWaypoints(int position, String options) {
        waypointArrayList.remove(position);
        addWaypoints(null,options);
    }
    public void addWaypoints(GeoCoordinates geoCoordinates, String options) {
        setRoutingEngine();

        if (startGeoCoordinates == null || destinationGeoCoordinates == null) {
            showDialog(context, "Error", "Please add a route first.");
            return;
        }

        if(geoCoordinates != null) waypointArrayList.add(new Waypoint(geoCoordinates));
        List<Waypoint> waypoints = new ArrayList<>();
        waypoints.add(new Waypoint(startGeoCoordinates));
        waypoints.addAll(waypointArrayList);
        waypoints.add(new Waypoint(destinationGeoCoordinates));

        calculateRouteWithOptions(waypoints, options);
    }

    public void clearMap() {
        clearWaypointMapMarker();
        clearRoute();
    }

    private void clearWaypointMapMarker() {
        for (MapMarker mapMarker : mapMarkerList) {
            mapView.getMapScene().removeMapMarker(mapMarker);
        }
        mapMarkerList.clear();
    }

    private void clearRoute() {
        for (MapPolyline mapPolyline : mapPolylines) {
            mapView.getMapScene().removeMapPolyline(mapPolyline);
        }
        mapPolylines.clear();
    }

    private GeoCoordinates createRandomGeoCoordinatesAroundMapCenter() {
        GeoCoordinates centerGeoCoordinates = mapView.viewToGeoCoordinates(
                new Point2D(mapView.getWidth() / 2, mapView.getHeight() / 2));
        if (centerGeoCoordinates == null) {
            // Should never happen for center coordinates.
            throw new RuntimeException("CenterGeoCoordinates are null");
        }
        double lat = centerGeoCoordinates.latitude;
        double lon = centerGeoCoordinates.longitude;
        return new GeoCoordinates(getRandom(lat - 0.02, lat + 0.02),
                getRandom(lon - 0.02, lon + 0.02));
    }

    private double getRandom(double min, double max) {
        return min + Math.random() * (max - min);
    }

    private void addCircleMapMarker(GeoCoordinates geoCoordinates, int resourceId) {
        MapImage mapImage = MapImageFactory.fromResource(context.getResources(), resourceId);
        MapMarker mapMarker = new MapMarker(geoCoordinates, mapImage);
        mapView.getMapScene().addMapMarker(mapMarker);
        mapMarkerList.add(mapMarker);
    }

    // Sets the OfflineRoutingEngine as main engine when the device is not connected, otherwise this will set the
    // RoutingEngine that requires connectivity.
    private void setRoutingEngine() {
        if (isDeviceConnected()) {
            routingEngine = onlineRoutingEngine;
        } else {
            routingEngine = offlineRoutingEngine;
        }
    }

    //    public void onSwitchOnlineButtonClicked() {
//        isDeviceConnected = true;
//        Toast.makeText(context,"The app will now use the RoutingEngine.", Toast.LENGTH_LONG).show();
//    }
//
//    public void onSwitchOfflineButtonClicked() {
//        isDeviceConnected = false;
//        Toast.makeText(context,"The app will now use the OfflineRoutingEngine.", Toast.LENGTH_LONG).show();
//    }
//
    private boolean isDeviceConnected() {
        // An application may define here a logic to determine whether a device is connected or not.
        // For this example app, the flag is set from UI.
//        return isDeviceConnected && !isOfflineModeEnabled;
        ConnectivityManager connectivityManager = ((ConnectivityManager) context.getSystemService(Context.CONNECTIVITY_SERVICE));
        boolean isDeviceConnected = connectivityManager.getActiveNetworkInfo() != null && connectivityManager.getActiveNetworkInfo().isConnected();
        return isDeviceConnected && !SharedPref.getInstance().getOfflineMode();
    }

    private void showDialog(Context context, String title, String message) {
        AlertDialog.Builder builder = new AlertDialog.Builder(context);
        builder.setTitle(title);
        builder.setMessage(message);
        builder.show();
    }
}
