
package com.example.heresdkandroidref.features;

import static android.content.Context.LAYOUT_INFLATER_SERVICE;

import static org.greenrobot.eventbus.EventBus.TAG;

import android.annotation.SuppressLint;
import android.content.Context;
import android.content.Intent;
import android.graphics.Color;
import android.graphics.drawable.ColorDrawable;
import android.net.ConnectivityManager;
import android.net.Uri;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.TextView;
import android.widget.Toast;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.appcompat.app.AlertDialog;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import com.example.heresdkandroidref.R;
import com.example.heresdkandroidref.activity.MainActivity;
import com.example.heresdkandroidref.adapters.ItemsAdapter;
import com.example.heresdkandroidref.events.GetPlacesEvent;
import com.example.heresdkandroidref.interfaces.ItemType;
import com.example.heresdkandroidref.models.PlaceData;
import com.example.heresdkandroidref.utils.SharedPref;
import com.here.sdk.core.Anchor2D;
import com.here.sdk.core.CustomMetadataValue;
import com.here.sdk.core.GeoBox;
import com.here.sdk.core.GeoCoordinates;
import com.here.sdk.core.LanguageCode;
import com.here.sdk.core.Metadata;
import com.here.sdk.core.Point2D;
import com.here.sdk.core.Rectangle2D;
import com.here.sdk.core.Size2D;
import com.here.sdk.core.errors.InstantiationErrorException;
import com.here.sdk.gestures.GestureState;
import com.here.sdk.mapview.MapCamera;
import com.here.sdk.mapview.MapCameraAnimation;
import com.here.sdk.mapview.MapCameraAnimationFactory;
import com.here.sdk.mapview.MapImage;
import com.here.sdk.mapview.MapImageFactory;
import com.here.sdk.mapview.MapMarker;
import com.here.sdk.mapview.MapMeasure;
import com.here.sdk.mapview.MapPolyline;
import com.here.sdk.mapview.MapView;
import com.here.sdk.mapview.MapViewBase;
import com.here.sdk.mapview.PickMapContentResult;
import com.here.sdk.search.Address;
import com.here.sdk.search.AddressQuery;
import com.here.sdk.search.Contact;
import com.here.sdk.search.EmailAddress;
import com.here.sdk.search.LandlinePhone;
import com.here.sdk.search.MobilePhone;
import com.here.sdk.search.OfflineSearchEngine;
import com.here.sdk.search.POIPaymentDetails;
import com.here.sdk.search.Place;
import com.here.sdk.search.PlaceIdQuery;
import com.here.sdk.search.PlaceIdSearchCallback;
import com.here.sdk.search.SearchCallback;
import com.here.sdk.search.SearchEngine;
import com.here.sdk.search.SearchError;
import com.here.sdk.search.SearchOptions;
import com.here.sdk.search.SuggestCallback;
import com.here.sdk.search.Suggestion;
import com.here.sdk.search.TextQuery;
import com.here.sdk.search.WebsiteAddress;
import com.here.time.Duration;

import org.apache.commons.lang3.StringUtils;
import org.greenrobot.eventbus.EventBus;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Objects;

public class Search {

    private static final String LOG_TAG = Search.class.getName();

    private final Context context;
    private final MapView mapView;
    private final MapCamera camera;
    public final List<MapMarker> mapMarkerList = new ArrayList<>();
    private final SearchEngine searchEngine;
    private final OfflineSearchEngine offlineSearchEngine;
    private final MarkerDetailListener markerDetailListener;
//    private boolean isDeviceConnected = true;
//    private boolean isOfflineModeEnabled;

    public Search(Context context, MapView mapView, MarkerDetailListener markerDetailListener) {
        this.context = context;
        this.mapView = mapView;
        this.markerDetailListener = markerDetailListener;
        camera = mapView.getCamera();
        double distanceInMeters = 5000;
        MapMeasure mapMeasureZoom = new MapMeasure(MapMeasure.Kind.DISTANCE, distanceInMeters);
        camera.lookAt(new GeoCoordinates(52.520798, 13.409408), mapMeasureZoom);
//        isOfflineModeEnabled = AppState.getOfflineMode(context);
//        if(isDeviceConnected()) {
        try {
            searchEngine = new SearchEngine();
        } catch (InstantiationErrorException e) {
            throw new RuntimeException("Initialization of SearchEngine failed: " + e.error.name());
        }
//        }else {
        try {
            // Allows to search on already downloaded or cached map data.
            offlineSearchEngine = new OfflineSearchEngine();
        } catch (InstantiationErrorException e) {
            throw new RuntimeException("Initialization of OfflineSearchEngine failed: " + e.error.name());
        }
//        }
        setTapGestureHandler();
        setLongPressGestureHandler();

//        Toast.makeText(context,"Long press on map to get the address for that position using reverse geocoding.", Toast.LENGTH_LONG).show();
    }

    public SearchEngine getSearchEngine() {
        return searchEngine;
    }

    public OfflineSearchEngine getOfflineSearchEngine() {
        return offlineSearchEngine;
    }

    public void onSearchButtonClicked(String query, boolean isSubmitted) {

        // Search for "Pizza" and show the results on the map.
        if (isSubmitted) {
            searchExample(query);
        } else {
            // Search for auto suggestions and log the results to the console.
            autoSuggestExample(query);
        }


    }

    public void onGeocodeButtonClicked() {
        // Search for the location that belongs to an address and show it on the map.
        geocodeAnAddress();
    }

    private void searchExample(String query) {
        String searchTerm = query;

        Toast.makeText(context, "Searching in viewport: " + searchTerm, Toast.LENGTH_LONG).show();
        searchInViewport(searchTerm);
    }

    private void geocodeAnAddress() {
        // Set map to expected location.
        GeoCoordinates geoCoordinates = new GeoCoordinates(52.53086, 13.38469);
        double distanceInMeters = 1000 * 7;
        MapMeasure mapMeasureZoom = new MapMeasure(MapMeasure.Kind.DISTANCE, distanceInMeters);
        camera.lookAt(geoCoordinates, mapMeasureZoom);
        String queryString = "Invalidenstraße 116, Berlin";

//        Toast.makeText(context,"Finding locations for: " + queryString
//                + ". Tap marker to see the coordinates. Check the logs for the address.", Toast.LENGTH_LONG).show();

        geocodeAddressAtLocation(queryString, geoCoordinates);
    }

    private void setTapGestureHandler() {
        mapView.getGestures().setTapListener(point2D -> pickMapMarker(point2D, markerDetailListener));
    }

    private void setLongPressGestureHandler() {
        mapView.getGestures().setLongPressListener((gestureState, touchPoint) -> {
            if (gestureState == GestureState.BEGIN) {
                GeoCoordinates geoCoordinates = mapView.viewToGeoCoordinates(touchPoint);
                if (geoCoordinates == null) {
                    return;
                }
//                addPoiMapMarker(geoCoordinates);
                getAddressForCoordinates(geoCoordinates);
            }
        });
    }

    private void getAddressForCoordinates(GeoCoordinates geoCoordinates) {
        SearchOptions reverseGeocodingOptions = new SearchOptions();
        reverseGeocodingOptions.languageCode = LanguageCode.EN_GB;
        reverseGeocodingOptions.maxItems = 1;

        if (isDeviceConnected()) {
            searchEngine.search(geoCoordinates, reverseGeocodingOptions, addressSearchCallback);
        } else {
            offlineSearchEngine.search(geoCoordinates, reverseGeocodingOptions, addressSearchCallback);
        }
    }

    private final SearchCallback addressSearchCallback = new SearchCallback() {
        @Override
        public void onSearchCompleted(@Nullable SearchError searchError, @Nullable List<Place> list) {
            if (searchError != null) {
                showDialog("Reverse geocoding", "Error: " + searchError.toString());
                return;
            }

            // If error is null, list is guaranteed to be not empty.
//            showDialog("Reverse geocoded address:", list.get(0).getAddress().addressText);
            Metadata metadata = new Metadata();
            metadata.setCustomValue("key_search_result", new SearchResultMetadata(list.get(0)));
            addPoiMapMarker(list.get(0).getGeoCoordinates(), metadata);
            showPoiDialog(context, list.get(0), markerDetailListener);
        }
    };

    private void pickMapMarker(final Point2D point2D, MarkerDetailListener placeDetailListener) {
        float radiusInPixel = 30;
        mapView.pickMapItems(point2D, radiusInPixel, pickMapItemsResult -> {
            if (pickMapItemsResult != null && (!pickMapItemsResult.getMarkers().isEmpty() || !pickMapItemsResult.getPolylines().isEmpty())) {

                List<MapMarker> mapMarkerList = pickMapItemsResult.getMarkers();
                if (!mapMarkerList.isEmpty()) {
                    MapMarker topmostMapMarker = mapMarkerList.get(0);

                    Metadata metadata = topmostMapMarker.getMetadata();
                    if (metadata != null) {
                        CustomMetadataValue customMetadataValue = metadata.getCustomValue("key_search_result");
                        if (customMetadataValue != null) {
                            SearchResultMetadata searchResultMetadata = (SearchResultMetadata) customMetadataValue;
                            showPoiDialog(context, searchResultMetadata.searchResult, placeDetailListener);
                            return;
                        }
                    }

                    showDialog("Picked Map Marker",
                            "Geographic coordinates: " +
                                    topmostMapMarker.getCoordinates().latitude + ", " +
                                    topmostMapMarker.getCoordinates().longitude);
                    return;
                }
                List<MapPolyline> mapPolyLinesList = pickMapItemsResult.getPolylines();
                if (mapPolyLinesList.isEmpty()) {
                    Log.e(LOG_TAG, "pickMapMarker: No map poly lines found.");
                    return;
                }
                placeDetailListener.onClickPolyLine(mapPolyLinesList);
//            _routesTabController.animateTo(_mapRoutes.indexOf(mapPolyLinesList.first));
            } else {
                Rectangle2D rectangle2D = new Rectangle2D(point2D, new Size2D(50, 50));
                mapView.pickMapContent(rectangle2D, new MapViewBase.PickMapContentCallback() {
                    @Override
                    public void onPickMapContent(@Nullable PickMapContentResult pickMapContentResult) {
                        if (pickMapContentResult == null) {
                            Log.e("onPickMapContent", "An error occurred while performing the pick operation.");
                            return;
                        }
                        List<PickMapContentResult.PoiResult> cartoPOIList = pickMapContentResult.getPois();
                        int listSize = cartoPOIList.size();
                        if (listSize == 0) {
                            return;
                        }

                        PickMapContentResult.PoiResult topmostCartoPOI = cartoPOIList.get(0);
                        geocodeAddressAtLocation(topmostCartoPOI.name, topmostCartoPOI.coordinates);
//                        showDialog("Carto POI picked:", topmostCartoPOI.name + ", Location: " +
//                                topmostCartoPOI.coordinates.latitude + ", " +
//                                topmostCartoPOI.coordinates.longitude + ". " +
//                                "See log for more place details.");
//
//                        // Now you can use the SearchEngine (via PickedPlace) or the OfflineSearchEngine
//                        // (via PickedPlace or offlineSearchId) to retrieve the Place object containing more details.
//                        // Below we use the offlineSearchId. Alternatively, you can use the
//                        // PickMapContentResult as data to create a PickedPlace object.
////                        fetchCartoPOIDetails(topmostCartoPOI.offlineSearchId);
//                        // Set null to get the results in their local language.
//                        LanguageCode languageCode = null;
//                        if (isDeviceConnected()){
//                            searchEngine.search(new PlaceIdQuery(topmostCartoPOI.placeCategoryId), languageCode, placeIdSearchCallback);
//                        }else {
//                            offlineSearchEngine.search(new PlaceIdQuery(topmostCartoPOI.offlineSearchId), languageCode, placeIdSearchCallback);
//                        }
                    }
                });
            }
        });
    }


    private void searchInViewport(String queryString) {
        clearMap();

//        GeoBox viewportGeoBox = getMapViewGeoBox();

        TextQuery query = new TextQuery(queryString, new TextQuery.Area(getMapViewCenter()));

        SearchOptions searchOptions = new SearchOptions();
        searchOptions.languageCode = LanguageCode.EN_US;
        searchOptions.maxItems = 100;

        if (isDeviceConnected()) {
            searchEngine.search(query, searchOptions, querySearchCallback);
        } else {
            offlineSearchEngine.search(query, searchOptions, querySearchCallback);
        }
    }

    private final SearchCallback querySearchCallback = new SearchCallback() {
        @Override
        public void onSearchCompleted(@Nullable SearchError searchError, @Nullable List<Place> list) {
            if (searchError != null) {
                // Note: When using the OfflineSearchEngine, the HERE SDK searches only on cached map data and
                // search results may not be available for all zoom levels.
                // Please also note that it may take time until the required map data is loaded.
                // Subsequently, the cache is filled when a user pans and zooms the map.
//                showDialog("Search", "Error: " + searchError.toString());
                return;
            }

            // If error is null, list is guaranteed to be not empty.
//            showDialog("Search", "Results: " + list.size());

            // Add new marker for each search result on map.
            ArrayList<PlaceData> places = new ArrayList<>();
            for (Place searchResult : list) {
                Metadata metadata = new Metadata();
                metadata.setCustomValue("key_search_result", new SearchResultMetadata(searchResult));
                // Note: getGeoCoordinates() may return null only for Suggestions.
                addPoiMapMarker(searchResult.getGeoCoordinates(), metadata);
                places.add(new PlaceData(searchResult.getTitle(), searchResult));
            }
            //show selected marker data
            if (context instanceof MainActivity) {
                markerDetailListener.onCategorySearched(list.get(0));
            }
            EventBus.getDefault().post(new GetPlacesEvent(places));
        }
    };

    public static class SearchResultMetadata implements CustomMetadataValue {

        public final Place searchResult;

        public SearchResultMetadata(Place searchResult) {
            this.searchResult = searchResult;
        }

        @NonNull
        @Override
        public String getTag() {
            return "SearchResult Metadata";
        }
    }

    private final SuggestCallback autosuggestCallback = new SuggestCallback() {
        @Override
        public void onSuggestCompleted(@Nullable SearchError searchError, @Nullable List<Suggestion> list) {
            if (searchError != null) {
                Log.d(LOG_TAG, "Autosuggest Error: " + searchError.name());
                return;
            }

            // If error is null, list is guaranteed to be not empty.
            Log.d(LOG_TAG, "Autosuggest results: " + list.size());
            ArrayList<PlaceData> places = new ArrayList<>();
            for (Suggestion autosuggestResult : list) {
                String addressText = "Not a place.";
                Place place = autosuggestResult.getPlace();
                if (place != null) {
                    addressText = place.getAddress().addressText;
                    places.add(new PlaceData(place.getTitle(), place));
                } else {
                    addressText = autosuggestResult.getTitle();
                    places.add(new PlaceData(autosuggestResult.getTitle(), place));
                }
                Log.d(LOG_TAG, "Autosuggest result: " + autosuggestResult.getTitle() +
                        " addressText: " + addressText);
            }
            EventBus.getDefault().post(new GetPlacesEvent(places));
        }
    };

    private final SearchCallback autoSearchCallback = new SearchCallback() {
        @Override
        public void onSearchCompleted(@Nullable SearchError searchError, @Nullable List<Place> list) {
            if (searchError != null) {
                Log.d(LOG_TAG, "Autosuggest Error: " + searchError.name());
                return;
            }

            // If error is null, list is guaranteed to be not empty.
            Log.d(LOG_TAG, "Autosuggest results: " + list.size());
            ArrayList<PlaceData> places = new ArrayList<>();
            for (Place place : list) {
//            String addressText = "Not a place.";
//            if (place != null) {
//                addressText = place.getAddress().addressText;
                places.add(new PlaceData(place.getTitle(), place));
//            } else {
//                addressText = autosuggestResult.getTitle();
//                places.add(new PlaceData(autosuggestResult.getTitle(), place));
//            }
                Log.d(LOG_TAG, "Autosuggest result: " + place.getTitle() +
                        " addressText: " + place.getAddress());
            }
            EventBus.getDefault().post(new GetPlacesEvent(places));
        }
    };

    private void autoSuggestExample(String query) {
        GeoCoordinates centerGeoCoordinates = getMapViewCenter();
        SearchOptions searchOptions = new SearchOptions();
        searchOptions.languageCode = LanguageCode.EN_US;
        searchOptions.maxItems = 100;
        TextQuery textQuery = new TextQuery(query, new TextQuery.Area(centerGeoCoordinates));
        if (isDeviceConnected()) {
            // Simulate a user typing a search term.
            searchEngine.suggest(
                    textQuery,
                    searchOptions,
                    autosuggestCallback);

//            searchEngine.suggest(
//                    new TextQuery("pi", // User typed "pi".
//                            centerGeoCoordinates),
//                    searchOptions,
//                    autosuggestCallback);
//
//            searchEngine.suggest(
//                    new TextQuery("piz", // User typed "piz".
//                            centerGeoCoordinates),
//                    searchOptions,
//                    autosuggestCallback);
        } else {
            // Simulate a user typing a search term.
            offlineSearchEngine.suggest(
                    textQuery,
                    searchOptions,
                    autosuggestCallback);
        }
    }

    private void geocodeAddressAtLocation(String queryString, GeoCoordinates geoCoordinates) {
        clearMap();

        AddressQuery query = new AddressQuery(queryString, geoCoordinates);

        SearchOptions options = new SearchOptions();
        options.languageCode = LanguageCode.EN_US;
        options.maxItems = 100;

        if (isDeviceConnected()) {
            searchEngine.search(query, options, geocodeAddressSearchCallback);
        } else {
            offlineSearchEngine.search(query, options, geocodeAddressSearchCallback);
        }
    }

    private final SearchCallback geocodeAddressSearchCallback = new SearchCallback() {
        @Override
        public void onSearchCompleted(SearchError searchError, List<Place> list) {
            if (searchError != null) {
                showDialog("Geocoding", "Error: " + searchError.toString());
                return;
            }

            for (Place geocodingResult : list) {
                // Note: getGeoCoordinates() may return null only for Suggestions.
                GeoCoordinates geoCoordinates = geocodingResult.getGeoCoordinates();
                Address address = geocodingResult.getAddress();
                String locationDetails = address.addressText
                        + ". GeoCoordinates: " + geoCoordinates.latitude
                        + ", " + geoCoordinates.longitude;

                Log.d(LOG_TAG, "GeocodingResult: " + locationDetails);
                Metadata metadata = new Metadata();
                metadata.setCustomValue("key_search_result", new SearchResultMetadata(list.get(0)));
                addPoiMapMarker(geoCoordinates, metadata);
            }

//            showDialog("Geocoding result", "Size: " + list.size());
        }
    };

//    public void addPoiMapMarker(GeoCoordinates geoCoordinates) {
//        MapMarker mapMarker = createPoiMapMarker(geoCoordinates);
//        mapView.getMapScene().addMapMarker(mapMarker);
//        mapMarkerList.add(mapMarker);
//    }

    public void addPoiMapMarker(GeoCoordinates geoCoordinates, Metadata metadata) {
        MapMarker mapMarker = createPoiMapMarker(geoCoordinates);
        mapMarker.setMetadata(metadata);
        mapView.getMapScene().addMapMarker(mapMarker);
        mapMarkerList.add(mapMarker);
    }

    private MapMarker createPoiMapMarker(GeoCoordinates geoCoordinates) {
        MapImage mapImage = MapImageFactory.fromResource(context.getResources(), R.drawable.poi);
        return new MapMarker(geoCoordinates, mapImage, new Anchor2D(0.5F, 1));
    }

    public GeoCoordinates getMapViewCenter() {
        return mapView.getCamera().getState().targetCoordinates;
    }

    public GeoBox getMapViewGeoBox() {
        int mapViewWidthInPixels = mapView.getWidth();
        int mapViewHeightInPixels = mapView.getHeight();
        Point2D bottomLeftPoint2D = new Point2D(0, mapViewHeightInPixels);
        Point2D topRightPoint2D = new Point2D(mapViewWidthInPixels, 0);

        GeoCoordinates southWestCorner = mapView.viewToGeoCoordinates(bottomLeftPoint2D);
        GeoCoordinates northEastCorner = mapView.viewToGeoCoordinates(topRightPoint2D);

        Log.d(TAG, "southWestCorner: " + southWestCorner);
        Log.d(TAG, "northEastCorner: " + northEastCorner);
        if (southWestCorner == null || northEastCorner == null) {
            throw new RuntimeException("GeoBox creation failed, corners are null.");
        }

        // Note: This algorithm assumes an unrotated map view.
        //System.out.print(new GeoBox(southWestCorner, northEastCorner));
        return new GeoBox(southWestCorner, northEastCorner);
    }

    private void clearMap() {
        for (MapMarker mapMarker : mapMarkerList) {
            mapView.getMapScene().removeMapMarker(mapMarker);
        }
        mapMarkerList.clear();
    }

//    public void onSwitchOnlineButtonClicked() {
//        isDeviceConnected = true;
//        Toast.makeText(context,"The app will now use the SearchEngine.", Toast.LENGTH_LONG).show();
//    }
//
//    public void onSwitchOfflineButtonClicked() {
//        isDeviceConnected = false;
//        Toast.makeText(context,"The app will now use the OfflineSearchEngine.", Toast.LENGTH_LONG).show();
//    }

    public boolean isDeviceConnected() {
//        // An application may define here a logic to determine whether a device is connected or not.
//        // For this example app, the flag is set from UI.
//        return isDeviceConnected && !isOfflineModeEnabled;
        ConnectivityManager connectivityManager = ((ConnectivityManager) context.getSystemService(Context.CONNECTIVITY_SERVICE));
        boolean isDeviceConnected = connectivityManager.getActiveNetworkInfo() != null && connectivityManager.getActiveNetworkInfo().isConnected();
        return isDeviceConnected && !SharedPref.getInstance().getOfflineMode();
    }

    private void showPoiDialog(Context context, Place metadata, MarkerDetailListener placeDetailListener) {
        String title = metadata.getTitle();
        String addressText = metadata.getAddress().addressText;
        List<Contact> contacts = metadata.getDetails().contacts;
        POIPaymentDetails paymentDetails = metadata.getDetails().payment;
        ArrayList<String> phones = new ArrayList<>();
        ArrayList<String> emails = new ArrayList<>();
        ArrayList<String> websites = new ArrayList<>();
        if (!contacts.isEmpty()) {
            if (!contacts.get(0).mobilePhones.isEmpty()) {
                for (MobilePhone mobilePhone : contacts.get(0).mobilePhones) {
                    phones.add(mobilePhone.phoneNumber);
                }
            }
            if (!contacts.get(0).landlinePhones.isEmpty()) {
                for (LandlinePhone landlinePhone : contacts.get(0).landlinePhones) {
                    phones.add(landlinePhone.phoneNumber);
                }
            }
            if (!contacts.get(0).emails.isEmpty()) {
                for (EmailAddress emailAddress : contacts.get(0).emails) {
                    emails.add(emailAddress.address);
                }
            }
            if (!contacts.get(0).websites.isEmpty()) {
                for (WebsiteAddress websiteAddress : contacts.get(0).websites) {
                    websites.add(websiteAddress.address);
                }
            }
        }

        AlertDialog.Builder builder = new AlertDialog.Builder(context);
        builder.setCancelable(false);
        LayoutInflater inflater = (LayoutInflater) context.getSystemService(LAYOUT_INFLATER_SERVICE);
        @SuppressLint("InflateParams")
        View view = inflater.inflate(R.layout.view_poi_dialog, null);
        builder.setView(view);
        AlertDialog dialog = builder.create();
        Objects.requireNonNull(dialog.getWindow()).setBackgroundDrawable(new ColorDrawable(Color.TRANSPARENT));
        dialog.show();
        ImageView closeIV = view.findViewById(R.id.close_img);
        TextView textViewDialogTitle = view.findViewById(R.id.textViewDialogTitle);
        TextView addressTV = view.findViewById(R.id.address_txt);
        TextView paymentTV = view.findViewById(R.id.payment_txt);
        RecyclerView phonesRV = view.findViewById(R.id.phonesRV);
        RecyclerView emailsRV = view.findViewById(R.id.emailsRV);
        RecyclerView websitesRV = view.findViewById(R.id.websitesRV);
        LinearLayout routeLL = view.findViewById(R.id.routeLL);
        LinearLayout addWaypointLL = view.findViewById(R.id.addWaypointLL);
        textViewDialogTitle.setText(title);
        addressTV.setText(addressText);
        if (paymentDetails != null) {
            paymentTV.setText("Payment Details: " + StringUtils.joinWith(", ", paymentDetails.acceptedMethods));
            paymentTV.setVisibility(View.VISIBLE);
        } else {
            paymentTV.setVisibility(View.GONE);
        }
        //phone list
        phonesRV.setHasFixedSize(true);
        phonesRV.setLayoutManager(new LinearLayoutManager(context));
        phonesRV.setAdapter(new ItemsAdapter(context, phones, ItemType.PHONES, (view12, position) -> {
            Uri call = Uri.parse("tel:" + phones.get(position));
            context.startActivity(new Intent(Intent.ACTION_DIAL, call));
        }));
        //emails list
        emailsRV.setHasFixedSize(true);
        emailsRV.setLayoutManager(new LinearLayoutManager(context));
        emailsRV.setAdapter(new ItemsAdapter(context, emails, ItemType.EMAILS, (view12, position) -> {
            Uri email = Uri.parse(emails.get(position));
            context.startActivity(Intent.createChooser(new Intent(Intent.ACTION_SENDTO, email), "Send email.."));
        }));
        //websites list
        websitesRV.setHasFixedSize(true);
        websitesRV.setLayoutManager(new LinearLayoutManager(context));
        websitesRV.setAdapter(new ItemsAdapter(context, websites, ItemType.WEBSITES, (view12, position) -> {
            Uri website = Uri.parse(websites.get(position));
            context.startActivity(new Intent(Intent.ACTION_VIEW, website));
        }));
        closeIV.setOnClickListener(view1 -> {
            if (dialog.isShowing()) {
                dialog.dismiss();
            }
        });
        routeLL.setOnClickListener(view1 -> {
            placeDetailListener.onClickRoute(metadata);
            if (dialog.isShowing()) {
                dialog.dismiss();
            }
        });
        addWaypointLL.setVisibility(context instanceof MainActivity ? View.GONE : View.VISIBLE);
        addWaypointLL.setOnClickListener(view1 -> {
            placeDetailListener.onClickAddWaypoint(metadata);
            if (dialog.isShowing()) {
                dialog.dismiss();
            }
        });

    }

    private void showDialog(String title, String message) {
        AlertDialog.Builder builder = new AlertDialog.Builder(context);
        builder.setTitle(title);
        builder.setMessage(message);
        builder.show();
    }

    public interface MarkerDetailListener {
        void onClickPolyLine(List<MapPolyline> mapPolylines);

        void onClickRoute(Place place);

        void onClickRouteNew(Place place);

        void onClickAddWaypoint(Place place);

        void onCategorySearched(Place place);
    }
}
