package com.example.heresdkandroidref.fragments;

import android.app.Activity;
import android.app.DatePickerDialog;
import android.app.TimePickerDialog;
import android.content.Context;
import android.content.Intent;
import android.os.Bundle;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.appcompat.widget.SwitchCompat;
import androidx.fragment.app.Fragment;

import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.AdapterView;
import android.widget.DatePicker;
import android.widget.LinearLayout;
import android.widget.TimePicker;
import android.widget.Toast;

import com.example.heresdkandroidref.R;
import com.example.heresdkandroidref.activity.AvoidCountriesActivity;
import com.example.heresdkandroidref.activity.HazardousGoodsActivity;
import com.example.heresdkandroidref.activity.RoadFeaturesActivity;
import com.example.heresdkandroidref.activity.TruckSpecificationsActivity;
import com.example.heresdkandroidref.utils.SharedPref;
import com.example.heresdkandroidref.utils.Utils;
import com.google.android.material.textfield.MaterialAutoCompleteTextView;
import com.google.android.material.textfield.TextInputEditText;
import com.google.gson.Gson;
import com.here.sdk.core.CountryCode;
import com.here.sdk.core.LanguageCode;
import com.here.sdk.core.TextFormat;
import com.here.sdk.core.UnitSystem;
import com.here.sdk.routing.AvoidanceOptions;
import com.here.sdk.routing.CarOptions;
import com.here.sdk.routing.OptimizationMode;
import com.here.sdk.routing.PedestrianOptions;
import com.here.sdk.routing.RoadFeatures;
import com.here.sdk.routing.RouteOptions;
import com.here.sdk.routing.RouteTextOptions;
import com.here.sdk.routing.ScooterOptions;
import com.here.sdk.routing.TruckOptions;
import com.here.sdk.transport.HazardousMaterial;
import com.here.sdk.transport.TruckSpecifications;
import com.here.sdk.transport.TunnelCategory;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Calendar;
import java.util.List;
import java.util.Locale;
import java.util.Map;

public class WalkPreferenceFragment extends Fragment implements View.OnClickListener, DatePickerDialog.OnDateSetListener, TimePickerDialog.OnTimeSetListener, AdapterView.OnItemClickListener {

    private Context mContext;
    private MaterialAutoCompleteTextView optimizationModeACTV, textFormatACTV, unitSystemACTV, languageCodeACTV, tunnelCategoryACTV;
    private TextInputEditText departureTimeET, routeAlternativesET, walkSpeedET;
    private LinearLayout avoidLayoutLL, avoidRoadFeatureLL, avoidCountriesLL, walkLayoutLL, specificationsLL, hazardousGoodsLL, highwayLayoutLL, truckLayoutLL;
    private SwitchCompat allowHighwaySB;
    final Calendar myCalendar = Calendar.getInstance();
    Map<String, OptimizationMode> optimizationModeMap = Utils.getOptimizationModeMap();
    Map<String, TextFormat> textFormatMap = Utils.getTextFormatMap();
    Map<String, UnitSystem> unitSystemMap = Utils.getUnitSystemMap();
    Map<String, LanguageCode> languageCodeMap = Utils.getLanguageCodeMap();
    Map<String, TunnelCategory> tunnelCategoryMap = Utils.getTunnelCategoryMap();
    Map<String, RoadFeatures> roadFeaturesMap = Utils.getRoadFeaturesMap();
    Map<String, CountryCode> countryCodeMap = Utils.getCountryCodeMap();
    Map<String, HazardousMaterial> hazardousGoodsMap = Utils.getHazardousGoodsMap();
    ArrayList<String> roadFeatures = new ArrayList<>();
    ArrayList<String> countryCodes = new ArrayList<>();
    ArrayList<String> hazardousMaterials = new ArrayList<>();
    TruckSpecifications truckSpecifications = new TruckSpecifications();
    private int preferenceType = 0;

    @Override
    public void onAttach(@NonNull Context context) {
        super.onAttach(context);
        mContext = context;
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
        return inflater.inflate(R.layout.fragment_walk_preference, container, false);
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        preferenceType = getArguments().getInt("preferenceType",0);
        initViews(view);
    }

    private void initViews(View view) {
        routeAlternativesET = view.findViewById(R.id.routeAlternativesET);
        departureTimeET = view.findViewById(R.id.departureTimeET);
        textFormatACTV = view.findViewById(R.id.textFormatACTV);
        unitSystemACTV = view.findViewById(R.id.unitSystemACTV);
        languageCodeACTV = view.findViewById(R.id.languageCodeACTV);
        optimizationModeACTV = view.findViewById(R.id.optimizationModeACTV);
        tunnelCategoryACTV = view.findViewById(R.id.tunnelCategoryACTV);
        walkSpeedET = view.findViewById(R.id.walkSpeedET);
        avoidLayoutLL = view.findViewById(R.id.avoidLayoutLL);
        avoidRoadFeatureLL = view.findViewById(R.id.avoidRoadFeatureLL);
        avoidCountriesLL = view.findViewById(R.id.avoidCountriesLL);
        walkLayoutLL = view.findViewById(R.id.walkLayoutLL);
        specificationsLL = view.findViewById(R.id.specificationsLL);
        hazardousGoodsLL = view.findViewById(R.id.hazardousGoodsLL);
        allowHighwaySB = view.findViewById(R.id.allowHighwaySB);
        truckLayoutLL = view.findViewById(R.id.truckLayoutLL);
        highwayLayoutLL = view.findViewById(R.id.highwayLayoutLL);
        //set items
        optimizationModeACTV.setSimpleItems(optimizationModeMap.keySet().toArray(new String[0]));
        textFormatACTV.setSimpleItems(textFormatMap.keySet().toArray(new String[0]));
        unitSystemACTV.setSimpleItems(unitSystemMap.keySet().toArray(new String[0]));
        languageCodeACTV.setSimpleItems(languageCodeMap.keySet().toArray(new String[0]));
        languageCodeACTV.setSimpleItems(languageCodeMap.keySet().toArray(new String[0]));
        tunnelCategoryACTV.setSimpleItems(tunnelCategoryMap.keySet().toArray(new String[0]));
        //select listener
        optimizationModeACTV.setOnItemClickListener(this);
        textFormatACTV.setOnItemClickListener(this);
        unitSystemACTV.setOnItemClickListener(this);
        languageCodeACTV.setOnItemClickListener(this);
        //click listener
        departureTimeET.setOnClickListener(this);
        avoidRoadFeatureLL.setOnClickListener(this);
        hazardousGoodsLL.setOnClickListener(this);
        avoidCountriesLL.setOnClickListener(this);
        specificationsLL.setOnClickListener(this);
        //setup views
        if(preferenceType==0){
            CarOptions options = SharedPref.getInstance().getCarPreferenceData();
            routeAlternativesET.setText(String.valueOf(options.routeOptions.alternatives > 0 ? options.routeOptions.alternatives : 1));
            optimizationModeACTV.setText(optimizationModeACTV.getAdapter().getItem(new ArrayList<>(optimizationModeMap.values()).indexOf(options.routeOptions.optimizationMode)).toString(), false);
            if (options.routeOptions.departureTime != null) {
                myCalendar.setTime(options.routeOptions.departureTime);
                departureTimeET.setText(Utils.getFormattedDateTime(myCalendar.getTime()));
            }
            textFormatACTV.setText(textFormatACTV.getAdapter().getItem(new ArrayList<>(textFormatMap.values()).indexOf(options.textOptions.instructionFormat)).toString(), false);
            unitSystemACTV.setText(unitSystemACTV.getAdapter().getItem(new ArrayList<>(unitSystemMap.values()).indexOf(options.textOptions.unitSystem)).toString(), false);
            languageCodeACTV.setText(languageCodeACTV.getAdapter().getItem(new ArrayList<>(languageCodeMap.values()).indexOf(options.textOptions.language)).toString(), false);
            //list
            ArrayList<String> selectedRoadFeaturesList = new ArrayList<>();
            for (RoadFeatures roadFeature : options.avoidanceOptions.roadFeatures) {
                selectedRoadFeaturesList.add(new ArrayList<>(roadFeaturesMap.keySet()).get(new ArrayList<>(roadFeaturesMap.values()).indexOf(roadFeature)));
            }
            roadFeatures = selectedRoadFeaturesList;
            ArrayList<String> selectedCountries = new ArrayList<>();
            for (CountryCode countryCode : options.avoidanceOptions.countries) {
                selectedCountries.add(new ArrayList<>(countryCodeMap.keySet()).get(new ArrayList<>(countryCodeMap.values()).indexOf(countryCode)));
            }
            countryCodes = selectedCountries;
            //setup views
            avoidLayoutLL.setVisibility(View.VISIBLE);
            truckLayoutLL.setVisibility(View.GONE);
            walkLayoutLL.setVisibility(View.GONE);
            highwayLayoutLL.setVisibility(View.GONE);
        }else if(preferenceType==1){
            TruckOptions options = SharedPref.getInstance().getTruckPreferenceData();
            routeAlternativesET.setText(String.valueOf(options.routeOptions.alternatives > 0 ? options.routeOptions.alternatives : 1));
            optimizationModeACTV.setText(optimizationModeACTV.getAdapter().getItem(new ArrayList<>(optimizationModeMap.values()).indexOf(options.routeOptions.optimizationMode)).toString(), false);
            if (options.routeOptions.departureTime != null) {
                myCalendar.setTime(options.routeOptions.departureTime);
                departureTimeET.setText(Utils.getFormattedDateTime(myCalendar.getTime()));
            }
            textFormatACTV.setText(textFormatACTV.getAdapter().getItem(new ArrayList<>(textFormatMap.values()).indexOf(options.textOptions.instructionFormat)).toString(), false);
            unitSystemACTV.setText(unitSystemACTV.getAdapter().getItem(new ArrayList<>(unitSystemMap.values()).indexOf(options.textOptions.unitSystem)).toString(), false);
            languageCodeACTV.setText(languageCodeACTV.getAdapter().getItem(new ArrayList<>(languageCodeMap.values()).indexOf(options.textOptions.language)).toString(), false);
            //list
            ArrayList<String> selectedRoadFeaturesList = new ArrayList<>();
            for (RoadFeatures roadFeature : options.avoidanceOptions.roadFeatures) {
                selectedRoadFeaturesList.add(new ArrayList<>(roadFeaturesMap.keySet()).get(new ArrayList<>(roadFeaturesMap.values()).indexOf(roadFeature)));
            }
            roadFeatures = selectedRoadFeaturesList;
            ArrayList<String> selectedCountries = new ArrayList<>();
            for (CountryCode countryCode : options.avoidanceOptions.countries) {
                if (new ArrayList<>(countryCodeMap.values()).contains(countryCode)) {
                    selectedCountries.add(new ArrayList<>(countryCodeMap.keySet()).get(new ArrayList<>(countryCodeMap.values()).indexOf(countryCode)));
                }
            }
            countryCodes = selectedCountries;
            truckSpecifications = options.truckSpecifications;
            ArrayList<String> selectedHazardousGoods = new ArrayList<>();
            for (HazardousMaterial hazardousMaterial : options.hazardousMaterials) {
                selectedHazardousGoods.add(new ArrayList<>(hazardousGoodsMap.keySet()).get(new ArrayList<>(hazardousGoodsMap.values()).indexOf(hazardousMaterial)));
            }
            countryCodes = selectedHazardousGoods;
            if(options.linkTunnelCategory!=null) {
                tunnelCategoryACTV.setText(tunnelCategoryACTV.getAdapter().getItem(new ArrayList<>(tunnelCategoryMap.values()).indexOf(options.linkTunnelCategory)).toString(), false);
            }
            //setup views
            avoidLayoutLL.setVisibility(View.VISIBLE);
            truckLayoutLL.setVisibility(View.VISIBLE);
            walkLayoutLL.setVisibility(View.GONE);
            highwayLayoutLL.setVisibility(View.GONE);
        }else if(preferenceType==2){
            ScooterOptions options = SharedPref.getInstance().getScooterPreferenceData();
            routeAlternativesET.setText(String.valueOf(options.routeOptions.alternatives > 0 ? options.routeOptions.alternatives : 1));
            optimizationModeACTV.setText(optimizationModeACTV.getAdapter().getItem(new ArrayList<>(optimizationModeMap.values()).indexOf(options.routeOptions.optimizationMode)).toString(), false);
            if (options.routeOptions.departureTime != null) {
                myCalendar.setTime(options.routeOptions.departureTime);
                departureTimeET.setText(Utils.getFormattedDateTime(myCalendar.getTime()));
            }
            textFormatACTV.setText(textFormatACTV.getAdapter().getItem(new ArrayList<>(textFormatMap.values()).indexOf(options.textOptions.instructionFormat)).toString(), false);
            unitSystemACTV.setText(unitSystemACTV.getAdapter().getItem(new ArrayList<>(unitSystemMap.values()).indexOf(options.textOptions.unitSystem)).toString(), false);
            languageCodeACTV.setText(languageCodeACTV.getAdapter().getItem(new ArrayList<>(languageCodeMap.values()).indexOf(options.textOptions.language)).toString(), false);
            //list
            ArrayList<String> selectedRoadFeaturesList = new ArrayList<>();
            for (RoadFeatures roadFeature : options.avoidanceOptions.roadFeatures) {
                selectedRoadFeaturesList.add(new ArrayList<>(roadFeaturesMap.keySet()).get(new ArrayList<>(roadFeaturesMap.values()).indexOf(roadFeature)));
            }
            roadFeatures = selectedRoadFeaturesList;
            ArrayList<String> selectedCountries = new ArrayList<>();
            for (CountryCode countryCode : options.avoidanceOptions.countries) {
                selectedCountries.add(new ArrayList<>(countryCodeMap.keySet()).get(new ArrayList<>(countryCodeMap.values()).indexOf(countryCode)));
            }
            countryCodes = selectedCountries;
            allowHighwaySB.setChecked(options.allowHighway);
            //setup views
            avoidLayoutLL.setVisibility(View.VISIBLE);
            truckLayoutLL.setVisibility(View.GONE);
            walkLayoutLL.setVisibility(View.GONE);
            highwayLayoutLL.setVisibility(View.VISIBLE);
        }else {
            PedestrianOptions options = SharedPref.getInstance().getWalkPreferenceData();
            routeAlternativesET.setText(String.valueOf(options.routeOptions.alternatives > 0 ? options.routeOptions.alternatives : 1));
            optimizationModeACTV.setText(optimizationModeACTV.getAdapter().getItem(new ArrayList<>(optimizationModeMap.values()).indexOf(options.routeOptions.optimizationMode)).toString(), false);
            if (options.routeOptions.departureTime != null) {
                myCalendar.setTime(options.routeOptions.departureTime);
                departureTimeET.setText(Utils.getFormattedDateTime(myCalendar.getTime()));
            }
            textFormatACTV.setText(textFormatACTV.getAdapter().getItem(new ArrayList<>(textFormatMap.values()).indexOf(options.textOptions.instructionFormat)).toString(), false);
            unitSystemACTV.setText(unitSystemACTV.getAdapter().getItem(new ArrayList<>(unitSystemMap.values()).indexOf(options.textOptions.unitSystem)).toString(), false);
            languageCodeACTV.setText(languageCodeACTV.getAdapter().getItem(new ArrayList<>(languageCodeMap.values()).indexOf(options.textOptions.language)).toString(), false);
            walkSpeedET.setText(String.valueOf(options.walkSpeedInMetersPerSecond));
            //setup views
            avoidLayoutLL.setVisibility(View.GONE);
            truckLayoutLL.setVisibility(View.GONE);
            walkLayoutLL.setVisibility(View.GONE);
            highwayLayoutLL.setVisibility(View.VISIBLE);
        }
    }

    @Override
    public void onClick(View view) {
        switch (view.getId()) {
            case R.id.departureTimeET:
                DatePickerDialog datePickerDialog = new DatePickerDialog(mContext, this, myCalendar.get(Calendar.YEAR), myCalendar.get(Calendar.MONTH), myCalendar.get(Calendar.DAY_OF_MONTH));
                datePickerDialog.getDatePicker().setMinDate(Calendar.getInstance().getTimeInMillis());
                Calendar maxDate = Calendar.getInstance();
                maxDate.set(Calendar.YEAR, maxDate.get(Calendar.YEAR) + 5);
                datePickerDialog.getDatePicker().setMaxDate(maxDate.getTimeInMillis());
                datePickerDialog.show();
                break;
            case R.id.avoidRoadFeatureLL:
                startActivityForResult(new Intent(mContext, RoadFeaturesActivity.class).putStringArrayListExtra("roadFeatures", roadFeatures), 3311);
                break;
            case R.id.hazardousGoodsLL:
                startActivityForResult(new Intent(mContext, HazardousGoodsActivity.class).putStringArrayListExtra("hazardousGoods", hazardousMaterials), 3322);
                break;
            case R.id.avoidCountriesLL:
                startActivityForResult(new Intent(mContext, AvoidCountriesActivity.class).putStringArrayListExtra("countries", countryCodes), 3333);
                break;
            case R.id.specificationsLL:
                startActivityForResult(new Intent(mContext, TruckSpecificationsActivity.class).putExtra("truckSpecifications", new Gson().toJson(truckSpecifications)), 3344);
                break;
        }
    }

    @Override
    public void onDateSet(DatePicker datePicker, int year, int month, int day) {
        myCalendar.set(Calendar.YEAR, year);
        myCalendar.set(Calendar.MONTH, month);
        myCalendar.set(Calendar.DAY_OF_MONTH, day);
        //init time picker
        new TimePickerDialog(mContext, this, myCalendar.get(Calendar.HOUR_OF_DAY), myCalendar.get(Calendar.MINUTE), false).show();
    }

    @Override
    public void onTimeSet(TimePicker timePicker, int hourOfDay, int minute) {
        myCalendar.set(Calendar.HOUR_OF_DAY, hourOfDay);
        myCalendar.set(Calendar.MINUTE, minute);
        //update departure time
        departureTimeET.setText(Utils.getFormattedDateTime(myCalendar.getTime()));
        //saveWalkPreference();
    }

    @Override
    public void onActivityResult(int requestCode, int resultCode, @Nullable Intent data) {
        super.onActivityResult(requestCode, resultCode, data);
        if (resultCode == Activity.RESULT_OK) {
            switch (requestCode) {
                case 3311:
                    roadFeatures = data != null ? data.getStringArrayListExtra("roadFeatures") : new ArrayList<>();
                    break;
                case 3322:
                    hazardousMaterials = data != null ? data.getStringArrayListExtra("hazardousGoods") : new ArrayList<>();
                    break;
                case 3333:
                    countryCodes = data != null ? data.getStringArrayListExtra("countries") : new ArrayList<>();
                    break;
                case 3344:
                    String specificationData = data != null ? data.getStringExtra("truckSpecifications") : "";
                    truckSpecifications = !specificationData.isEmpty() ? new Gson().fromJson(specificationData, TruckSpecifications.class) : new TruckSpecifications();
                    break;
            }
        }
    }

    @Override
    public void onStop() {
        saveRoutePreference();
        super.onStop();
    }

    private void saveRoutePreference() {
        switch (preferenceType) {
            case 0:
                saveCarPreference();
                break;
            case 1:
                saveTruckPreference();
                break;
            case 2:
                saveScooterPreference();
                break;
            default:
                saveWalkPreference();
                break;
        }
    }

    private void saveCarPreference() {
        int alternativeRoute = Integer.parseInt(routeAlternativesET.getText() != null ? routeAlternativesET.getText().toString() : "1");
        CarOptions options = new CarOptions();
        options.routeOptions = new RouteOptions(optimizationModeMap.get(optimizationModeACTV.getText().toString()), alternativeRoute, departureTimeET.getText().toString().isEmpty() ? null : myCalendar.getTime());
        RouteTextOptions routeTextOptions = new RouteTextOptions();
        routeTextOptions.language = languageCodeMap.get(languageCodeACTV.getText().toString());
        routeTextOptions.instructionFormat = textFormatMap.get(textFormatACTV.getText().toString());
        routeTextOptions.unitSystem = unitSystemMap.get(unitSystemACTV.getText().toString());
        options.textOptions = routeTextOptions;
        AvoidanceOptions avoidanceOptions = new AvoidanceOptions();
        List<RoadFeatures> selectedRoadFeaturesList = new ArrayList<>();
        for (String roadFeature : roadFeatures) {
            selectedRoadFeaturesList.add(roadFeaturesMap.get(roadFeature));
        }
        avoidanceOptions.roadFeatures = selectedRoadFeaturesList;
        List<CountryCode> selectedCountryCodesList = new ArrayList<>();
        for (String countryCode : countryCodes) {
            selectedCountryCodesList.add(countryCodeMap.get(countryCode));
        }
        avoidanceOptions.countries = selectedCountryCodesList;
        options.avoidanceOptions = avoidanceOptions;
        SharedPref.getInstance().saveCarPreferenceData(options);
    }

    private void saveTruckPreference() {
        int alternativeRoute = Integer.parseInt(routeAlternativesET.getText() != null ? routeAlternativesET.getText().toString() : "1");
        TruckOptions options = new TruckOptions();
        options.routeOptions = new RouteOptions(optimizationModeMap.get(optimizationModeACTV.getText().toString()), alternativeRoute, departureTimeET.getText().toString().isEmpty() ? null : myCalendar.getTime());
        RouteTextOptions routeTextOptions = new RouteTextOptions();
        routeTextOptions.language = languageCodeMap.get(languageCodeACTV.getText().toString());
        routeTextOptions.instructionFormat = textFormatMap.get(textFormatACTV.getText().toString());
        routeTextOptions.unitSystem = unitSystemMap.get(unitSystemACTV.getText().toString());
        options.textOptions = routeTextOptions;
        AvoidanceOptions avoidanceOptions = new AvoidanceOptions();
        List<RoadFeatures> selectedRoadFeaturesList = new ArrayList<>();
        for (String roadFeature : roadFeatures) {
            selectedRoadFeaturesList.add(roadFeaturesMap.get(roadFeature));
        }
        avoidanceOptions.roadFeatures = selectedRoadFeaturesList;
        List<CountryCode> selectedCountryCodesList = new ArrayList<>();
        for (String countryCode : countryCodes) {
            selectedCountryCodesList.add(countryCodeMap.get(countryCode));
        }
        avoidanceOptions.countries = selectedCountryCodesList;
        options.avoidanceOptions = avoidanceOptions;
        List<HazardousMaterial> selectedHazardousMaterialsList = new ArrayList<>();
        for (String hazardousMaterials : hazardousMaterials) {
            selectedHazardousMaterialsList.add(hazardousGoodsMap.get(hazardousMaterials));
        }
        options.hazardousMaterials = selectedHazardousMaterialsList;
        options.truckSpecifications = truckSpecifications;
        options.linkTunnelCategory = tunnelCategoryMap.get(tunnelCategoryACTV.getText().toString());
        SharedPref.getInstance().saveTruckPreferenceData(options);
    }

    private void saveScooterPreference() {
        int alternativeRoute = Integer.parseInt(routeAlternativesET.getText() != null ? routeAlternativesET.getText().toString() : "1");
        ScooterOptions options = new ScooterOptions();
        options.routeOptions = new RouteOptions(optimizationModeMap.get(optimizationModeACTV.getText().toString()), alternativeRoute, departureTimeET.getText().toString().isEmpty() ? null : myCalendar.getTime());
        RouteTextOptions routeTextOptions = new RouteTextOptions();
        routeTextOptions.language = languageCodeMap.get(languageCodeACTV.getText().toString());
        routeTextOptions.instructionFormat = textFormatMap.get(textFormatACTV.getText().toString());
        routeTextOptions.unitSystem = unitSystemMap.get(unitSystemACTV.getText().toString());
        options.textOptions = routeTextOptions;
        AvoidanceOptions avoidanceOptions = new AvoidanceOptions();
        List<RoadFeatures> selectedRoadFeaturesList = new ArrayList<>();
        for (String roadFeature : roadFeatures) {
            selectedRoadFeaturesList.add(roadFeaturesMap.get(roadFeature));
        }
        avoidanceOptions.roadFeatures = selectedRoadFeaturesList;
        List<CountryCode> selectedCountryCodesList = new ArrayList<>();
        for (String countryCode : countryCodes) {
            selectedCountryCodesList.add(countryCodeMap.get(countryCode));
        }
        avoidanceOptions.countries = selectedCountryCodesList;
        options.avoidanceOptions = avoidanceOptions;
        options.allowHighway = allowHighwaySB.isChecked();
        SharedPref.getInstance().saveScooterPreferenceData(options);
    }

    private void saveWalkPreference() {
        int alternativeRoute = Integer.parseInt(routeAlternativesET.getText() != null ? routeAlternativesET.getText().toString() : "1");
        double walkSpeedInMeterPerSecond = Double.parseDouble(walkSpeedET.getText() != null ? walkSpeedET.getText().toString() : "1.0");
        PedestrianOptions options = new PedestrianOptions();
        options.walkSpeedInMetersPerSecond = walkSpeedInMeterPerSecond;
        options.routeOptions = new RouteOptions(optimizationModeMap.get(optimizationModeACTV.getText().toString()), alternativeRoute, departureTimeET.getText().toString().isEmpty() ? null : myCalendar.getTime());
        RouteTextOptions routeTextOptions = new RouteTextOptions();
        routeTextOptions.language = languageCodeMap.get(languageCodeACTV.getText().toString());
        routeTextOptions.instructionFormat = textFormatMap.get(textFormatACTV.getText().toString());
        routeTextOptions.unitSystem = unitSystemMap.get(unitSystemACTV.getText().toString());
        options.textOptions = routeTextOptions;
        SharedPref.getInstance().saveWalkPreferenceData(options);
    }

    @Override
    public void onItemClick(AdapterView<?> adapterView, View view, int i, long l) {
//        saveWalkPreference();
    }


}