package com.example.heresdkandroidref.utils;

import android.content.Context;
import android.content.SharedPreferences;
import android.util.Log;

import com.example.heresdkandroidref.models.PlaceData;
import com.google.gson.Gson;
import com.google.gson.GsonBuilder;
import com.google.gson.JsonDeserializationContext;
import com.google.gson.JsonDeserializer;
import com.google.gson.JsonElement;
import com.google.gson.JsonObject;
import com.google.gson.JsonParseException;
import com.google.gson.JsonSerializer;
import com.here.sdk.routing.CarOptions;
import com.here.sdk.routing.PedestrianOptions;
import com.here.sdk.routing.ScooterOptions;
import com.here.sdk.routing.TruckOptions;
import com.here.sdk.search.Place;
import com.here.sdk.search.PlaceSerializationException;

import java.lang.reflect.Type;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

public class SharedPref {
    private static SharedPref ourInstance;
    private SharedPreferences preferences;

    public static SharedPref getInstance() {
        return ourInstance;
    }

    public SharedPref(Context context) {
        ourInstance = this;
        preferences = context.getSharedPreferences(Constant.SHARED_PREF_NAME, Context.MODE_PRIVATE);
    }

    public ArrayList<PlaceData> getRecentPlacesList() throws PlaceSerializationException {
        ArrayList<PlaceData> recentPlaces = new ArrayList<>();
        Set<String> placeTextSet = preferences.getStringSet(Constant.RECENT_PLACES, Collections.emptySet());
        GsonBuilder gsonBuilder = new GsonBuilder();
        // change serialization for specific types
        JsonDeserializer<PlaceData> deserializer = (json, typeOfT, context) -> {
            JsonObject jsonObject = json.getAsJsonObject();
            Place place = null;
            try {
                if(jsonObject.get("place")!=null) {
                    place = Place.deserialize(jsonObject.get("place").getAsString());
                }
            } catch (PlaceSerializationException e) {
                e.printStackTrace();
            }
            return new PlaceData(
                    jsonObject.get("title").getAsString(),
                    place
            );
        };
        ; // will implement in a second
        gsonBuilder.registerTypeAdapter(PlaceData.class, deserializer);
        Gson customGson = gsonBuilder.create();

        for (String jsonText : placeTextSet) {
            recentPlaces.add(customGson.fromJson(jsonText, PlaceData.class));
        }

        return recentPlaces;
    }

    public void saveRecentPlace(PlaceData recentPlace) {
        Set<String> placeTextSet = new HashSet<>(preferences.getStringSet(Constant.RECENT_PLACES, Collections.emptySet()));
        //remove old places
        if (placeTextSet.size() >= 5) {
            placeTextSet.remove(placeTextSet.toArray()[0]);
        }
        GsonBuilder gsonBuilder = new GsonBuilder();
        // change serialization for specific types
        JsonSerializer<PlaceData> deserializer = (src, typeOfSrc, context) -> {
            JsonObject jsonMerchant = new JsonObject();

            jsonMerchant.addProperty("title", src.getTitle());
            jsonMerchant.addProperty("place", src.getPlace() != null ? src.getPlace().serializeCompact() : null);
            return jsonMerchant;
        };
        gsonBuilder.registerTypeAdapter(PlaceData.class, deserializer);
        Gson customGson = gsonBuilder.create();

        placeTextSet.add(customGson.toJson(recentPlace));
        preferences.edit().putStringSet(Constant.RECENT_PLACES, placeTextSet).apply();
    }

    public void clearRecentPlaces() {
        SharedPreferences.Editor prefEditor = preferences.edit();
        prefEditor.putString(Constant.RECENT_PLACES, "");
        prefEditor.apply();
    }


    public void setOfflineMode(boolean offlineMode) {
        SharedPreferences.Editor prefEditor = preferences.edit();
        prefEditor.putBoolean(Constant.OFFLINE_MODE, offlineMode);
        prefEditor.apply();
    }

    public boolean getOfflineMode() {
        return preferences.getBoolean(Constant.OFFLINE_MODE, false);
    }

    public void saveWaypoints(List<String> waypoints) {
        SharedPreferences.Editor prefEditor = preferences.edit();
        prefEditor.putStringSet(Constant.WAYPOINT_LIST, new HashSet<>(waypoints));
        prefEditor.apply();
    }

    public List<String> getWaypoints() {
        Set<String> stringSet =  preferences.getStringSet(Constant.WAYPOINT_LIST, new HashSet<>());
        return new ArrayList<>(stringSet);
    }

    public void saveRouteData(String route) {
        SharedPreferences.Editor prefEditor = preferences.edit();
        prefEditor.putString(Constant.ROUTE_DATA, route);
        prefEditor.apply();
    }

    public String getRouteData() {
        return preferences.getString(Constant.ROUTE_DATA, "");
    }

    public void saveCarPreferenceData(CarOptions carOptions) {
        SharedPreferences.Editor prefEditor = preferences.edit();
        prefEditor.putString(Constant.CAR_OPTIONS, new Gson().toJson(carOptions));
        prefEditor.apply();
    }

    public CarOptions getCarPreferenceData() {
        String carData = preferences.getString(Constant.CAR_OPTIONS, "");
        Log.e("getTruckPreferenceData:", carData);
        CarOptions carOptions;
        if (!carData.isEmpty()) {
            carOptions = new Gson().fromJson(carData, CarOptions.class);
        } else {
            carOptions = new CarOptions();
        }
        return carOptions;
    }

    public void saveTruckPreferenceData(TruckOptions truckOptions) {
        SharedPreferences.Editor prefEditor = preferences.edit();
        prefEditor.putString(Constant.TRUCK_OPTIONS, new Gson().toJson(truckOptions));
        prefEditor.apply();
    }

    public TruckOptions getTruckPreferenceData() {
        String truckData = preferences.getString(Constant.TRUCK_OPTIONS, "");
        Log.e("getTruckPreferenceData:", truckData);
        TruckOptions truckOptions;
        if (!truckData.isEmpty()) {
            truckOptions = new Gson().fromJson(truckData, TruckOptions.class);
        } else {
            truckOptions = new TruckOptions();
        }
        return truckOptions;
    }

    public void saveScooterPreferenceData(ScooterOptions scooterOptions) {
        SharedPreferences.Editor prefEditor = preferences.edit();
        prefEditor.putString(Constant.SCOOTER_OPTIONS, new Gson().toJson(scooterOptions));
        prefEditor.apply();
    }

    public ScooterOptions getScooterPreferenceData() {
        String scooterData = preferences.getString(Constant.SCOOTER_OPTIONS, "");
        Log.e("scooterPreferenceData:", scooterData);
        ScooterOptions scooterOptions;
        if (!scooterData.isEmpty()) {
            scooterOptions = new Gson().fromJson(scooterData, ScooterOptions.class);
        } else {
            scooterOptions = new ScooterOptions();
        }
        return scooterOptions;
    }

    public void saveWalkPreferenceData(PedestrianOptions pedestrianOptions) {
        SharedPreferences.Editor prefEditor = preferences.edit();
        prefEditor.putString(Constant.PEDESTRIAN_OPTIONS, new Gson().toJson(pedestrianOptions));
        prefEditor.apply();
    }

    public PedestrianOptions getWalkPreferenceData() {
        String pedestrianData = preferences.getString(Constant.PEDESTRIAN_OPTIONS, "");
        Log.e("getWalkPreferenceData: ", pedestrianData);
        PedestrianOptions pedestrianOptions;
        if (!pedestrianData.isEmpty()) {
            pedestrianOptions = new Gson().fromJson(pedestrianData, PedestrianOptions.class);
        } else {
            pedestrianOptions = new PedestrianOptions();
        }
        return pedestrianOptions;
    }

}
