package com.example.heresdkandroidref.utils;

import android.app.Activity;
import android.content.Context;
import android.view.View;
import android.view.inputmethod.InputMethodManager;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;

import com.example.heresdkandroidref.R;
import com.example.heresdkandroidref.models.TrafficLineColor;
import com.here.sdk.core.Color;
import com.here.sdk.core.CountryCode;
import com.here.sdk.core.GeoCoordinates;
import com.here.sdk.core.LanguageCode;
import com.here.sdk.core.TextFormat;
import com.here.sdk.core.UnitSystem;
import com.here.sdk.navigation.LaneDirectionCategory;
import com.here.sdk.routing.Maneuver;
import com.here.sdk.routing.ManeuverAction;
import com.here.sdk.routing.OptimizationMode;
import com.here.sdk.routing.RoadFeatures;
import com.here.sdk.routing.RoadTexts;
import com.here.sdk.routing.RoadType;
import com.here.sdk.routing.Span;
import com.here.sdk.transport.HazardousMaterial;
import com.here.sdk.transport.TruckType;
import com.here.sdk.transport.TunnelCategory;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.regex.Pattern;

public class Utils {
    private final Pattern sPattern = Pattern.compile("^(([a-zA-Z0-9]+))");

    static public String makeDistanceString(int distance) {
        try {
            if (distance == 0) {
                return "";
            } else if (distance < 1000) {
                return distance + " m ";
            } else if (distance < 10000) {
                String dist = String.format("%.1f", (float) (distance / 1000));
                return dist + " km ";
            } else {
                return (distance / 1000) + " km ";
            }
        } catch (Exception ex) {
            ex.printStackTrace();
            return "";
        }
    }

    public static String makeDurationString(long durationInSeconds) {
        try {
            int minutes = Math.round((durationInSeconds / 60));
            int hours = Math.round((minutes / 60));
            minutes = minutes % 60;

            if (hours == 0) {
                return minutes + " min ";
            } else {
                String result = hours + " h ";
                if (minutes != 0) {
                    result += minutes + " min ";
                }
                return result;
            }
        } catch (Exception ex) {
            ex.printStackTrace();
            return "";
        }
    }

    public static String getHourFromDuration(long durationInSeconds) {
        String time = makeDurationString(durationInSeconds);
        if (time.contains(" h ")) {
            return time.split(" h ")[0];
        } else {
            return "0";
        }
    }

    public static String getMinutesFromDuration(long durationInSeconds) {
        String time = makeDurationString(durationInSeconds);
        if (time.contains(" h ")) {
            return time.split(" h ")[1].split(" min ")[0];
        } else {
            return time.split(" min ")[0];
        }
    }

    /// Returns a formatted string constructed from a [template] and a list of [replacements].
//    public  static String formatString(String template, List replacements) {
//        final regExp = RegExp(_placeholderPattern);
//        assert(
//                regExp.allMatches(template).length == replacements.length, "Template and Replacements length are incompatible");
//
//        for (replacements in replacements) {
//            template = template.replaceFirst(regExp, replacement.toString());
//        }
//
//        return template;
//    }

    public static void disableSoftKeyboard(Context context) {
        InputMethodManager imm = (InputMethodManager) context.getSystemService(
                Activity.INPUT_METHOD_SERVICE);
        imm.toggleSoftInput(InputMethodManager.HIDE_IMPLICIT_ONLY, 0);
    }

    public static void hideKeyboard(Activity activity) {
        InputMethodManager imm = (InputMethodManager) activity.getSystemService(Activity.INPUT_METHOD_SERVICE);
        //Find the currently focused view, so we can grab the correct window token from it.
        View view = activity.getCurrentFocus();
        //If no view currently has focus, create a new one, just so we can grab a window token from it
        if (view == null) {
            view = new View(activity);
        }
        imm.hideSoftInputFromWindow(view.getWindowToken(), 0);
    }

    public static int getManeuverImage(int value) {
        int resourceId = 0;
        if (ManeuverAction.DEPART.value == value) {
            resourceId = R.drawable.depart;
        } else if (ManeuverAction.ARRIVE.value == value) {
            resourceId = R.drawable.arrive;
        } else if (ManeuverAction.LEFT_U_TURN.value == value) {
            resourceId = R.drawable.left_u_turn;
        } else if (ManeuverAction.SHARP_LEFT_TURN.value == value) {
            resourceId = R.drawable.sharp_left_turn;
        } else if (ManeuverAction.LEFT_TURN.value == value) {
            resourceId = R.drawable.left_turn;
        } else if (ManeuverAction.SLIGHT_LEFT_TURN.value == value) {
            resourceId = R.drawable.slight_left_turn;
        } else if (ManeuverAction.CONTINUE_ON.value == value) {
            resourceId = R.drawable.continue_on;
        } else if (ManeuverAction.SLIGHT_RIGHT_TURN.value == value) {
            resourceId = R.drawable.slight_right_turn;
        } else if (ManeuverAction.RIGHT_TURN.value == value) {
            resourceId = R.drawable.right_turn;
        } else if (ManeuverAction.SHARP_RIGHT_TURN.value == value) {
            resourceId = R.drawable.sharp_right_turn;
        } else if (ManeuverAction.RIGHT_U_TURN.value == value) {
            resourceId = R.drawable.right_u_turn;
        } else if (ManeuverAction.LEFT_EXIT.value == value) {
            resourceId = R.drawable.left_exit;
        } else if (ManeuverAction.RIGHT_EXIT.value == value) {
            resourceId = R.drawable.right_exit;
        } else if (ManeuverAction.LEFT_RAMP.value == value) {
            resourceId = R.drawable.left_ramp;
        } else if (ManeuverAction.RIGHT_RAMP.value == value) {
            resourceId = R.drawable.right_ramp;
        } else if (ManeuverAction.LEFT_FORK.value == value) {
            resourceId = R.drawable.left_fork;
        } else if (ManeuverAction.MIDDLE_FORK.value == value) {
            resourceId = R.drawable.middle_fork;
        } else if (ManeuverAction.RIGHT_FORK.value == value) {
            resourceId = R.drawable.right_fork;
//        } else if (ManeuverAction.FERRY.value == value) {
//            resourceId = R.drawable.ferry;
        } else if (ManeuverAction.LEFT_ROUNDABOUT_ENTER.value == value) {
            resourceId = R.drawable.left_turn;
        } else if (ManeuverAction.RIGHT_ROUNDABOUT_ENTER.value == value) {
            resourceId = R.drawable.right_turn;
        } else if (ManeuverAction.LEFT_ROUNDABOUT_PASS.value == value) {
            resourceId = R.drawable.left_turn;
        } else if (ManeuverAction.RIGHT_ROUNDABOUT_PASS.value == value) {
            resourceId = R.drawable.right_turn;
        } else if (ManeuverAction.LEFT_ROUNDABOUT_EXIT1.value == value) {
            resourceId = R.drawable.left_roundabout_exit_1;
        } else if (ManeuverAction.LEFT_ROUNDABOUT_EXIT2.value == value) {
            resourceId = R.drawable.left_roundabout_exit_2;
        } else if (ManeuverAction.LEFT_ROUNDABOUT_EXIT3.value == value) {
            resourceId = R.drawable.left_roundabout_exit_3;
        } else if (ManeuverAction.LEFT_ROUNDABOUT_EXIT4.value == value) {
            resourceId = R.drawable.left_roundabout_exit_4;
        } else if (ManeuverAction.LEFT_ROUNDABOUT_EXIT5.value == value) {
            resourceId = R.drawable.left_roundabout_exit_5;
        } else if (ManeuverAction.LEFT_ROUNDABOUT_EXIT6.value == value) {
            resourceId = R.drawable.left_roundabout_exit_6;
        } else if (ManeuverAction.LEFT_ROUNDABOUT_EXIT7.value == value) {
            resourceId = R.drawable.left_roundabout_exit_7;
        } else if (ManeuverAction.LEFT_ROUNDABOUT_EXIT8.value == value) {
            resourceId = R.drawable.left_roundabout_exit_8;
        } else if (ManeuverAction.LEFT_ROUNDABOUT_EXIT9.value == value) {
            resourceId = R.drawable.left_roundabout_exit_9;
        } else if (ManeuverAction.LEFT_ROUNDABOUT_EXIT10.value == value) {
            resourceId = R.drawable.left_roundabout_exit_10;
        } else if (ManeuverAction.LEFT_ROUNDABOUT_EXIT11.value == value) {
            resourceId = R.drawable.left_roundabout_exit_11;
        } else if (ManeuverAction.LEFT_ROUNDABOUT_EXIT12.value == value) {
            resourceId = R.drawable.left_roundabout_exit_12;
        } else if (ManeuverAction.RIGHT_ROUNDABOUT_EXIT1.value == value) {
            resourceId = R.drawable.right_roundabout_exit_1;
        } else if (ManeuverAction.RIGHT_ROUNDABOUT_EXIT2.value == value) {
            resourceId = R.drawable.right_roundabout_exit_2;
        } else if (ManeuverAction.RIGHT_ROUNDABOUT_EXIT3.value == value) {
            resourceId = R.drawable.right_roundabout_exit_3;
        } else if (ManeuverAction.RIGHT_ROUNDABOUT_EXIT4.value == value) {
            resourceId = R.drawable.right_roundabout_exit_4;
        } else if (ManeuverAction.RIGHT_ROUNDABOUT_EXIT5.value == value) {
            resourceId = R.drawable.right_roundabout_exit_5;
        } else if (ManeuverAction.RIGHT_ROUNDABOUT_EXIT6.value == value) {
            resourceId = R.drawable.right_roundabout_exit_6;
        } else if (ManeuverAction.RIGHT_ROUNDABOUT_EXIT7.value == value) {
            resourceId = R.drawable.right_roundabout_exit_7;
        } else if (ManeuverAction.RIGHT_ROUNDABOUT_EXIT8.value == value) {
            resourceId = R.drawable.right_roundabout_exit_8;
        } else if (ManeuverAction.RIGHT_ROUNDABOUT_EXIT9.value == value) {
            resourceId = R.drawable.right_roundabout_exit_9;
        } else if (ManeuverAction.RIGHT_ROUNDABOUT_EXIT10.value == value) {
            resourceId = R.drawable.right_roundabout_exit_10;
        } else if (ManeuverAction.RIGHT_ROUNDABOUT_EXIT11.value == value) {
            resourceId = R.drawable.right_roundabout_exit_11;
        } else if (ManeuverAction.RIGHT_ROUNDABOUT_EXIT12.value == value) {
            resourceId = R.drawable.right_roundabout_exit_12;
        }
        return resourceId;
    }

    public static int getLanesImage(@NonNull LaneDirectionCategory directionCategory) {
        int resourceId = 0;
        if (directionCategory.straight) {
            resourceId = R.drawable.continue_on_white;
        } else if (directionCategory.slightlyLeft) {
            resourceId = R.drawable.slight_left_turn_white;
        } else if (directionCategory.slightlyRight) {
            resourceId = R.drawable.slight_right_turn_white;
        } else if (directionCategory.quiteLeft) {
            resourceId = R.drawable.left_turn_white;
        } else if (directionCategory.quiteRight) {
            resourceId = R.drawable.right_turn_white;
        } else if (directionCategory.hardLeft) {
            resourceId = R.drawable.sharp_left_turn_white;
        } else if (directionCategory.hardRight) {
            resourceId = R.drawable.sharp_right_turn_white;
        } else if (directionCategory.uTurnLeft) {
            resourceId = R.drawable.left_u_turn_white;
        } else if (directionCategory.uTurnRight) {
            resourceId = R.drawable.right_u_turn_white;
        }
        return resourceId;
    }

    public static int getManeuverImageWhite(Maneuver maneuver) {
        int value = maneuver.getAction().value;
        int resourceId = 0;
        if (ManeuverAction.DEPART.value == value) {
            resourceId = R.drawable.depart_white;
        } else if (ManeuverAction.ARRIVE.value == value) {
            resourceId = R.drawable.arrive_white;
        } else if (ManeuverAction.LEFT_U_TURN.value == value) {
            resourceId = R.drawable.left_u_turn_white;
        } else if (ManeuverAction.SHARP_LEFT_TURN.value == value) {
            resourceId = R.drawable.sharp_left_turn_white;
        } else if (ManeuverAction.LEFT_TURN.value == value) {
            resourceId = R.drawable.left_turn_white;
        } else if (ManeuverAction.SLIGHT_LEFT_TURN.value == value) {
            resourceId = R.drawable.slight_left_turn_white;
        } else if (ManeuverAction.CONTINUE_ON.value == value) {
            resourceId = R.drawable.continue_on_white;
        } else if (ManeuverAction.SLIGHT_RIGHT_TURN.value == value) {
            resourceId = R.drawable.slight_right_turn_white;
        } else if (ManeuverAction.RIGHT_TURN.value == value) {
            resourceId = R.drawable.right_turn_white;
        } else if (ManeuverAction.SHARP_RIGHT_TURN.value == value) {
            resourceId = R.drawable.sharp_right_turn_white;
        } else if (ManeuverAction.RIGHT_U_TURN.value == value) {
            resourceId = R.drawable.right_u_turn_white;
        } else if (ManeuverAction.LEFT_EXIT.value == value) {
            resourceId = R.drawable.left_exit_white;
        } else if (ManeuverAction.RIGHT_EXIT.value == value) {
            resourceId = R.drawable.right_exit_white;
        } else if (ManeuverAction.LEFT_RAMP.value == value) {
            resourceId = R.drawable.left_ramp_white;
        } else if (ManeuverAction.RIGHT_RAMP.value == value) {
            resourceId = R.drawable.right_ramp_white;
        } else if (ManeuverAction.LEFT_FORK.value == value) {
            resourceId = R.drawable.left_fork_white;
        } else if (ManeuverAction.MIDDLE_FORK.value == value) {
            resourceId = R.drawable.middle_fork_white;
        } else if (ManeuverAction.RIGHT_FORK.value == value) {
            resourceId = R.drawable.right_fork_white;
//        } else if (ManeuverAction.FERRY.value == value) {
//            resourceId = R.drawable.ferry_white;
        } else if (ManeuverAction.LEFT_ROUNDABOUT_ENTER.value == value) {
            resourceId = R.drawable.left_turn_white;
        } else if (ManeuverAction.RIGHT_ROUNDABOUT_ENTER.value == value) {
            resourceId = R.drawable.right_turn_white;
        } else if (ManeuverAction.LEFT_ROUNDABOUT_PASS.value == value) {
            resourceId = R.drawable.left_turn_white;
        } else if (ManeuverAction.RIGHT_ROUNDABOUT_PASS.value == value) {
            resourceId = R.drawable.right_turn_white;
        } else if (ManeuverAction.LEFT_ROUNDABOUT_EXIT1.value == value) {
            resourceId = R.drawable.left_roundaboutexit1_white;
        } else if (ManeuverAction.LEFT_ROUNDABOUT_EXIT2.value == value) {
            resourceId = R.drawable.left_roundaboutexit2_white;
        } else if (ManeuverAction.LEFT_ROUNDABOUT_EXIT3.value == value) {
            resourceId = R.drawable.left_roundaboutexit3_white;
        } else if (ManeuverAction.LEFT_ROUNDABOUT_EXIT4.value == value) {
            resourceId = R.drawable.left_roundaboutexit4_white;
        } else if (ManeuverAction.LEFT_ROUNDABOUT_EXIT5.value == value) {
            resourceId = R.drawable.left_roundaboutexit5_white;
        } else if (ManeuverAction.LEFT_ROUNDABOUT_EXIT6.value == value) {
            resourceId = R.drawable.left_roundaboutexit6_white;
        } else if (ManeuverAction.LEFT_ROUNDABOUT_EXIT7.value == value) {
            resourceId = R.drawable.left_roundaboutexit7_white;
        } else if (ManeuverAction.LEFT_ROUNDABOUT_EXIT8.value == value) {
            resourceId = R.drawable.left_roundaboutexit8_white;
        } else if (ManeuverAction.LEFT_ROUNDABOUT_EXIT9.value == value) {
            resourceId = R.drawable.left_roundaboutexit9_white;
        } else if (ManeuverAction.LEFT_ROUNDABOUT_EXIT10.value == value) {
            resourceId = R.drawable.left_roundaboutexit10_white;
        } else if (ManeuverAction.LEFT_ROUNDABOUT_EXIT11.value == value) {
            resourceId = R.drawable.left_roundaboutexit11_white;
        } else if (ManeuverAction.LEFT_ROUNDABOUT_EXIT12.value == value) {
            resourceId = R.drawable.left_roundaboutexit12_white;
        } else if (ManeuverAction.RIGHT_ROUNDABOUT_EXIT1.value == value) {
            resourceId = R.drawable.right_roundaboutexit1_white;
        } else if (ManeuverAction.RIGHT_ROUNDABOUT_EXIT2.value == value) {
            resourceId = R.drawable.right_roundaboutexit2_white;
        } else if (ManeuverAction.RIGHT_ROUNDABOUT_EXIT3.value == value) {
            resourceId = R.drawable.right_roundaboutexit3_white;
        } else if (ManeuverAction.RIGHT_ROUNDABOUT_EXIT4.value == value) {
            resourceId = R.drawable.right_roundaboutexit4_white;
        } else if (ManeuverAction.RIGHT_ROUNDABOUT_EXIT5.value == value) {
            resourceId = R.drawable.right_roundaboutexit5_white;
        } else if (ManeuverAction.RIGHT_ROUNDABOUT_EXIT6.value == value) {
            resourceId = R.drawable.right_roundaboutexit6_white;
        } else if (ManeuverAction.RIGHT_ROUNDABOUT_EXIT7.value == value) {
            resourceId = R.drawable.right_roundaboutexit7_white;
        } else if (ManeuverAction.RIGHT_ROUNDABOUT_EXIT8.value == value) {
            resourceId = R.drawable.right_roundaboutexit8_white;
        } else if (ManeuverAction.RIGHT_ROUNDABOUT_EXIT9.value == value) {
            resourceId = R.drawable.right_roundaboutexit9_white;
        } else if (ManeuverAction.RIGHT_ROUNDABOUT_EXIT10.value == value) {
            resourceId = R.drawable.right_roundaboutexit10_white;
        } else if (ManeuverAction.RIGHT_ROUNDABOUT_EXIT11.value == value) {
            resourceId = R.drawable.right_roundaboutexit11_white;
        } else if (ManeuverAction.RIGHT_ROUNDABOUT_EXIT12.value == value) {
            resourceId = R.drawable.right_roundaboutexit12_white;
        }
        return resourceId;
    }

    static public String getRoadName(Maneuver maneuver) {
        try {


            RoadTexts currentRoadTexts = maneuver.getRoadTexts();
            RoadTexts nextRoadTexts = maneuver.getNextRoadTexts();

            String currentRoadName = currentRoadTexts.names.getDefaultValue();
            String currentRoadNumber = currentRoadTexts.numbersWithDirection.getDefaultValue();
            String nextRoadName = nextRoadTexts.names.getDefaultValue();
            String nextRoadNumber = nextRoadTexts.numbersWithDirection.getDefaultValue();

            String roadName = nextRoadName == null ? nextRoadNumber : nextRoadName;

            // On highways, we want to show the highway number instead of a possible road name,
            // while for inner city and urban areas road names are preferred over road numbers.
            if (maneuver.getNextRoadType() == RoadType.HIGHWAY) {
                roadName = nextRoadNumber == null ? nextRoadName : nextRoadNumber;
            }

            if (maneuver.getAction() == ManeuverAction.ARRIVE) {
                // We are approaching the destination, so there's no next road.
                roadName = currentRoadName == null ? currentRoadNumber : currentRoadName;
            }

            if (roadName == null) {
                // Happens only in rare cases, when also the fallback is null.
                roadName = "unnamed road";
            }

            return roadName;
        } catch (Exception ex) {
            ex.printStackTrace();
            return "";
        }
    }


    static public String getActionText(Maneuver maneuver) {
        try {
            int value = maneuver.getAction().value;
            RoadTexts currentRoadTexts = maneuver.getRoadTexts();
            RoadTexts nextRoadTexts = maneuver.getNextRoadTexts();
            final String roadName = currentRoadTexts.names.getDefaultValue();
            final String nextRoadName = nextRoadTexts.names.getDefaultValue();


            String actionText = "";
            if (ManeuverAction.DEPART.value == value) {
                actionText = _makeActionString("Start", "Start at ", roadName);
            } else if (ManeuverAction.ARRIVE.value == value) {
                actionText = "Arrive at destination";
            } else if (ManeuverAction.LEFT_U_TURN.value == value) {
                actionText = _makeActionString("Make a left U-turn", "Make a left U-turn at ", nextRoadName);
            } else if (ManeuverAction.SHARP_LEFT_TURN.value == value) {
                actionText = _makeActionString("Make a hard left turn", "Make a hard left turn onto ", nextRoadName);

            } else if (ManeuverAction.LEFT_TURN.value == value) {
                actionText = _makeActionString("Turn left", "Turn left on ", nextRoadName);
            } else if (ManeuverAction.SLIGHT_LEFT_TURN.value == value) {
                actionText = _makeActionString("Bear left", "Bear left on ", nextRoadName);
            } else if (ManeuverAction.CONTINUE_ON.value == value) {
                actionText = _makeActionString("Continue on", "Continue on ", roadName);
            } else if (ManeuverAction.SLIGHT_RIGHT_TURN.value == value) {
                actionText = _makeActionString("Bear right", "Bear right on ", nextRoadName);
            } else if (ManeuverAction.RIGHT_TURN.value == value) {
                actionText = _makeActionString("Turn right", "Turn right on ", nextRoadName);
            } else if (ManeuverAction.SHARP_RIGHT_TURN.value == value) {
                actionText = _makeActionString("Make a hard right turn", "Make a hard right turn onto ", nextRoadName);
            } else if (ManeuverAction.RIGHT_U_TURN.value == value) {
                actionText = _makeActionString("Make a right U-turn", "Make a right U-turn at ", nextRoadName);
            } else if (ManeuverAction.LEFT_EXIT.value == value) {
                actionText = _makeActionString("Take the left exit", "Take the left exit to ", nextRoadName);
            } else if (ManeuverAction.RIGHT_EXIT.value == value) {
                actionText = _makeActionString("Take the right exit", "Take the right exit to ", nextRoadName);
            } else if (ManeuverAction.LEFT_RAMP.value == value) {
                actionText = _makeActionString("Take the left ramp", "Take the left ramp onto ", nextRoadName);
            } else if (ManeuverAction.RIGHT_RAMP.value == value) {
                actionText = _makeActionString("Take the right ramp", "Take the right ramp onto ", nextRoadName);
            } else if (ManeuverAction.LEFT_FORK.value == value) {

                actionText = _makeActionString("Take the left fork", "Take the left fork onto ", nextRoadName);
            } else if (ManeuverAction.MIDDLE_FORK.value == value) {
                actionText = _makeActionString("Take the middle fork", "Take the middle fork onto ", nextRoadName);
            } else if (ManeuverAction.RIGHT_FORK.value == value) {
                actionText = _makeActionString("Take the right fork", "Take the right fork onto ", nextRoadName);
//            } else if (ManeuverAction.FERRY.value == value) {
//                actionText = _makeActionString("Take the ferry", "Take the ferry onto ", nextRoadName);
            } else if (ManeuverAction.LEFT_ROUNDABOUT_ENTER.value == value) {
                actionText = "Enter the roundabout";
            } else if (ManeuverAction.RIGHT_ROUNDABOUT_ENTER.value == value) {
                actionText = "Enter the roundabout";
            } else if (ManeuverAction.LEFT_ROUNDABOUT_PASS.value == value) {
                actionText = "Pass the roundabout";
            } else if (ManeuverAction.RIGHT_ROUNDABOUT_PASS.value == value) {
                actionText = "Pass the roundabout";
            } else if (ManeuverAction.LEFT_ROUNDABOUT_EXIT1.value == value) {
                actionText = "Take the first exit of the roundabout";
            } else if (ManeuverAction.LEFT_ROUNDABOUT_EXIT2.value == value) {
                actionText = "Take the second exit of the roundabout";
            } else if (ManeuverAction.LEFT_ROUNDABOUT_EXIT3.value == value) {
                actionText = "Take the third exit of the roundabout";
            } else if (ManeuverAction.LEFT_ROUNDABOUT_EXIT4.value == value) {
                actionText = "Take the fourth exit of the roundabout";
            } else if (ManeuverAction.LEFT_ROUNDABOUT_EXIT5.value == value) {
                actionText = "Take the fifth exit of the roundabout";
            } else if (ManeuverAction.LEFT_ROUNDABOUT_EXIT6.value == value) {
                actionText = "Take the sixth exit of the roundabout";
            } else if (ManeuverAction.LEFT_ROUNDABOUT_EXIT7.value == value) {
                actionText = "Take the 7th exit of the roundabout";
            } else if (ManeuverAction.LEFT_ROUNDABOUT_EXIT8.value == value) {
                actionText = "Take the 8th exit of the roundabout";
            } else if (ManeuverAction.LEFT_ROUNDABOUT_EXIT9.value == value) {
                actionText = "Take the 9th exit of the roundabout";
            } else if (ManeuverAction.LEFT_ROUNDABOUT_EXIT10.value == value) {
                actionText = "Take the 10th exit of the roundabout";
            } else if (ManeuverAction.LEFT_ROUNDABOUT_EXIT11.value == value) {
                actionText = "Take the 11th exit of the roundabout";
            } else if (ManeuverAction.LEFT_ROUNDABOUT_EXIT12.value == value) {
                actionText = "Take the 12th exit of the roundabout";
            } else if (ManeuverAction.RIGHT_ROUNDABOUT_EXIT1.value == value) {
                actionText = "Take the first exit of the roundabout";
            } else if (ManeuverAction.RIGHT_ROUNDABOUT_EXIT2.value == value) {
                actionText = "Take the second exit of the roundabout";
            } else if (ManeuverAction.RIGHT_ROUNDABOUT_EXIT3.value == value) {
                actionText = "Take the third exit of the roundabout";
            } else if (ManeuverAction.RIGHT_ROUNDABOUT_EXIT4.value == value) {
                actionText = "Take the fourth exit of the roundabout";
            } else if (ManeuverAction.RIGHT_ROUNDABOUT_EXIT5.value == value) {
                actionText = "Take the fifth exit of the roundabout";
            } else if (ManeuverAction.RIGHT_ROUNDABOUT_EXIT6.value == value) {
                actionText = "Take the sixth exit of the roundabout";
            } else if (ManeuverAction.RIGHT_ROUNDABOUT_EXIT7.value == value) {
                actionText = "Take the 7th exit of the roundabout";
            } else if (ManeuverAction.RIGHT_ROUNDABOUT_EXIT8.value == value) {
                actionText = "Take the 8th exit of the roundabout";
            } else if (ManeuverAction.RIGHT_ROUNDABOUT_EXIT9.value == value) {
                actionText = "Take the 9th exit of the roundabout";
            } else if (ManeuverAction.RIGHT_ROUNDABOUT_EXIT10.value == value) {
                actionText = "Take the 10th exit of the roundabout";
            } else if (ManeuverAction.RIGHT_ROUNDABOUT_EXIT11.value == value) {
                actionText = "Take the 11th exit of the roundabout";
            } else if (ManeuverAction.RIGHT_ROUNDABOUT_EXIT12.value == value) {
                actionText = "Take the 12th exit of the roundabout";
            }
            return actionText;
        } catch (Exception ex) {
            ex.printStackTrace();
            return "";
        }
    }

    static String _makeActionString(String text, String template, String roadName) {
        if (roadName == null || roadName.isEmpty()) {
            return text;
        }
        return template + roadName;
//        return formatString(template, roadName);
    }

    public static String getFormattedDateTime(Date date) {
        String myFormat = "dd/MM/yyyy, hh:mm aa";
        SimpleDateFormat dateFormat = new SimpleDateFormat(myFormat, Locale.US);
        return dateFormat.format(date);
    }

    public static Map<String, OptimizationMode> getOptimizationModeMap() {
        Map<String, OptimizationMode> optimizationModeMap = new HashMap<>();
        for (OptimizationMode optimizationMode : OptimizationMode.values()) {
            switch (optimizationMode) {
                case FASTEST:
                    optimizationModeMap.put("Fastest route", OptimizationMode.FASTEST);
                    break;
                case SHORTEST:
                    optimizationModeMap.put("Shortest route", OptimizationMode.SHORTEST);
                    break;
                default:
                    break;
            }
        }
        return optimizationModeMap;
    }

    public static Map<String, TextFormat> getTextFormatMap() {
        Map<String, TextFormat> textFormatHashMap = new HashMap<>();
        for (TextFormat textFormat : TextFormat.values()) {
            switch (textFormat) {
                case HTML:
                    textFormatHashMap.put("Html", TextFormat.HTML);
                    break;
                case PLAIN:
                    textFormatHashMap.put("Plain", TextFormat.PLAIN);
                    break;
                default:
                    break;
            }
        }
        return textFormatHashMap;
    }

    public static Map<String, UnitSystem> getUnitSystemMap() {
        Map<String, UnitSystem> unitSystemMap = new HashMap<>();
        for (UnitSystem unitSystem : UnitSystem.values()) {
            switch (unitSystem) {
                case METRIC:
                    unitSystemMap.put("Metric", UnitSystem.METRIC);
                    break;
                case IMPERIAL_UK:
                    unitSystemMap.put("Imperial UK", UnitSystem.IMPERIAL_UK);
                    break;
                case IMPERIAL_US:
                    unitSystemMap.put("Imperial US", UnitSystem.IMPERIAL_US);
                    break;
                default:
                    break;
            }
        }
        return unitSystemMap;
    }

    public static Map<String, LanguageCode> getLanguageCodeMap() {
        Map<String, LanguageCode> languageCodeMap = new HashMap<>();
        for (LanguageCode languageCode : LanguageCode.values()) {
            switch (languageCode) {
                case EN_US:
                    languageCodeMap.put("English (United States)", LanguageCode.EN_US);
                    break;
                case AR_SA:
                    languageCodeMap.put("Arabic (Saudi Arabia)", LanguageCode.AR_SA);
                    break;
                case ZH_CN:
                    languageCodeMap.put("Chinese (Simplified China)", LanguageCode.ZH_CN);
                    break;
                case FR_FR:
                    languageCodeMap.put("French", LanguageCode.FR_FR);
                    break;
                case DE_DE:
                    languageCodeMap.put("German", LanguageCode.DE_DE);
                    break;
                case HI_IN:
                    languageCodeMap.put("Hindi", LanguageCode.HI_IN);
                    break;
                case PT_PT:
                    languageCodeMap.put("Portuguese (Portugal)", LanguageCode.PT_PT);
                    break;
                case ES_ES:
                    languageCodeMap.put("Spanish (Spain)", LanguageCode.ES_ES);
                    break;
                default:
                    break;
            }
        }
        return languageCodeMap;
    }

    public static Map<String, TunnelCategory> getTunnelCategoryMap() {
        Map<String, TunnelCategory> tunnelCategoryMap = new HashMap<>();
        for (TunnelCategory tunnelCategory : TunnelCategory.values()) {
            switch (tunnelCategory) {
                case B:
                    tunnelCategoryMap.put("B category", TunnelCategory.B);
                    break;
                case C:
                    tunnelCategoryMap.put("C category", TunnelCategory.C);
                    break;
                case D:
                    tunnelCategoryMap.put("D category", TunnelCategory.D);
                    break;
                case E:
                    tunnelCategoryMap.put("E category", TunnelCategory.E);
                    break;
                default:
                    break;
            }
        }
        return tunnelCategoryMap;
    }

    public static Map<String, RoadFeatures> getRoadFeaturesMap() {
        Map<String, RoadFeatures> roadFeaturesMap = new HashMap<>();
        for (RoadFeatures roadFeatures : RoadFeatures.values()) {
            switch (roadFeatures) {
                case CAR_SHUTTLE_TRAIN:
                    roadFeaturesMap.put("Car shuttle train", RoadFeatures.CAR_SHUTTLE_TRAIN);
                    break;
                case CONTROLLED_ACCESS_HIGHWAY:
                    roadFeaturesMap.put("Controlled access highway", RoadFeatures.CONTROLLED_ACCESS_HIGHWAY);
                    break;
                case U_TURNS:
                    roadFeaturesMap.put("U turns", RoadFeatures.U_TURNS);
                    break;
                case DIRT_ROAD:
                    roadFeaturesMap.put("Dirt road", RoadFeatures.DIRT_ROAD);
                    break;
                case FERRY:
                    roadFeaturesMap.put("Ferry", RoadFeatures.FERRY);
                    break;
                case SEASONAL_CLOSURE:
                    roadFeaturesMap.put("Seasonal closure", RoadFeatures.SEASONAL_CLOSURE);
                    break;
                case TOLL_ROAD:
                    roadFeaturesMap.put("Toll road", RoadFeatures.TOLL_ROAD);
                    break;
                case TUNNEL:
                    roadFeaturesMap.put("Tunnel", RoadFeatures.TUNNEL);
                    break;
                default:
                    break;
            }
        }
        return roadFeaturesMap;
    }

    public static Map<String, CountryCode> getCountryCodeMap() {
        Map<String, CountryCode> countryCodeMap = new HashMap<>();
        for (CountryCode countryCode : CountryCode.values()) {
            switch (countryCode) {
                case ABW:
                    countryCodeMap.put("Aruba", CountryCode.ABW);
                    break;
                case AFG:
                    countryCodeMap.put("Afghanistan", CountryCode.AFG);
                    break;
                case AGO:
                    countryCodeMap.put("Angola", CountryCode.AGO);
                    break;
                case AIA:
                    countryCodeMap.put("Anguilla", CountryCode.AIA);
                    break;
                case ALA:
                    countryCodeMap.put("Aland Islands", CountryCode.ALA);
                    break;
                case ALB:
                    countryCodeMap.put("Albania", CountryCode.ALB);
                    break;
                case AND:
                    countryCodeMap.put("Andorra", CountryCode.AND);
                    break;
                case ARE:
                    countryCodeMap.put("United Arab Emirates", CountryCode.ARE);
                    break;
                case ARG:
                    countryCodeMap.put("Argentina", CountryCode.ARG);
                    break;
                case ARM:
                    countryCodeMap.put("Armenia", CountryCode.ARM);
                    break;
                case ASM:
                    countryCodeMap.put("American Samoa", CountryCode.ASM);
                    break;
                case ATA:
                    countryCodeMap.put("Antarctica", CountryCode.ATA);
                    break;
                case ATF:
                    countryCodeMap.put("French Southern Territories", CountryCode.ATF);
                    break;
                case ATG:
                    countryCodeMap.put("Antigua And Barbuda", CountryCode.ATG);
                    break;
                case AUS:
                    countryCodeMap.put("Australia", CountryCode.AUS);
                    break;
                case AUT:
                    countryCodeMap.put("Austria", CountryCode.AUT);
                    break;
                case AZE:
                    countryCodeMap.put("Azerbaijan", CountryCode.AZE);
                    break;
                case BDI:
                    countryCodeMap.put("Burundi", CountryCode.BDI);
                    break;
                case BEL:
                    countryCodeMap.put("Belgium", CountryCode.BEL);
                    break;
                case BEN:
                    countryCodeMap.put("Benin", CountryCode.BEN);
                    break;
                case BES:
                    countryCodeMap.put("Bonaire, Sint Eustatius And Saba", CountryCode.BES);
                    break;
                case BFA:
                    countryCodeMap.put("Burkina Faso", CountryCode.BFA);
                    break;
                case BGD:
                    countryCodeMap.put("Bangladesh", CountryCode.BGD);
                    break;
                case BGR:
                    countryCodeMap.put("Bulgaria", CountryCode.BGR);
                    break;
                case BHR:
                    countryCodeMap.put("Bahrain", CountryCode.BHR);
                    break;
                case BIH:
                    countryCodeMap.put("Bosnia And Herzegovina", CountryCode.BIH);
                    break;
                case BLM:
                    countryCodeMap.put("Saint Barthelemy", CountryCode.BLM);
                    break;
                case BLZ:
                    countryCodeMap.put("Belize", CountryCode.BLZ);
                    break;
                case BMU:
                    countryCodeMap.put("Bermuda", CountryCode.BMU);
                    break;
                case BOL:
                    countryCodeMap.put("Bolivia (plurinational State Of)", CountryCode.BOL);
                    break;
                case BRA:
                    countryCodeMap.put("Brazil", CountryCode.BRA);
                    break;
                case BRB:
                    countryCodeMap.put("Barbados", CountryCode.BRB);
                    break;
                case BRN:
                    countryCodeMap.put("Brunei Darussalam", CountryCode.BRN);
                    break;
                case BTN:
                    countryCodeMap.put("Bhutan", CountryCode.BTN);
                    break;
                case BVT:
                    countryCodeMap.put("Bouvet Island", CountryCode.BVT);
                    break;
                case BWA:
                    countryCodeMap.put("Botswana", CountryCode.BWA);
                    break;
                case CAF:
                    countryCodeMap.put("Central African Republic", CountryCode.CAF);
                    break;
                case CAN:
                    countryCodeMap.put("Canada", CountryCode.CAN);
                    break;
                case CCK:
                    countryCodeMap.put("Cocos (keeling) Islands", CountryCode.CCK);
                    break;
                case CHE:
                    countryCodeMap.put("Switzerland", CountryCode.CHE);
                    break;
                case CHL:
                    countryCodeMap.put("Chile", CountryCode.CHL);
                    break;
                case CHN:
                    countryCodeMap.put("China", CountryCode.CHN);
                    break;
                case CIV:
                    countryCodeMap.put("Cote D'ivoire", CountryCode.CIV);
                    break;
                case CMR:
                    countryCodeMap.put("Cameroon", CountryCode.CMR);
                    break;
                case COD:
                    countryCodeMap.put("Congo, Democratic Republic Of The", CountryCode.COD);
                    break;
                case COG:
                    countryCodeMap.put("Congo", CountryCode.COG);
                    break;
                case COK:
                    countryCodeMap.put("Cook Islands", CountryCode.COK);
                    break;
                case COL:
                    countryCodeMap.put("Colombia", CountryCode.COL);
                    break;
                case COM:
                    countryCodeMap.put("Comoros", CountryCode.COM);
                    break;
                case CPV:
                    countryCodeMap.put("Cabo Verde", CountryCode.CPV);
                    break;
                case CRI:
                    countryCodeMap.put("Costa Rica", CountryCode.CRI);
                    break;
                case CUB:
                    countryCodeMap.put("Cuba", CountryCode.CUB);
                    break;
                case CUW:
                    countryCodeMap.put("Curacao", CountryCode.CUW);
                    break;
                case CXR:
                    countryCodeMap.put("Christmas Island", CountryCode.CXR);
                    break;
                case CYM:
                    countryCodeMap.put("Cayman Islands", CountryCode.CYM);
                    break;
                case CYP:
                    countryCodeMap.put("Cyprus", CountryCode.CYP);
                    break;
                case CZE:
                    countryCodeMap.put("Czechia", CountryCode.CZE);
                    break;
                case DEU:
                    countryCodeMap.put("Germany", CountryCode.DEU);
                    break;
                case DJI:
                    countryCodeMap.put("Djibouti", CountryCode.DJI);
                    break;
                case DMA:
                    countryCodeMap.put("Dominica", CountryCode.DMA);
                    break;
                case DNK:
                    countryCodeMap.put("Denmark", CountryCode.DNK);
                    break;
                case DOM:
                    countryCodeMap.put("Dominican Republic", CountryCode.DOM);
                    break;
                case DZA:
                    countryCodeMap.put("Algeria", CountryCode.DZA);
                    break;
                case ECU:
                    countryCodeMap.put("Ecuador", CountryCode.ECU);
                    break;
                case EGY:
                    countryCodeMap.put("Egypt", CountryCode.EGY);
                    break;
                case ERI:
                    countryCodeMap.put("Eritrea", CountryCode.ERI);
                    break;
                case ESH:
                    countryCodeMap.put("Western Sahara", CountryCode.ESH);
                    break;
                case ESP:
                    countryCodeMap.put("Spain", CountryCode.ESP);
                    break;
                case EST:
                    countryCodeMap.put("Estonia", CountryCode.EST);
                    break;
                case ETH:
                    countryCodeMap.put("Ethiopia", CountryCode.ETH);
                    break;
                case FIN:
                    countryCodeMap.put("Finland", CountryCode.FIN);
                    break;
                case FJI:
                    countryCodeMap.put("Fiji", CountryCode.FJI);
                    break;
                case FLK:
                    countryCodeMap.put("Falkland Islands (malvinas)", CountryCode.FLK);
                    break;
                case FRA:
                    countryCodeMap.put("France", CountryCode.FRA);
                    break;
                case FRO:
                    countryCodeMap.put("Faroe Islands", CountryCode.FRO);
                    break;
                case FSM:
                    countryCodeMap.put("Micronesia (federated States Of)", CountryCode.FSM);
                    break;
                case GAB:
                    countryCodeMap.put("Gabon", CountryCode.GAB);
                    break;
                case GBR:
                    countryCodeMap.put("United Kingdom Of Great Britain And Northern Ireland", CountryCode.GBR);
                    break;
                case GEO:
                    countryCodeMap.put("Georgia", CountryCode.GEO);
                    break;
                case GGY:
                    countryCodeMap.put("Guernsey", CountryCode.GGY);
                    break;
                case GHA:
                    countryCodeMap.put("Ghana", CountryCode.GHA);
                    break;
                case GIB:
                    countryCodeMap.put("Gibraltar", CountryCode.GIB);
                    break;
                case GIN:
                    countryCodeMap.put("Guinea", CountryCode.GIN);
                    break;
                case GLP:
                    countryCodeMap.put("Guadeloupe", CountryCode.GLP);
                    break;
                case GMB:
                    countryCodeMap.put("Gambia", CountryCode.GMB);
                    break;
                case GNB:
                    countryCodeMap.put("Guinea-bissau", CountryCode.GNB);
                    break;
                case GNQ:
                    countryCodeMap.put("Equatorial Guinea", CountryCode.GNQ);
                    break;
                case GRC:
                    countryCodeMap.put("Greece", CountryCode.GRC);
                    break;
                case GRD:
                    countryCodeMap.put("Grenada", CountryCode.GRD);
                    break;
                case GRL:
                    countryCodeMap.put("Greenland", CountryCode.GRL);
                    break;
                case GTM:
                    countryCodeMap.put("Guatemala", CountryCode.GTM);
                    break;
                case GUF:
                    countryCodeMap.put("French Guiana", CountryCode.GUF);
                    break;
                case GUM:
                    countryCodeMap.put("Guam", CountryCode.GUM);
                    break;
                case GUY:
                    countryCodeMap.put("Guyana", CountryCode.GUY);
                    break;
                case HKG:
                    countryCodeMap.put("Hong Kong", CountryCode.HKG);
                    break;
                case HMD:
                    countryCodeMap.put("Heard Island And Mcdonald Islands", CountryCode.HMD);
                    break;
                case HND:
                    countryCodeMap.put("Honduras", CountryCode.HND);
                    break;
                case HRV:
                    countryCodeMap.put("Croatia", CountryCode.HRV);
                    break;
                case HTI:
                    countryCodeMap.put("Haiti", CountryCode.HTI);
                    break;
                case HUN:
                    countryCodeMap.put("Hungary", CountryCode.HUN);
                    break;
                case IDN:
                    countryCodeMap.put("Indonesia", CountryCode.IDN);
                    break;
                case IMN:
                    countryCodeMap.put("Isle Of Man", CountryCode.IMN);
                    break;
                case IND:
                    countryCodeMap.put("India", CountryCode.IND);
                    break;
                case IOT:
                    countryCodeMap.put("British Indian Ocean Territory", CountryCode.IOT);
                    break;
                case IRL:
                    countryCodeMap.put("Ireland", CountryCode.IRL);
                    break;
                case IRN:
                    countryCodeMap.put("Iran (islamic Republic Of)", CountryCode.IRN);
                    break;
                case IRQ:
                    countryCodeMap.put("Iraq", CountryCode.IRQ);
                    break;
                case ISL:
                    countryCodeMap.put("Iceland", CountryCode.ISL);
                    break;
                case ISR:
                    countryCodeMap.put("Israel", CountryCode.ISR);
                    break;
                case ITA:
                    countryCodeMap.put("Italy", CountryCode.ITA);
                    break;
                case JAM:
                    countryCodeMap.put("Jamaica", CountryCode.JAM);
                    break;
                case JEY:
                    countryCodeMap.put("Jersey", CountryCode.JEY);
                    break;
                case JOR:
                    countryCodeMap.put("Jordan", CountryCode.JOR);
                    break;
                case JPN:
                    countryCodeMap.put("Japan", CountryCode.JPN);
                    break;
                case KAZ:
                    countryCodeMap.put("Kazakhstan", CountryCode.KAZ);
                    break;
                case KEN:
                    countryCodeMap.put("Kenya", CountryCode.KEN);
                    break;
                case KGZ:
                    countryCodeMap.put("Kyrgyzstan", CountryCode.KGZ);
                    break;
                case KHM:
                    countryCodeMap.put("Cambodia", CountryCode.KHM);
                    break;
                case KIR:
                    countryCodeMap.put("Kiribati", CountryCode.KIR);
                    break;
                case KNA:
                    countryCodeMap.put("Saint Kitts And Nevis", CountryCode.KNA);
                    break;
                case KOR:
                    countryCodeMap.put("Korea, Republic Of", CountryCode.KOR);
                    break;
                case KWT:
                    countryCodeMap.put("Kuwait", CountryCode.KWT);
                    break;
                case LAO:
                    countryCodeMap.put("Lao People's Democratic Republic", CountryCode.LAO);
                    break;
                case LBN:
                    countryCodeMap.put("Lebanon", CountryCode.LBN);
                    break;
                case LBR:
                    countryCodeMap.put("Liberia", CountryCode.LBR);
                    break;
                case LBY:
                    countryCodeMap.put("Libya", CountryCode.LBY);
                    break;
                case LCA:
                    countryCodeMap.put("Saint Lucia", CountryCode.LCA);
                    break;
                case LIE:
                    countryCodeMap.put("Liechtenstein", CountryCode.LIE);
                    break;
                case LKA:
                    countryCodeMap.put("Sri Lanka", CountryCode.LKA);
                    break;
                case LSO:
                    countryCodeMap.put("Lesotho", CountryCode.LSO);
                    break;
                case LTU:
                    countryCodeMap.put("Lithuania", CountryCode.LTU);
                    break;
                case LUX:
                    countryCodeMap.put("Luxembourg", CountryCode.LUX);
                    break;
                case LVA:
                    countryCodeMap.put("Latvia", CountryCode.LVA);
                    break;
                case MAC:
                    countryCodeMap.put("Macao", CountryCode.MAC);
                    break;
                case MAF:
                    countryCodeMap.put("Saint Martin (french Part)", CountryCode.MAF);
                    break;
                case MAR:
                    countryCodeMap.put("Morocco", CountryCode.MAR);
                    break;
                case MCO:
                    countryCodeMap.put("Monaco", CountryCode.MCO);
                    break;
                case MDA:
                    countryCodeMap.put("Moldova, Republic Of", CountryCode.MDA);
                    break;
                case MDG:
                    countryCodeMap.put("Madagascar", CountryCode.MDG);
                    break;
                case MDV:
                    countryCodeMap.put("Maldives", CountryCode.MDV);
                    break;
                case MEX:
                    countryCodeMap.put("Mexico", CountryCode.MEX);
                    break;
                case MHL:
                    countryCodeMap.put("Marshall Islands", CountryCode.MHL);
                    break;
                case MKD:
                    countryCodeMap.put("North Macedonia", CountryCode.MKD);
                    break;
                case MLI:
                    countryCodeMap.put("Mali", CountryCode.MLI);
                    break;
                case MLT:
                    countryCodeMap.put("Malta", CountryCode.MLT);
                    break;
                case MMR:
                    countryCodeMap.put("Myanmar", CountryCode.MMR);
                    break;
                case MNE:
                    countryCodeMap.put("Montenegro", CountryCode.MNE);
                    break;
                case MNG:
                    countryCodeMap.put("Mongolia", CountryCode.MNG);
                    break;
                case MNP:
                    countryCodeMap.put("Northern Mariana Islands", CountryCode.MNP);
                    break;
                case MOZ:
                    countryCodeMap.put("Mozambique", CountryCode.MOZ);
                    break;
                case MRT:
                    countryCodeMap.put("Mauritania", CountryCode.MRT);
                    break;
                case MSR:
                    countryCodeMap.put("Montserrat", CountryCode.MSR);
                    break;
                case MTQ:
                    countryCodeMap.put("Martinique", CountryCode.MTQ);
                    break;
                case MUS:
                    countryCodeMap.put("Mauritius", CountryCode.MUS);
                    break;
                case MWI:
                    countryCodeMap.put("Malawi", CountryCode.MWI);
                    break;
                case MYS:
                    countryCodeMap.put("Malaysia", CountryCode.MYS);
                    break;
                case MYT:
                    countryCodeMap.put("Mayotte", CountryCode.MYT);
                    break;
                case NAM:
                    countryCodeMap.put("Namibia", CountryCode.NAM);
                    break;
                case NCL:
                    countryCodeMap.put("New Caledonia", CountryCode.NCL);
                    break;
                case NER:
                    countryCodeMap.put("Niger", CountryCode.NER);
                    break;
                case NFK:
                    countryCodeMap.put("Norfolk Island", CountryCode.NFK);
                    break;
                case NGA:
                    countryCodeMap.put("Nigeria", CountryCode.NGA);
                    break;
                case NIC:
                    countryCodeMap.put("Nicaragua", CountryCode.NIC);
                    break;
                case NIU:
                    countryCodeMap.put("Niue", CountryCode.NIU);
                    break;
                case NLD:
                    countryCodeMap.put("Netherlands", CountryCode.NLD);
                    break;
                case NOR:
                    countryCodeMap.put("Norway", CountryCode.NOR);
                    break;
                case NPL:
                    countryCodeMap.put("Nepal", CountryCode.NPL);
                    break;
                case NRU:
                    countryCodeMap.put("Nauru", CountryCode.NRU);
                    break;
                case NZL:
                    countryCodeMap.put("New Zealand", CountryCode.NZL);
                    break;
                case OMN:
                    countryCodeMap.put("Oman", CountryCode.OMN);
                    break;
                case PAK:
                    countryCodeMap.put("Pakistan", CountryCode.PAK);
                    break;
                case PAN:
                    countryCodeMap.put("Panama", CountryCode.PAN);
                    break;
                case PCN:
                    countryCodeMap.put("Pitcairn", CountryCode.PCN);
                    break;
                case PER:
                    countryCodeMap.put("Peru", CountryCode.PER);
                    break;
                case PHL:
                    countryCodeMap.put("Philippines", CountryCode.PHL);
                    break;
                case PLW:
                    countryCodeMap.put("Palau", CountryCode.PLW);
                    break;
                case PNG:
                    countryCodeMap.put("Papua New Guinea", CountryCode.PNG);
                    break;
                case POL:
                    countryCodeMap.put("Poland", CountryCode.POL);
                    break;
                case PRI:
                    countryCodeMap.put("Puerto Rico", CountryCode.PRI);
                    break;
                case PRK:
                    countryCodeMap.put("Korea (democratic People's Republic Of)", CountryCode.PRK);
                    break;
                case PRT:
                    countryCodeMap.put("Portugal", CountryCode.PRT);
                    break;
                case PRY:
                    countryCodeMap.put("Paraguay", CountryCode.PRY);
                    break;
                case PSE:
                    countryCodeMap.put("Palestine, State Of", CountryCode.PSE);
                    break;
                case PYF:
                    countryCodeMap.put("French Polynesia", CountryCode.PYF);
                    break;
                case QAT:
                    countryCodeMap.put("Qatar", CountryCode.QAT);
                    break;
                case REU:
                    countryCodeMap.put("Reunion", CountryCode.REU);
                    break;
                case ROU:
                    countryCodeMap.put("Romania", CountryCode.ROU);
                    break;
                case RUS:
                    countryCodeMap.put("Russian Federation", CountryCode.RUS);
                    break;
                case RWA:
                    countryCodeMap.put("Rwanda", CountryCode.RWA);
                    break;
                case SAU:
                    countryCodeMap.put("Saudi Arabia", CountryCode.SAU);
                    break;
                case SDN:
                    countryCodeMap.put("Sudan", CountryCode.SDN);
                    break;
                case SEN:
                    countryCodeMap.put("Senegal", CountryCode.SEN);
                    break;
                case SGP:
                    countryCodeMap.put("Singapore", CountryCode.SGP);
                    break;
                case SGS:
                    countryCodeMap.put("South Georgia And The South Sandwich Islands", CountryCode.SGS);
                    break;
                case SHN:
                    countryCodeMap.put("Saint Helena, Ascension And Tristan Da Cunha", CountryCode.SHN);
                    break;
                case SJM:
                    countryCodeMap.put("Svalbard And Jan Mayen", CountryCode.SJM);
                    break;
                case SLB:
                    countryCodeMap.put("Solomon Islands", CountryCode.SLB);
                    break;
                case SLE:
                    countryCodeMap.put("Sierra Leone", CountryCode.SLE);
                    break;
                case SLV:
                    countryCodeMap.put("El Salvador", CountryCode.SLV);
                    break;
                case SMR:
                    countryCodeMap.put("San Marino", CountryCode.SMR);
                    break;
                case SOM:
                    countryCodeMap.put("Somalia", CountryCode.SOM);
                    break;
                case SPM:
                    countryCodeMap.put("Saint Pierre And Miquelon", CountryCode.SPM);
                    break;
                case SRB:
                    countryCodeMap.put("Serbia", CountryCode.SRB);
                    break;
                case SSD:
                    countryCodeMap.put("South Sudan", CountryCode.SSD);
                    break;
                case STP:
                    countryCodeMap.put("Sao Tome And Principe", CountryCode.STP);
                    break;
                case SUR:
                    countryCodeMap.put("Suriname", CountryCode.SUR);
                    break;
                case SVK:
                    countryCodeMap.put("Slovakia", CountryCode.SVK);
                    break;
                case SVN:
                    countryCodeMap.put("Slovenia", CountryCode.SVN);
                    break;
                case SWE:
                    countryCodeMap.put("Sweden", CountryCode.SWE);
                    break;
                case SWZ:
                    countryCodeMap.put("Eswatini", CountryCode.SWZ);
                    break;
                case SXM:
                    countryCodeMap.put("Sint Maarten (dutch Part)", CountryCode.SXM);
                    break;
                case SYC:
                    countryCodeMap.put("Seychelles", CountryCode.SYC);
                    break;
                case SYR:
                    countryCodeMap.put("Syrian Arab Republic", CountryCode.SYR);
                    break;
                case TCA:
                    countryCodeMap.put("Turks And Caicos Islands", CountryCode.TCA);
                    break;
                case TCD:
                    countryCodeMap.put("Chad", CountryCode.TCD);
                    break;
                case TGO:
                    countryCodeMap.put("Togo", CountryCode.TGO);
                    break;
                case THA:
                    countryCodeMap.put("Thailand", CountryCode.THA);
                    break;
                case TJK:
                    countryCodeMap.put("Tajikistan", CountryCode.TJK);
                    break;
                case TKL:
                    countryCodeMap.put("Tokelau", CountryCode.TKL);
                    break;
                case TKM:
                    countryCodeMap.put("Turkmenistan", CountryCode.TKM);
                    break;
                case TLS:
                    countryCodeMap.put("Timor-leste", CountryCode.TLS);
                    break;
                case TON:
                    countryCodeMap.put("Tonga", CountryCode.TON);
                    break;
                case TTO:
                    countryCodeMap.put("Trinidad And Tobago", CountryCode.TTO);
                    break;
                case TUN:
                    countryCodeMap.put("Tunisia", CountryCode.TUN);
                    break;
                case TUR:
                    countryCodeMap.put("Turkey", CountryCode.TUR);
                    break;
                case TUV:
                    countryCodeMap.put("Tuvalu", CountryCode.TUV);
                    break;
                case TWN:
                    countryCodeMap.put("Taiwan, Province Of China", CountryCode.TWN);
                    break;
                case TZA:
                    countryCodeMap.put("Tanzania, United Republic Of", CountryCode.TZA);
                    break;
                case UGA:
                    countryCodeMap.put("Uganda", CountryCode.UGA);
                    break;
                case UKR:
                    countryCodeMap.put("Ukraine", CountryCode.UKR);
                    break;
                case UMI:
                    countryCodeMap.put("United States Minor Outlying Islands", CountryCode.UMI);
                    break;
                case URY:
                    countryCodeMap.put("Uruguay", CountryCode.URY);
                    break;
                case USA:
                    countryCodeMap.put("United States Of America", CountryCode.USA);
                    break;
                case UZB:
                    countryCodeMap.put("Uzbekistan", CountryCode.UZB);
                    break;
                case VAT:
                    countryCodeMap.put("Holy See", CountryCode.VAT);
                    break;
                case VCT:
                    countryCodeMap.put("Saint Vincent And The Grenadines", CountryCode.VCT);
                    break;
                case VEN:
                    countryCodeMap.put("Venezuela (bolivarian Republic Of)", CountryCode.VEN);
                    break;
                case VGB:
                    countryCodeMap.put("Virgin Islands (british)", CountryCode.VGB);
                    break;
                case VIR:
                    countryCodeMap.put("Virgin Islands (u.s.)", CountryCode.VIR);
                    break;
                case VNM:
                    countryCodeMap.put("Viet Nam", CountryCode.VNM);
                    break;
                case VUT:
                    countryCodeMap.put("Vanuatu", CountryCode.VUT);
                    break;
                case WLF:
                    countryCodeMap.put("Wallis And Futuna", CountryCode.WLF);
                    break;
                case WSM:
                    countryCodeMap.put("Samoa", CountryCode.WSM);
                    break;
                case YEM:
                    countryCodeMap.put("Yemen", CountryCode.YEM);
                    break;
                case ZAF:
                    countryCodeMap.put("South Africa", CountryCode.ZAF);
                    break;
                case ZMB:
                    countryCodeMap.put("Zambia", CountryCode.ZMB);
                    break;
                case ZWE:
                    countryCodeMap.put("Zimbabwe", CountryCode.ZWE);
                    break;
                default:
                    countryCodeMap.put(countryCode.name(), countryCode);
                    break;
            }
        }
        return countryCodeMap;
    }

    public static Map<String, HazardousMaterial> getHazardousGoodsMap() {
        Map<String, HazardousMaterial> hazardousGoodMap = new HashMap<>();
        for (HazardousMaterial hazardousGood : HazardousMaterial.values()) {
            switch (hazardousGood) {
                case COMBUSTIBLE:
                    hazardousGoodMap.put("Combustible", HazardousMaterial.COMBUSTIBLE);
                    break;
                case CORROSIVE:
                    hazardousGoodMap.put("Corrosive", HazardousMaterial.CORROSIVE);
                    break;
                case EXPLOSIVE:
                    hazardousGoodMap.put("Explosive", HazardousMaterial.EXPLOSIVE);
                    break;
                case GAS:
                    hazardousGoodMap.put("Gas", HazardousMaterial.GAS);
                    break;
                case HARMFUL_TO_WATER:
                    hazardousGoodMap.put("Harmful To Water", HazardousMaterial.HARMFUL_TO_WATER);
                    break;
                case ORGANIC:
                    hazardousGoodMap.put("Organize", HazardousMaterial.ORGANIC);
                    break;
                case OTHER:
                    hazardousGoodMap.put("Other", HazardousMaterial.OTHER);
                    break;
                case POISON:
                    hazardousGoodMap.put("Poison", HazardousMaterial.POISON);
                    break;
                case POISONOUS_INHALATION:
                    hazardousGoodMap.put("Poisonous Inhalation", HazardousMaterial.POISONOUS_INHALATION);
                    break;
                case RADIOACTIVE:
                    hazardousGoodMap.put("Radioactive", HazardousMaterial.RADIOACTIVE);
                    break;
                default:
                    break;
            }
        }
        return hazardousGoodMap;
    }

    public static Map<String, TruckType> getTruckTypeMap() {
        Map<String, TruckType> truckTypeMap = new HashMap<>();
        for (TruckType truckType : TruckType.values()) {
            switch (truckType) {
                case STRAIGHT -> truckTypeMap.put("Straight", TruckType.STRAIGHT);
                case TRACTOR -> truckTypeMap.put("Tractor", TruckType.TRACTOR);
                default -> {
                }
            }
        }
        return truckTypeMap;
    }

    public static List<TrafficLineColor> calculateSpans(Context context, List<Span> spanList) {
        List<TrafficLineColor> trafficLineColors = new ArrayList<>();
        List<GeoCoordinates> spanPolyline = new ArrayList<>();
        Color tempLineColor = Color.valueOf(context.getResources().getColor(R.color.selectedRouteColor));
        for (int i = 0; i < spanList.size(); i++) {
//            Span span = spanList.get(i);
//        int i = 0, spanListSize = spanList.size();
//        while (i < spanListSize) {
            double jamFactor = spanList.get(i).getTrafficSpeed().jamFactor;
//                        double jamFactor = span.getTrafficSpeed().jamFactor;
//                        Log.e("Jam Factor: ", jamFactor + "");
//                    0 <= jamFactor < 4: No or light traffic.
//                    4 <= jamFactor < 8: Moderate or slow traffic.
//                    8 <= jamFactor < 10: Severe traffic.
//                    jamFactor = 10: No traffic, ie. the road is blocked.
            Color lineColor;
//            if (jamFactor >= 0 && jamFactor < 4) {
//                lineColor = Color.valueOf(context.getResources().getColor(R.color.selectedRouteColor));
//            } else if (jamFactor >= 4 && jamFactor < 8) {
//                lineColor = Color.valueOf(context.getResources().getColor(R.color.yellow));
//            } else if (jamFactor >= 8 && jamFactor < 10) {
//                lineColor = Color.valueOf(context.getResources().getColor(android.R.color.holo_red_light));
//            } else if (jamFactor == 10) {
//                lineColor = Color.valueOf(context.getResources().getColor(android.R.color.black));
//            } else {
//                lineColor = Color.valueOf(context.getResources().getColor(R.color.selectedRouteColor));
//            }
            if (jamFactor >= 8) {
                lineColor = Color.valueOf(context.getResources().getColor(android.R.color.holo_red_light));
            } else if (jamFactor >= 4) {
                lineColor = Color.valueOf(context.getResources().getColor(R.color.yellow));
            } else {
                if (!spanPolyline.isEmpty()) {
                    trafficLineColors.add(new TrafficLineColor(new ArrayList<>(spanPolyline), tempLineColor));
                    spanPolyline.clear();
                }
                tempLineColor = Color.valueOf(context.getResources().getColor(R.color.selectedRouteColor));
                i++;
                continue;
            }
            if (tempLineColor.toArgb() == lineColor.toArgb()) {
                tempLineColor = lineColor;
                spanPolyline.addAll(spanList.get(i).getGeometry().vertices);
            } else {
                if (!spanPolyline.isEmpty()) {
                    trafficLineColors.add(new TrafficLineColor(new ArrayList<>(spanPolyline), tempLineColor));
                    spanPolyline.clear();
                }
                spanPolyline.addAll(spanList.get(i).getGeometry().vertices);
                tempLineColor = lineColor;
            }
//            i++;
        }
        return trafficLineColors;
    }

//    public static List<TrafficLineColor> calculateSpans(Context context,  int initialLimit, int countLimit, Section section) {
//        List<TrafficLineColor> trafficLineColors = new ArrayList<>();
//        List<GeoCoordinates> spanPolyline = new ArrayList<>();
//        Color tempLineColor = Color.valueOf(context.getResources().getColor(R.color.selectedRouteColor));
//        for (int i = initialLimit; i < countLimit; i++) {
//            Span span = section.getSpans().get(i);
//            TrafficSpeed firstTrafficSpeed = span.getTrafficSpeed();
//            double jamFactor = firstTrafficSpeed.jamFactor;
////                        double jamFactor = span.getTrafficSpeed().jamFactor;
////                        Log.e("Jam Factor: ", jamFactor + "");
////                    0 <= jamFactor < 4: No or light traffic.
////                    4 <= jamFactor < 8: Moderate or slow traffic.
////                    8 <= jamFactor < 10: Severe traffic.
////                    jamFactor = 10: No traffic, ie. the road is blocked.
//            Color lineColor;
////            if (jamFactor >= 0 && jamFactor < 4) {
////                lineColor = Color.valueOf(context.getResources().getColor(R.color.selectedRouteColor));
////            } else if (jamFactor >= 4 && jamFactor < 8) {
////                lineColor = Color.valueOf(context.getResources().getColor(R.color.yellow));
////            } else if (jamFactor >= 8 && jamFactor < 10) {
////                lineColor = Color.valueOf(context.getResources().getColor(android.R.color.holo_red_light));
////            } else if (jamFactor == 10) {
////                lineColor = Color.valueOf(context.getResources().getColor(android.R.color.black));
////            } else {
////                lineColor = Color.valueOf(context.getResources().getColor(R.color.selectedRouteColor));
////            }
//            if (jamFactor >= 8) {
//                lineColor = Color.valueOf(context.getResources().getColor(android.R.color.holo_red_light));
//            } else if(jamFactor >= 4){
//                lineColor = Color.valueOf(context.getResources().getColor(R.color.yellow));
//            }else{
//                if (!spanPolyline.isEmpty()) {
//                    trafficLineColors.add(new TrafficLineColor(new ArrayList<>(spanPolyline), tempLineColor));
//                    spanPolyline.clear();
//                }
//                tempLineColor = Color.valueOf(context.getResources().getColor(R.color.selectedRouteColor));
//                continue;
//            }
//            if (tempLineColor.toArgb() == lineColor.toArgb()) {
//                tempLineColor = lineColor;
//                spanPolyline.addAll(span.getPolyline());
//            } else {
//                if (!spanPolyline.isEmpty()) {
//                    trafficLineColors.add(new TrafficLineColor(new ArrayList<>(spanPolyline), tempLineColor));
//                    spanPolyline.clear();
//                }
//                spanPolyline.addAll(span.getPolyline());
//                tempLineColor = lineColor;
//            }
//        }
//        return trafficLineColors;
//    }


    // Define a traffic color scheme based on the route's jam factor.
    // 0 <= jamFactor < 4: No or light traffic.
    // 4 <= jamFactor < 8: Moderate or slow traffic.
    // 8 <= jamFactor < 10: Severe traffic.
    // jamFactor = 10: No traffic, ie. the road is blocked.
    // Returns null in case of no or light traffic.
    @Nullable
    public static Color getTrafficColor(Double jamFactor) {
        if (jamFactor == null || jamFactor < 4) {
            return null;
        } else if (jamFactor >= 4 && jamFactor < 8) {
            return Color.valueOf(1, 1, 0, 0.63f); // Yellow
        } else if (jamFactor >= 8 && jamFactor < 10) {
            return Color.valueOf(1, 0, 0, 0.63f); // Red
        }
        return Color.valueOf(0, 0, 0, 0.63f); // Black
    }
}
